/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2023 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
import java.io.FileInputStream
import java.util.Properties

plugins {
    id("mingraviditet.android.application")
    id("mingraviditet.android.application.compose")
    id("mingraviditet.android.application.jacoco")
    id("mingraviditet.android.application.flavors")
    id("mingraviditet.android.hilt")
    id("jacoco")
    id("kotlin-parcelize")
    id("androidx.navigation.safeargs.kotlin")
    // To enable usage of JUnit5 for Android projects
    alias(libs.plugins.android.junit5)
}

android {
    defaultConfig {
        applicationId = "dk.nsp.mingraviditet"
        versionCode = rootProject.extra["versionCode"] as Int
        versionName = rootProject.extra["versionName"] as String

        multiDexEnabled = true

        testInstrumentationRunner = "androidx.test.runner.AndroidJUnitRunner"
    }

    signingConfigs {
        create("release") {
            val localPropertiesFile = rootProject.file("local.properties")
            val localProperties = Properties()

            try {
                localProperties.load(FileInputStream(localPropertiesFile))
                storeFile = rootProject.file("./signing/MinGraviditetKeystore20211203.jks")
            } catch (localFileError: Throwable) {
                // When building via Azure Pipelines, the local.properties and Keystore file does not exist
                println("@Build.gradle (app): Attempting to default to System Environment variables. ErrorMessage: ${localFileError.localizedMessage}")

                try {
                    // This is to allow for running build/tests without the signing credentials present
                    // E.g. when Azure runs unit tests it needs a
                    val keyStoreFilePath = System.getenv("KeystoreFileLocation")
                    val propertiesFilePath = System.getenv("PropertiesFileLocation")
                    println("Using KeystoreFile at: $keyStoreFilePath and properties file at : $propertiesFilePath")

                    localProperties.load(FileInputStream(propertiesFilePath))
                    storeFile = File(keyStoreFilePath)
                } catch (systemEnvError: Throwable) {

                    println("@Build.gradle (app): Failed to setup files from System environment. ErrorMessage: ${systemEnvError.localizedMessage}")
                }
            }

            storePassword = localProperties.getProperty("storePassword")
            keyAlias = localProperties.getProperty("private_alias")
            keyPassword = localProperties.getProperty("private_password")
        }
    }

    buildTypes {
        release {
            isMinifyEnabled = false
            proguardFiles(getDefaultProguardFile("proguard-android-optimize.txt"), "proguard-rules.pro")
            signingConfig = signingConfigs.getByName("release")
        }
        debug {
            isMinifyEnabled = false
        }
    }

    namespace = "dk.nsp.mingraviditet"
}

dependencies {
    // Modules
    implementation(project(":repo"))
    implementation(project(":shared"))
    implementation(project(":ui-compose"))
    implementation(project(":model"))

    implementation(libs.androidx.compose.runtime)
    implementation(libs.androidx.activity.compose)
    implementation(libs.androidx.navigation.compose)
    implementation(libs.androidx.hilt.navigation.compose)

    // Android
    implementation(libs.androidx.splashscreen)
    implementation(libs.androidx.core)
    implementation(libs.androidx.appCompat)
    implementation(libs.androidx.lifecycle.viewModel)
    implementation(libs.androidx.lifecycle.runtime)
    implementation(libs.androidx.lifecycle.runtimeCompose)
    implementation(libs.androidx.biometric)
    implementation(libs.androidx.webKit)
    implementation(libs.androidx.browser)

    // Material Design
    implementation(libs.google.material)
    implementation(libs.androidx.compose.material.iconsExtended)

    // Testing
    androidTestImplementation(libs.androidx.test.extJunit)
    androidTestImplementation(libs.espresso.core)
    testImplementation(libs.roboelectric)

    //Glide
    implementation(libs.glide.glide)
    implementation(libs.glide.compose)
    kapt(libs.glide.compiler)
    annotationProcessor(libs.glide.compiler)

    implementation(libs.hilt.android)

    implementation(libs.simplexNoise)
}

kapt {
    useBuildCache = true

    javacOptions {
        // Increase the max count of errors from annotation processors.
        // Default is 100.
        option("-Xmaxerrs", 500)
    }
}
