/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2023 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsp.mingraviditet.ui.common.compose

import androidx.compose.animation.animateContentSize
import androidx.compose.foundation.clickable
import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.PaddingValues
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.padding
import androidx.compose.runtime.Composable
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableIntStateOf
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.saveable.rememberSaveable
import androidx.compose.runtime.setValue
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.res.stringResource
import androidx.compose.ui.semantics.Role
import androidx.compose.ui.text.style.TextOverflow
import androidx.compose.ui.unit.dp
import dk.nsp.mingraviditet.R
import dk.nsp.mingraviditet.extensions.toMediumFormat
import dk.nsp.mingraviditet.gravidcompose.component.GravidCard
import dk.nsp.mingraviditet.gravidcompose.component.GravidDivider
import dk.nsp.mingraviditet.gravidcompose.component.GravidText
import dk.nsp.mingraviditet.gravidcompose.theme.GravidTheme
import dk.nsp.mingraviditet.gravidcompose.utils.GravidPreview
import java.time.LocalDateTime

@Composable
fun ExpandableCard(
    title: CharSequence,
    description: CharSequence,
    modifier: Modifier = Modifier,
    maxLinesWhenCollapsed: Int = 2,
    lastUpdated: LocalDateTime? = null,
) {
    var expanded by rememberSaveable {
        mutableStateOf(false)
    }

    val maxLines by remember(expanded) {
        mutableIntStateOf(if (expanded) Int.MAX_VALUE else maxLinesWhenCollapsed)
    }

    GravidCard(
        modifier = modifier,
        innerPaddingValues = PaddingValues(top = GravidTheme.dimens.largeSpacing),
    ) {
        Column(
            modifier = Modifier
                .animateContentSize()
                .fillMaxWidth(),
        ) {
            lastUpdated?.toMediumFormat()?.let {
                GravidText(
                    modifier = Modifier
                        .padding(horizontal = GravidTheme.dimens.largeSpacing)
                        .padding(bottom = GravidTheme.dimens.defaultSpacing),
                    text = stringResource(id = R.string.article_last_updated_on, it),
                    style = GravidTheme.typography.Info,
                )
            }
            GravidText(
                modifier = Modifier
                    .padding(horizontal = GravidTheme.dimens.largeSpacing),
                text = title,
                style = GravidTheme.typography.Header4,
            )

            GravidText(
                modifier = Modifier
                    .padding(horizontal = GravidTheme.dimens.largeSpacing)
                    .padding(top = GravidTheme.dimens.trumpSpacing),
                text = description,
                overflow = TextOverflow.Ellipsis,
                maxLines = maxLines,
            )

            GravidDivider(modifier = Modifier.padding(top = GravidTheme.dimens.largeSpacing))

            Box(
                modifier = Modifier
                    .fillMaxWidth()
                    .clickable(role = Role.Button) { expanded = !expanded },
                contentAlignment = Alignment.Center,
            ) {
                GravidText(
                    modifier = Modifier.padding(vertical = 16.dp),
                    text = if (expanded) stringResource(R.string.common_cta_collapse) else stringResource(id = R.string.common_cta_expand),
                    style = GravidTheme.typography.Header5,
                    color = GravidTheme.colorScheme.tertiary,
                )
            }
        }
    }
}

@GravidPreview
@Composable
private fun ExpandableCardPreview() {
    GravidTheme {
        Column(
            modifier = Modifier
                .padding(horizontal = GravidTheme.dimens.largeSpacing)
                .padding(vertical = GravidTheme.dimens.defaultSpacing),
            verticalArrangement = Arrangement.spacedBy(GravidTheme.dimens.defaultSpacing),
        ) {
            ExpandableCard(
                lastUpdated = LocalDateTime.now(),
                title = "Test card",
                description = "Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris nisi ut aliquip ex ea commodo consequat. Duis aute irure dolor in reprehenderit in voluptate velit esse cillum dolore eu fugiat nulla pariatur. Excepteur sint occaecat cupidatat non proident, sunt in culpa qui officia deserunt mollit anim id est laborum.",
            )

            ExpandableCard(
                title = "Test card",
                description = "Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris nisi ut aliquip ex ea commodo consequat. Duis aute irure dolor in reprehenderit in voluptate velit esse cillum dolore eu fugiat nulla pariatur. Excepteur sint occaecat cupidatat non proident, sunt in culpa qui officia deserunt mollit anim id est laborum.",
            )
        }
    }
}
