/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2023 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsp.mingraviditet.ui.common.compose

import android.annotation.SuppressLint
import android.view.View
import android.webkit.WebChromeClient
import android.webkit.WebChromeClient.CustomViewCallback
import androidx.compose.animation.animateContentSize
import androidx.compose.animation.core.animateDpAsState
import androidx.compose.foundation.clickable
import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.PaddingValues
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.height
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.lazy.LazyColumn
import androidx.compose.runtime.Composable
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.saveable.rememberSaveable
import androidx.compose.runtime.setValue
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.draw.drawWithContent
import androidx.compose.ui.graphics.Brush
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.res.stringResource
import androidx.compose.ui.semantics.Role
import androidx.compose.ui.tooling.preview.Preview
import androidx.compose.ui.unit.Dp
import androidx.compose.ui.unit.dp
import dk.nsp.mingraviditet.R
import dk.nsp.mingraviditet.gravidcompose.component.GravidCard
import dk.nsp.mingraviditet.gravidcompose.component.GravidDivider
import dk.nsp.mingraviditet.gravidcompose.component.GravidText
import dk.nsp.mingraviditet.gravidcompose.theme.GravidTheme
import dk.nsp.mingraviditet.ui.main.common.compose.LocalWebView
import dk.nsp.mingraviditet.util.android.WebViewHelper

@SuppressLint("SetJavaScriptEnabled")
@Composable
fun FoldableCard(
    title: String,
    content: String,
    modifier: Modifier = Modifier,
    isExpanded: Boolean = false,
    onInternalLinkClicked: (String) -> Unit = {},
    onShowFullScreen: (view: View, callback: CustomViewCallback) -> Unit,
    onCloseFullScreen: () -> Unit,
) {
    val colors = GravidTheme.colorScheme
    var expanded by rememberSaveable(isExpanded) {
        mutableStateOf(isExpanded)
    }

    val webViewHeight by remember(expanded) {
        mutableStateOf(if (expanded) Dp.Infinity else 75.dp)
    }

    val heightAnimate by animateDpAsState(targetValue = webViewHeight, label = "Expand animation")

    GravidCard(
        modifier = Modifier
            .fillMaxWidth()
            .then(modifier),
        innerPaddingValues = PaddingValues(0.dp),
    ) {
        Column(modifier = Modifier.animateContentSize()) {
            GravidText(
                modifier = Modifier
                    .fillMaxWidth()
                    .padding(horizontal = 16.dp)
                    .padding(top = 16.dp),
                text = title,
                style = GravidTheme.typography.Header4,
            )

            LocalWebView(
                content = content,
                webViewType = WebViewHelper.Type.Article,
                webChromeClient = WebChromeVideoClientImpl(
                    onFullScreen = onShowFullScreen,
                    onCloseFullscreen = onCloseFullScreen,
                ),
                onInternalLinkClicked = onInternalLinkClicked,
                modifier = Modifier
                    .padding(horizontal = 16.dp)
                    .padding(top = 8.dp, bottom = 16.dp)
                    .height(heightAnimate)
                    .drawWithContent {
                        drawContent()
                        if (!expanded) {
                            drawRect(
                                Brush.verticalGradient(
                                    0f to Color.Transparent,
                                    0.5f to Color.Transparent,
                                    1f to colors.surface,
                                ),
                            )
                        }
                    },
            )

            GravidDivider()

            Box(
                modifier = Modifier
                    .fillMaxWidth()
                    .clickable(role = Role.Button) { expanded = !expanded },
                contentAlignment = Alignment.Center,
            ) {
                GravidText(
                    modifier = Modifier.padding(vertical = 16.dp),
                    text = if (expanded) stringResource(R.string.common_cta_collapse) else stringResource(id = R.string.common_cta_expand),
                    style = GravidTheme.typography.Header5,
                    color = GravidTheme.colorScheme.tertiary,
                )
            }
        }
    }
}

private class WebChromeVideoClientImpl(
    val onFullScreen: (view: View, callback: CustomViewCallback) -> Unit,
    val onCloseFullscreen: () -> Unit,
) : WebChromeClient() {
    override fun onShowCustomView(view: View, callback: CustomViewCallback) {
        onFullScreen(view, callback)
    }

    override fun onHideCustomView() {
        onCloseFullscreen()
    }
}

@Preview
@Composable
private fun FoldableCardPreview() {
    GravidTheme {
        LazyColumn(
            verticalArrangement = Arrangement.spacedBy(8.dp),
            contentPadding = PaddingValues(16.dp),
        ) {
            items(10) { i ->
                FoldableCard(
                    title = "Foldable card",
                    content = """## Emphasis

**This is bold text**

__This is bold text__

*This is italic text*

_This is italic text_

~~Strikethrough~~  
                    """.trimIndent(),
                    isExpanded = i == 0,
                    onShowFullScreen = { _, _ -> },
                    onCloseFullScreen = {},
                )
            }
        }
    }
}
