/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2023 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsp.mingraviditet.ui.common.compose

import androidx.compose.foundation.background
import androidx.compose.foundation.clickable
import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.heightIn
import androidx.compose.runtime.Composable
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.setValue
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.res.painterResource
import androidx.compose.ui.unit.dp
import dk.nsp.mingraviditet.R
import dk.nsp.mingraviditet.gravidcompose.component.GravidIcon
import dk.nsp.mingraviditet.gravidcompose.component.GravidSwitch
import dk.nsp.mingraviditet.gravidcompose.component.GravidText
import dk.nsp.mingraviditet.gravidcompose.extensions.conditional
import dk.nsp.mingraviditet.gravidcompose.theme.GravidTheme
import dk.nsp.mingraviditet.gravidcompose.utils.GravidPreview

@Composable
fun GravidListItem(
    title: CharSequence,
    modifier: Modifier = Modifier,
    subtitle: CharSequence? = null,
    onClick: (() -> Unit)? = null,
    trailingComponent: @Composable (() -> Unit)? = null,
) {
    Row(
        modifier =
        Modifier
            .fillMaxWidth()
            .heightIn(min = 55.dp)
            .conditional(onClick != null, modifier = { Modifier.clickable(onClick = onClick!!) })
            .then(modifier),
        verticalAlignment = Alignment.CenterVertically,
    ) {
        Column(modifier = Modifier.weight(1f), verticalArrangement = Arrangement.Center) {
            GravidText(text = title, style = GravidTheme.typography.Header4)
            subtitle?.let {
                GravidText(text = subtitle)
            }
        }

        trailingComponent?.invoke()
    }
}

@Composable
fun ToggleListItem(
    title: CharSequence,
    subtitle: CharSequence? = null,
    checked: Boolean,
    onCheckedChanged: (Boolean) -> Unit,
) {
    GravidListItem(
        title = title,
        subtitle = subtitle,
        trailingComponent = {
            GravidSwitch(
                checked = checked,
                onCheckedChanged = onCheckedChanged,
            )
        },
    )
}

@Composable
fun ChevronListItem(
    title: CharSequence,
    onClick: () -> Unit,
    modifier: Modifier = Modifier,
    subtitle: CharSequence? = null,
) {
    GravidListItem(
        modifier = modifier,
        title = title,
        subtitle = subtitle,
        onClick = onClick,
        trailingComponent = {
            GravidIcon(painter = painterResource(id = R.drawable.ic_chevron_right))
        },
    )
}

@GravidPreview
@Composable
private fun ListItemPreview() {
    var previewChecked by remember { mutableStateOf(false) }

    GravidTheme {
        Column(modifier = Modifier.background(GravidTheme.colorScheme.surface), verticalArrangement = Arrangement.spacedBy(8.dp)) {
            ChevronListItem(title = "Test item", onClick = { /*TODO*/ })
            ChevronListItem(title = "Test item", subtitle = "Sub goes here", onClick = { /*TODO*/ })
            ToggleListItem(title = "Title", subtitle = "Subtitle here", checked = previewChecked, onCheckedChanged = { previewChecked = !previewChecked })
            ToggleListItem(title = "Title", checked = !previewChecked, onCheckedChanged = { previewChecked = !previewChecked })
            ToggleListItem(title = "Title", checked = previewChecked, onCheckedChanged = { previewChecked = !previewChecked })
        }
    }
}
