/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2023 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsp.mingraviditet.ui.common.compose

import androidx.compose.foundation.Image
import androidx.compose.foundation.background
import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.padding
import androidx.compose.runtime.Composable
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.res.painterResource
import androidx.compose.ui.text.style.TextAlign
import androidx.compose.ui.unit.dp
import com.bumptech.glide.integration.compose.ExperimentalGlideComposeApi
import com.bumptech.glide.integration.compose.GlideImage
import dk.nsp.mingraviditet.R
import dk.nsp.mingraviditet.gravidcompose.component.GravidText
import dk.nsp.mingraviditet.gravidcompose.theme.GravidTheme
import dk.nsp.mingraviditet.gravidcompose.utils.GravidIconType
import dk.nsp.mingraviditet.gravidcompose.utils.GravidPreview
import dk.nsp.mingraviditet.ui.main.common.compose.InnerCircularCard
import dk.nsp.mingraviditet.ui.main.overview.compose.widget.OuterFetusWidgetCircle

@OptIn(ExperimentalGlideComposeApi::class)
@Composable
fun HeaderWithIcon(
    title: CharSequence,
    description: CharSequence,
    modifier: Modifier = Modifier,
    icon: GravidIconType,
) {
    Column(
        modifier = Modifier
            .fillMaxWidth()
            .then(modifier),
        horizontalAlignment = Alignment.CenterHorizontally,
        verticalArrangement = Arrangement.spacedBy(GravidTheme.dimens.trumpSpacing),
    ) {
        val outerCircleSize = 98.dp
        val spacingBetweenOuterAndInnerCircle = 13.dp
        val innerCircleSize = outerCircleSize - spacingBetweenOuterAndInnerCircle * 2

        OuterFetusWidgetCircle(
            modifier = Modifier
                .align(Alignment.CenterHorizontally),
            size = outerCircleSize,
        ) {
            Box(
                modifier = Modifier
                    .fillMaxSize()
                    .background(GravidTheme.colorScheme.tertiaryContainer),
                contentAlignment = Alignment.Center,
            ) {
                InnerCircularCard(
                    size = innerCircleSize,
                    containerColor = GravidTheme.colorScheme.secondaryContainer,
                ) {
                    Box(Modifier.fillMaxSize(), contentAlignment = Alignment.Center) {
                        when (icon) {
                            is GravidIconType.Drawable -> {
                                Image(
                                    painter = painterResource(id = icon.resourceId),
                                    contentDescription = null,
                                    modifier = Modifier
                                        .fillMaxSize()
                                        .padding(GravidTheme.dimens.largeSpacing),
                                )
                            }

                            is GravidIconType.Url -> {
                                GlideImage(
                                    model = icon.url,
                                    contentDescription = null,
                                    modifier = Modifier
                                        .fillMaxSize()
                                        .padding(GravidTheme.dimens.largeSpacing),
                                )
                            }
                        }
                    }
                }
            }
        }

        GravidText(
            modifier = Modifier.fillMaxWidth(),
            textAlign = TextAlign.Center,
            text = title,
            style = GravidTheme.typography.Header1,
        )
        GravidText(
            modifier = Modifier.fillMaxWidth(),
            textAlign = TextAlign.Center,
            text = description,
        )
    }
}

@GravidPreview
@Composable
private fun HeaderWithIconPreview() {
    GravidTheme {
        Column {
            HeaderWithIcon(icon = GravidIconType.Drawable(R.drawable.ic_settings), title = "Title here", description = "Some description")
        }
    }
}
