/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2023 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsp.mingraviditet.ui.main

import androidx.lifecycle.DefaultLifecycleObserver
import androidx.lifecycle.LifecycleOwner
import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import dagger.hilt.android.lifecycle.HiltViewModel
import dk.nsp.domain.repository.SettingsRepository
import kotlinx.coroutines.flow.MutableSharedFlow
import kotlinx.coroutines.flow.MutableStateFlow
import kotlinx.coroutines.flow.collectLatest
import kotlinx.coroutines.flow.map
import kotlinx.coroutines.launch
import java.time.LocalDateTime
import javax.inject.Inject

private const val TIMEOUT_SECONDS = 5 * 60L // 5 minutes

@HiltViewModel
class AppInBackgroundViewModel @Inject constructor(
    settingsRepository: SettingsRepository,
) : ViewModel(), DefaultLifecycleObserver {

    private var onStopTime = MutableStateFlow<LocalDateTime?>(null)

    val logoutEvent = MutableSharedFlow<Unit?>(replay = 0)

    private val useBiometricsPreference = settingsRepository.preferences.map {
        it.useBiometrics
    }

    override fun onResume(owner: LifecycleOwner) {
        super.onResume(owner)

        viewModelScope.launch {
            // Copy of onStopTime, since collectLatest is a terminal flow operator
            useBiometricsPreference.collectLatest { useBiometrics ->
                if (useBiometrics && onStopTime.value?.isBefore(LocalDateTime.now().minusSeconds(TIMEOUT_SECONDS)) == true) {
                    logoutEvent.emit(Unit)
                }

                // Reset timer
                onStopTime.value = null
            }
        }
    }

    override fun onStop(owner: LifecycleOwner) {
        super.onStop(owner)
        onStopTime.value = LocalDateTime.now()
    }
}
