/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2023 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsp.mingraviditet.ui.main

import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.WindowInsets
import androidx.compose.foundation.layout.consumeWindowInsets
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.padding
import androidx.compose.material3.Icon
import androidx.compose.material3.Scaffold
import androidx.compose.material3.Surface
import androidx.compose.material3.Text
import androidx.compose.runtime.Composable
import androidx.compose.ui.Modifier
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.res.painterResource
import androidx.compose.ui.res.stringResource
import androidx.compose.ui.unit.dp
import androidx.navigation.NavDestination
import androidx.navigation.NavDestination.Companion.hierarchy
import dk.nsp.mingraviditet.gravidcompose.component.MinGraviditetNavigationBar
import dk.nsp.mingraviditet.gravidcompose.component.MinGraviditetNavigationBarItem
import dk.nsp.mingraviditet.gravidcompose.extensions.bold
import dk.nsp.mingraviditet.gravidcompose.theme.GravidTheme
import dk.nsp.mingraviditet.ui.main.navigation.GravidNavHost
import dk.nsp.mingraviditet.ui.main.navigation.TopLevelDestination
import dk.nsp.mingraviditet.ui.main.overview.navigation.overviewNavigationRoute
import dk.nsp.mingraviditet.ui.useradmission.login.navigation.loginNavigationRoute
import dk.nsp.mingraviditet.ui.useradmission.onboarding.navigation.onboardingNavigationRoute

@Composable
fun GravidApp(
    isAppSecure: Boolean,
    showOnboarding: Boolean,
    appState: GravidAppState = rememberGravidAppState(),
) {
    val startDestination = if (isAppSecure) {
        loginNavigationRoute
    } else if (showOnboarding) {
        onboardingNavigationRoute
    } else {
        overviewNavigationRoute
    }

    Surface(
        color = GravidTheme.colorScheme.background,
        tonalElevation = 0.dp,
        modifier = Modifier.fillMaxSize(),
    ) {
        Scaffold(
            containerColor = Color.Transparent,
            contentColor = GravidTheme.colorScheme.onBackground,
            contentWindowInsets = WindowInsets(0, 0, 0, 0),
            bottomBar = {
                val currentDestination = appState.currentDestination

                val showBottomNavigation = when (currentDestination?.route) {
                    onboardingNavigationRoute, loginNavigationRoute -> false
                    else -> true
                }

                if (showBottomNavigation) {
                    MGBottomBar(
                        destinations = appState.topLevelDestinations,
                        onNavigateToDestination = { destination, selected ->
                            appState.navigateToTopLevelDestination(
                                topLevelDestination = destination,
                                restoreTabStack = !selected,
                            )
                        },
                        currentDestination = currentDestination,
                    )
                }
            },
        ) { padding ->
            Box(
                Modifier
                    .fillMaxSize()
                    .padding(padding)
                    .consumeWindowInsets(padding),
            ) {
                GravidNavHost(
                    appState = appState,
                    startDestination = startDestination,
                )
            }
        }
    }
}

private fun NavDestination?.isTopLevelDestinationInHierarchy(destination: TopLevelDestination) =
    this?.hierarchy?.any {
        it.route?.contains(destination.name, true) ?: false
    } ?: false

@Composable
fun MGBottomBar(
    destinations: List<TopLevelDestination>,
    onNavigateToDestination: (TopLevelDestination, selected: Boolean) -> Unit,
    currentDestination: NavDestination?,
    modifier: Modifier = Modifier,
) {
    MinGraviditetNavigationBar(modifier = modifier) {
        destinations.forEach { destination ->
            val selected = currentDestination.isTopLevelDestinationInHierarchy(destination)

            MinGraviditetNavigationBarItem(
                selected = selected,
                onClick = { onNavigateToDestination(destination, selected) },
                icon = {
                    Icon(
                        painterResource(id = destination.icon),
                        contentDescription = destination.name,
                    )
                },
                label = { Text(text = stringResource(id = destination.titleTextId), style = GravidTheme.typography.Info.bold) },
            )
        }
    }
}
