/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2023 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsp.mingraviditet.ui.main.careplanactivities

import androidx.compose.foundation.Image
import androidx.compose.foundation.isSystemInDarkTheme
import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.lazy.LazyColumn
import androidx.compose.foundation.lazy.items
import androidx.compose.material3.Scaffold
import androidx.compose.material3.Text
import androidx.compose.material3.TopAppBarDefaults
import androidx.compose.runtime.Composable
import androidx.compose.runtime.LaunchedEffect
import androidx.compose.runtime.collectAsState
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.saveable.rememberSaveable
import androidx.compose.runtime.setValue
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.draw.drawWithContent
import androidx.compose.ui.geometry.Offset
import androidx.compose.ui.input.nestedscroll.nestedScroll
import androidx.compose.ui.layout.ContentScale
import androidx.compose.ui.res.painterResource
import androidx.compose.ui.res.stringResource
import androidx.compose.ui.unit.dp
import androidx.hilt.navigation.compose.hiltViewModel
import androidx.lifecycle.compose.collectAsStateWithLifecycle
import dk.nsp.domain.careplan.SplitCareplanActivities
import dk.nsp.mingraviditet.R
import dk.nsp.mingraviditet.gravidcompose.component.GravidExpandableTopBar
import dk.nsp.mingraviditet.gravidcompose.extensions.isScrollingUp
import dk.nsp.mingraviditet.gravidcompose.extensions.toPx
import dk.nsp.mingraviditet.gravidcompose.theme.GravidTheme
import dk.nsp.mingraviditet.gravidcompose.utils.GravidPreview
import dk.nsp.mingraviditet.gravidcompose.utils.rememberForeverLazyListState
import dk.nsp.mingraviditet.gravidcompose.utils.rememberForeverTopAppBarState
import dk.nsp.mingraviditet.model.data.CareplanActivity
import dk.nsp.mingraviditet.ui.common.compose.InfoBox
import dk.nsp.mingraviditet.ui.main.careplanactivities.compose.ActivityRow
import dk.nsp.mingraviditet.ui.main.careplanactivities.compose.ActivityTimelineHeader

@Composable
fun CareplanActivitiesRoute(
    vm: CareplanActivitiesViewModel = hiltViewModel(),
    onClickActivity: (Int) -> Unit,
) {
    val careplanActivities by vm.careplanActivities.collectAsState()

    val hasGivenBirth by vm.hasGivenBirth.collectAsStateWithLifecycle()

    CareplanActivitiesScreen(
        splitActivities = careplanActivities,
        onClickActivity = onClickActivity,
        hasGivenBirth = hasGivenBirth,
    )
}

@Composable
fun CareplanActivitiesScreen(
    splitActivities: SplitCareplanActivities,
    onClickActivity: (Int) -> Unit,
    hasGivenBirth: Boolean,
) {
    val lazyState = rememberForeverLazyListState(key = "AppointmentsScreen")
    val barState = rememberForeverTopAppBarState("CareplanActivities")
    val scrollBehavior = TopAppBarDefaults.exitUntilCollapsedScrollBehavior(
        state = barState,
        canScroll = { true },
    )
    val scrollConnection = scrollBehavior.nestedScrollConnection

    var hasScrolled by rememberSaveable(hasGivenBirth) { mutableStateOf(false) }

    LaunchedEffect(splitActivities) {
        if (!hasScrolled && splitActivities.isNotEmpty()) {
            if (!hasGivenBirth) {
                lazyState.animateScrollToItem(splitActivities.past.size + 1) // + 1 is the header
            } else {
                lazyState.animateScrollToItem(splitActivities.past.size + 3) // + 3 is the two headers and the Due date card
            }

            hasScrolled = true
        }
    }

    val (pastBackgroundColor, futureBackgroundColor) = if (isSystemInDarkTheme()) {
        GravidTheme.colorScheme.background to GravidTheme.colorScheme.surfaceVariant
    } else {
        GravidTheme.gravidColorSystem.neutrals.grey02 to GravidTheme.colorScheme.background
    }

    Box(modifier = Modifier.fillMaxSize()) {
        Scaffold(
            modifier = Modifier
                .fillMaxSize()
                .nestedScroll(scrollConnection),
            topBar = {
                GravidExpandableTopBar(
                    title = { Text(stringResource(id = R.string.bottom_navigation_title_appointments)) },
                    scrollBehavior = scrollBehavior,
                )
            },
            containerColor = futureBackgroundColor,
        ) { innerPadding ->
            LazyColumn(
                modifier = Modifier.padding(top = innerPadding.calculateTopPadding()),
                state = lazyState,
            ) {
                items(splitActivities.past) { activity ->

                    ActivityRow(
                        activity = activity,
                        backgroundColor = pastBackgroundColor,
                        onClick = { onClickActivity(activity.id) },
                    )
                }
                if (splitActivities.past.isNotEmpty()) {
                    item {
                        ActivityTimelineHeader(
                            title = stringResource(id = R.string.past_appointments_title),
                            isBeforeToday = true,
                            backgroundColor = pastBackgroundColor,
                        )
                    }
                }

                if (splitActivities.future.isNotEmpty()) {
                    item {
                        ActivityTimelineHeader(
                            title = stringResource(id = R.string.future_appointments_title),
                            isBeforeToday = false,
                            backgroundColor = futureBackgroundColor,
                        )
                    }
                }

                items(splitActivities.future) { activity ->
                    ActivityRow(
                        activity = activity,
                        backgroundColor = futureBackgroundColor,
                        onClick = { onClickActivity(activity.id) },
                    )
                }

                item {
                    val offsetX = GravidTheme.dimens.largeSpacing.toPx
                    val lineColor = GravidTheme.colorScheme.onBackground

                    Box(
                        modifier = Modifier
                            .fillMaxWidth()
                            .align(Alignment.BottomCenter)
                            .drawWithContent {
                                // Draw image first
                                drawContent()
                                // Then draw the line above it
                                drawLine(
                                    color = lineColor,
                                    start = Offset(x = offsetX, y = 0f),
                                    end = Offset(x = offsetX, y = size.height),
                                    strokeWidth = 2.dp.toPx(),
                                )
                            },
                    ) {
                        Image(
                            modifier = Modifier.fillMaxWidth(),
                            painter = painterResource(id = R.drawable.ill_aftaler),
                            contentDescription = null,
                            contentScale = ContentScale.FillWidth,
                        )
                    }
                }
            }
        }

        InfoBox(
            modifier = Modifier.align(Alignment.BottomCenter),
            text = stringResource(id = R.string.appointment_info),
            isVisible = lazyState.isScrollingUp(),
        )
    }
}

@GravidPreview
@Composable
fun AppointmentsScreenPreview() {
    GravidTheme {
        CareplanActivitiesScreen(
            splitActivities = SplitCareplanActivities(
                past = listOf(
                    CareplanActivity(
                        id = 0,
                        scheduledString = "Uge 0 - 6",
                        title = "Opstart",
                        description = "Description here",
                        untilWeek = 6,
                        performer = CareplanActivity.Performer.Hospital,
                    ),
                ),
                future = listOf(
                    CareplanActivity(
                        id = 1,
                        scheduledString = "Uge 40",
                        title = "Fødselsforberedelse",
                        description = "Description here",
                        untilWeek = 40,
                        performer = CareplanActivity.Performer.OwnDoctor,
                    ),
                ),
            ),
            onClickActivity = {},
            hasGivenBirth = false,
        )
    }
}
