/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2023 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsp.mingraviditet.ui.main.careplanactivities.compose

import androidx.compose.foundation.background
import androidx.compose.foundation.isSystemInDarkTheme
import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.size
import androidx.compose.foundation.shape.CircleShape
import androidx.compose.material3.Surface
import androidx.compose.runtime.Composable
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.draw.drawBehind
import androidx.compose.ui.draw.drawWithCache
import androidx.compose.ui.geometry.Offset
import androidx.compose.ui.geometry.Rect
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.graphics.Path
import androidx.compose.ui.graphics.drawscope.clipPath
import androidx.compose.ui.res.painterResource
import androidx.compose.ui.unit.Dp
import androidx.compose.ui.unit.dp
import dk.nsp.mingraviditet.R
import dk.nsp.mingraviditet.gravidcompose.component.GravidCard
import dk.nsp.mingraviditet.gravidcompose.component.GravidIcon
import dk.nsp.mingraviditet.gravidcompose.component.GravidText
import dk.nsp.mingraviditet.gravidcompose.extensions.bold
import dk.nsp.mingraviditet.gravidcompose.extensions.conditional
import dk.nsp.mingraviditet.gravidcompose.theme.GravidTheme
import dk.nsp.mingraviditet.gravidcompose.utils.GravidPreview
import dk.nsp.mingraviditet.model.data.CareplanActivity

@Composable
fun ActivityRow(
    activity: CareplanActivity,
    backgroundColor: Color = Color.Transparent,
    onClick: () -> Unit,
) {
    val dotColor = when (activity.performer) {
        CareplanActivity.Performer.OwnDoctor -> GravidTheme.colorScheme.primary
        CareplanActivity.Performer.Hospital -> GravidTheme.gravidColorSystem.identity.treePoppy
        CareplanActivity.Performer.DueDate -> if (isSystemInDarkTheme()) GravidTheme.gravidColorSystem.darkMode.logan else GravidTheme.gravidColorSystem.text.martinique
    }

    ActivityRow(
        modifier = Modifier.padding(end = GravidTheme.dimens.largeSpacing),
        dotColor = dotColor,
        title = activity.title,
        subtitle = activity.scheduledString,
        isDueDate = activity.performer == CareplanActivity.Performer.DueDate,
        backgroundColor = backgroundColor,
        onClick = onClick,
    )
}

@Composable
fun ActivityRow(
    title: CharSequence,
    subtitle: CharSequence,
    backgroundColor: Color = Color.Transparent,
    isDueDate: Boolean,
    modifier: Modifier = Modifier,
    dotColor: Color = Color.Magenta,
    onClick: () -> Unit,
) {
    val lineColor = GravidTheme.colorScheme.onBackground
    Box(
        modifier = Modifier
            .fillMaxWidth()
            .background(backgroundColor)
            .drawBehind {
                drawLine(
                    color = lineColor,
                    start = Offset(x = 16.dp.toPx(), y = 0f),
                    end = Offset(x = 16.dp.toPx(), y = size.height),
                    strokeWidth = 2.dp.toPx(),
                )
            }
            .padding(vertical = GravidTheme.dimens.defaultSpacing)
            .then(modifier),
    ) {
        Row(modifier = Modifier.padding(start = 7.dp)) {
            AppointmentDot(
                modifier = Modifier.padding(top = GravidTheme.dimens.largeSpacing),
                color = dotColor,
            )

            GravidCard(
                modifier = Modifier
                    .padding(start = GravidTheme.dimens.largerSpacing)
                    .conditional(
                        condition = isDueDate,
                        ifTrue = { fillMaxWidth(0.5f) },
                        ifFalse = { fillMaxWidth() },
                    ),
                onClick = onClick,
            ) {
                Column {
                    GravidText(
                        text = title,
                        style = GravidTheme.typography.Header3,
                    )
                    Row(verticalAlignment = Alignment.CenterVertically) {
                        GravidText(
                            modifier = Modifier.weight(1f),
                            text = subtitle,
                            style = GravidTheme.typography.Info.bold,
                        )
                        GravidIcon(painter = painterResource(id = R.drawable.ic_chevron_right))
                    }
                }
            }
        }
    }
}

@Composable
private fun AppointmentDot(
    color: Color,
    modifier: Modifier = Modifier,
    horizontalLineWidth: Dp = 10.dp,
) {
    Surface(
        modifier = modifier.then(
            Modifier
                .size(18.dp)
                .drawWithCache {
                    val path = Path()
                    path.addOval(
                        Rect(
                            topLeft = Offset(
                                x = -size.width,
                                y = -size.height,
                            ),
                            bottomRight = Offset(
                                x = size.width * 2,
                                y = size.height * 2,
                            ),
                        ),
                    )

                    onDrawWithContent {
                        clipPath(path) {
                            this@onDrawWithContent.drawContent()
                        }

                        val largeDotRadius = 6.dp.toPx()
                        val smallDotRadius = 3.dp.toPx()

                        drawCircle(
                            color = color,
                            radius = largeDotRadius,
                            center = Offset(
                                x = size.width / 2,
                                y = size.height / 2,
                            ),
                        )

                        drawCircle(
                            color = color,
                            radius = smallDotRadius,
                            center = Offset(
                                x = size.width / 2 + largeDotRadius + smallDotRadius + horizontalLineWidth.toPx(),
                                y = size.height / 2,
                            ),
                        )

                        drawLine(
                            color = color,
                            start = Offset(
                                x = size.width / 2,
                                y = size.height / 2,
                            ),
                            end = Offset(
                                x = size.width / 2 + largeDotRadius + smallDotRadius + horizontalLineWidth.toPx(),
                                y = size.height / 2,
                            ),
                            strokeWidth = 2.dp.toPx(),
                        )
                    }
                },
        ),
        shadowElevation = GravidTheme.dimens.defaultCardElevation,
        color = GravidTheme.gravidColorSystem.neutrals.white,
        shape = CircleShape,
        content = {},
    )
}

@GravidPreview
@Composable
fun AppointmentRowPreview() {
    GravidTheme {
        Column {
            ActivityRow(
                activity = CareplanActivity(
                    id = 0,
                    scheduledString = "Uge 6 - 10",
                    title = "Læge konsultation",
                    description = "",
                    untilWeek = 10,
                    performer = CareplanActivity.Performer.OwnDoctor,
                ),
                onClick = {},
            )

            ActivityRow(
                CareplanActivity(
                    id = 1,
                    title = "Termin",
                    scheduledString = "Uge 41",
                    description = "",
                    untilWeek = 41,
                    performer = CareplanActivity.Performer.DueDate,
                ),
                onClick = {},
            )

            ActivityRow(
                CareplanActivity(
                    id = 2,
                    title = "Jordemoderbesøg",
                    scheduledString = "Uge 41",
                    description = "",
                    untilWeek = 41,
                    performer = CareplanActivity.Performer.Hospital,
                ),
                onClick = {},
            )
        }
    }
}
