/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2023 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsp.mingraviditet.ui.main.careplanactivities.details

import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.rememberScrollState
import androidx.compose.foundation.shape.RoundedCornerShape
import androidx.compose.foundation.verticalScroll
import androidx.compose.material3.Scaffold
import androidx.compose.material3.Surface
import androidx.compose.runtime.Composable
import androidx.compose.runtime.getValue
import androidx.compose.ui.Modifier
import androidx.compose.ui.res.stringResource
import androidx.hilt.navigation.compose.hiltViewModel
import androidx.lifecycle.compose.collectAsStateWithLifecycle
import dk.nsp.mingraviditet.R
import dk.nsp.mingraviditet.gravidcompose.component.GravidText
import dk.nsp.mingraviditet.gravidcompose.component.GravidTopBar
import dk.nsp.mingraviditet.gravidcompose.theme.GravidTheme
import dk.nsp.mingraviditet.ui.common.compose.LoadingScreen

@Composable
fun CareplanActivityRoute(
    viewModel: CareplanActivityDetailsViewModel = hiltViewModel(),
    onBackPressed: () -> Unit,
) {
    val activity by viewModel.careplanActivity.collectAsStateWithLifecycle()

    CareplanActivityScreen(
        state = activity,
        onBackPressed = onBackPressed,
    )
}

@Composable
fun CareplanActivityScreen(
    state: CareplanActivityDetailsUiState,
    onBackPressed: () -> Unit,
) {
    Scaffold(
        modifier = Modifier.fillMaxSize(),
        containerColor = GravidTheme.colorScheme.background,
        topBar = {
            GravidTopBar(
                title = stringResource(id = R.string.appointment_details_title),
                onNavigationClick = onBackPressed,
            )
        },
    ) { innerPadding ->
        Column(
            modifier = Modifier
                .fillMaxSize()
                .padding(top = innerPadding.calculateTopPadding()),
        ) {
            when (state) {
                CareplanActivityDetailsUiState.Loading -> LoadingScreen()
                is CareplanActivityDetailsUiState.Success -> {
                    Column(
                        Modifier.padding(
                            horizontal = GravidTheme.dimens.largeSpacing,
                            vertical = GravidTheme.dimens.largerSpacing,
                        ),
                    ) {
                        GravidText(
                            text = state.activity.title,
                            style = GravidTheme.typography.Header1,
                        )
                        GravidText(text = state.activity.scheduledString)
                    }

                    Surface(
                        modifier = Modifier
                            .fillMaxWidth()
                            .weight(1f),
                        color = GravidTheme.colorScheme.surfaceVariant,
                        shape = RoundedCornerShape(
                            topStart = GravidTheme.dimens.detailsSurfaceCornerRadius,
                            topEnd = GravidTheme.dimens.detailsSurfaceCornerRadius,
                        ),
                        shadowElevation = GravidTheme.dimens.defaultCardElevation,
                    ) {
                        Column(
                            modifier = Modifier
                                .padding(
                                    horizontal = GravidTheme.dimens.largeSpacing,
                                    vertical = GravidTheme.dimens.largerSpacing,
                                )
                                .verticalScroll(rememberScrollState()),
                        ) {
                            GravidText(
                                text = stringResource(id = R.string.appointment_details_description),
                                style = GravidTheme.typography.Header4,
                            )
                            GravidText(
                                modifier = Modifier.padding(top = GravidTheme.dimens.defaultSpacing),
                                text = state.activity.description,
                            )
                        }
                    }
                }
            }
        }
    }
}
