/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2023 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsp.mingraviditet.ui.main.common.compose

import androidx.compose.foundation.Image
import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.padding
import androidx.compose.runtime.Composable
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.res.painterResource
import androidx.compose.ui.text.style.TextOverflow
import androidx.compose.ui.unit.dp
import com.bumptech.glide.integration.compose.ExperimentalGlideComposeApi
import com.bumptech.glide.integration.compose.GlideImage
import dk.nsp.mingraviditet.R
import dk.nsp.mingraviditet.gravidcompose.component.GravidCard
import dk.nsp.mingraviditet.gravidcompose.component.GravidIcon
import dk.nsp.mingraviditet.gravidcompose.component.GravidText
import dk.nsp.mingraviditet.gravidcompose.theme.GravidTheme
import dk.nsp.mingraviditet.gravidcompose.utils.GravidIconType
import dk.nsp.mingraviditet.gravidcompose.utils.GravidPreview

@OptIn(ExperimentalGlideComposeApi::class)
@Composable
fun GravidCardWithIcon(
    title: CharSequence,
    description: CharSequence,
    modifier: Modifier = Modifier,
    tickerText: CharSequence = "",
    icon: GravidIconType? = null,
    onClick: () -> Unit,
) {
    GravidCard(
        modifier = modifier,
        onClick = onClick,
    ) {
        Row {
            if (icon != null) {
                InnerCircularCard(size = 48.dp, modifier = Modifier.padding(end = GravidTheme.dimens.largeSpacing)) {
                    when (icon) {
                        is GravidIconType.Drawable -> {
                            Image(
                                painter = painterResource(id = icon.resourceId),
                                contentDescription = null,
                                modifier = Modifier
                                    .fillMaxSize()
                                    .padding(GravidTheme.dimens.defaultSpacing),
                            )
                        }

                        is GravidIconType.Url -> {
                            icon.url?.let { url ->
                                GlideImage(
                                    model = url,
                                    contentDescription = null,
                                    modifier = Modifier
                                        .fillMaxSize()
                                        .padding(GravidTheme.dimens.defaultSpacing),
                                )
                            }
                        }
                    }
                }
            }
            Column(
                Modifier
                    .weight(1f),
                verticalArrangement = Arrangement.spacedBy(GravidTheme.dimens.trumpSpacing),
            ) {
                GravidText(
                    text = title,
                    style = GravidTheme.typography.Header4,
                )
                GravidText(
                    text = description,
                    maxLines = 2,
                    overflow = TextOverflow.Ellipsis,
                )
                Row(verticalAlignment = Alignment.CenterVertically) {
                    GravidText(
                        modifier = Modifier.weight(1f),
                        text = tickerText,
                        style = GravidTheme.typography.Info,
                        color = GravidTheme.colorScheme.onSurface,
                    )
                    GravidIcon(painter = painterResource(id = R.drawable.ic_chevron_right))
                }
            }
        }
    }
}

@GravidPreview
@Composable
private fun MinGraviditetCardPreview() {
    GravidTheme() {
        Column(verticalArrangement = Arrangement.spacedBy(GravidTheme.dimens.largeSpacing)) {
            GravidCardWithIcon(
                title = "Covid-19",
                description = "Det er vigtigt, at du som gravid eller fødende oplyser, om du eller din partner har symptomer på coronavirus",
                tickerText = "Vigtig information",
            ) {
            }

            GravidCardWithIcon(
                title = "Settings",
                description = "Access to data in the app, privacy policy and security in the app",
                icon = GravidIconType.Drawable(R.drawable.ic_settings),
            ) {
            }
        }
    }
}
