/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2023 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsp.mingraviditet.ui.main.knowledge

import androidx.compose.foundation.Image
import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.PaddingValues
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.lazy.LazyColumn
import androidx.compose.foundation.lazy.LazyListScope
import androidx.compose.foundation.lazy.LazyRow
import androidx.compose.foundation.lazy.items
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.filled.Search
import androidx.compose.material3.IconButton
import androidx.compose.material3.Scaffold
import androidx.compose.material3.Text
import androidx.compose.material3.TopAppBarDefaults
import androidx.compose.runtime.Composable
import androidx.compose.runtime.LaunchedEffect
import androidx.compose.runtime.getValue
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.input.nestedscroll.nestedScroll
import androidx.compose.ui.layout.ContentScale
import androidx.compose.ui.platform.LocalSoftwareKeyboardController
import androidx.compose.ui.res.painterResource
import androidx.compose.ui.res.pluralStringResource
import androidx.compose.ui.res.stringResource
import androidx.hilt.navigation.compose.hiltViewModel
import androidx.lifecycle.compose.collectAsStateWithLifecycle
import dk.nsp.domain.knowledge.Checklist
import dk.nsp.domain.knowledge.ChecklistItem
import dk.nsp.mingraviditet.R
import dk.nsp.mingraviditet.gravidcompose.component.GravidExpandableTopBar
import dk.nsp.mingraviditet.gravidcompose.component.GravidIcon
import dk.nsp.mingraviditet.gravidcompose.extensions.isScrollingDown
import dk.nsp.mingraviditet.gravidcompose.theme.GravidTheme
import dk.nsp.mingraviditet.gravidcompose.utils.GravidIconType
import dk.nsp.mingraviditet.gravidcompose.utils.GravidPreview
import dk.nsp.mingraviditet.gravidcompose.utils.rememberForeverLazyListState
import dk.nsp.mingraviditet.gravidcompose.utils.rememberForeverTopAppBarState
import dk.nsp.mingraviditet.model.data.ArticleCategory
import dk.nsp.mingraviditet.model.data.ArticleCategoryWithArticleCount
import dk.nsp.mingraviditet.ui.common.compose.ErrorStateRow
import dk.nsp.mingraviditet.ui.common.compose.loadingScreen
import dk.nsp.mingraviditet.ui.main.common.compose.GravidCardWithIcon
import dk.nsp.mingraviditet.ui.main.knowledge.checklist.compose.ChecklistCard
import dk.nsp.mingraviditet.ui.main.overview.compose.ArticleHeader
import dk.nsp.shared.result.Result

@Composable
fun KnowledgeRoute(
    onNavigateToChecklist: (checklistId: Int) -> Unit,
    onNavigateToCategory: (categoryId: Int) -> Unit,
    onNavigateToSearch: () -> Unit,
    knowledgeViewModel: KnowledgeViewModel = hiltViewModel(),
) {
    val state by knowledgeViewModel.knowledgeUiState.collectAsStateWithLifecycle()

    KnowledgeScreen(
        onNavigateToChecklist = onNavigateToChecklist,
        onNavigateToCategory = onNavigateToCategory,
        state = state,
        onNavigateToSearch = onNavigateToSearch,
    )
}

@Composable
fun KnowledgeScreen(
    onNavigateToChecklist: (checklistId: Int) -> Unit,
    onNavigateToCategory: (categoryId: Int) -> Unit,
    state: KnowledgeScreenState,
    onNavigateToSearch: () -> Unit,
) {
    val lazyState = rememberForeverLazyListState(key = "Knowledge")
    val barState = rememberForeverTopAppBarState("Knowledge")
    val scrollBehavior = TopAppBarDefaults.exitUntilCollapsedScrollBehavior(
        state = barState,
        canScroll = { true },
    )
    val scrollConnection = scrollBehavior.nestedScrollConnection

    // Hide keyboard when user scrolls in search results
    val scrollingDown = lazyState.isScrollingDown()
    val keyboardController = LocalSoftwareKeyboardController.current
    LaunchedEffect(scrollingDown) {
        if (scrollingDown) {
            keyboardController?.hide()
        }
    }

    Scaffold(
        modifier = Modifier
            .fillMaxSize()
            .nestedScroll(scrollConnection),
        containerColor = Color.Transparent,
        topBar = {
            GravidExpandableTopBar(
                title = { Text(text = stringResource(id = R.string.bottom_navigation_title_knowledge)) },
                scrollBehavior = scrollBehavior,
                actions = {
                    IconButton(onClick = onNavigateToSearch) {
                        GravidIcon(imageVector = Icons.Default.Search, contentDescription = stringResource(id = R.string.search_hint))
                    }
                },
            )
        },
    ) { innerPadding ->
        LazyColumn(
            state = lazyState,
            modifier = Modifier
                .fillMaxSize()
                .padding(top = innerPadding.calculateTopPadding()),
            horizontalAlignment = Alignment.CenterHorizontally,
            verticalArrangement = Arrangement.spacedBy(GravidTheme.dimens.defaultSpacing),
            contentPadding = PaddingValues(
                vertical = GravidTheme.dimens.largeSpacing,
            ),
        ) {
            when (state) {
                is KnowledgeScreenState.Success -> knowledgeOverviewScreen(state, onNavigateToChecklist, onNavigateToCategory)

                is KnowledgeScreenState.Loading -> loadingScreen()
                is KnowledgeScreenState.Failed -> item { ErrorStateRow() }
            }
        }
    }
}

private fun LazyListScope.knowledgeOverviewScreen(
    state: KnowledgeScreenState.Success,
    onNavigateToChecklist: (checklistId: Int) -> Unit,
    onNavigateToCategory: (categoryId: Int) -> Unit,
) {
    item {
        Column {
            ArticleHeader(
                modifier = Modifier.padding(horizontal = GravidTheme.dimens.largeSpacing),
                title = stringResource(id = R.string.checklists_title),
            )

            LazyRow(
                modifier = Modifier
                    .padding(vertical = GravidTheme.dimens.defaultSpacing),
                horizontalArrangement = Arrangement.spacedBy(GravidTheme.dimens.defaultSpacing),
                contentPadding = PaddingValues(horizontal = GravidTheme.dimens.largeSpacing),
            ) {
                items(state.checklists) { checklist ->
                    ChecklistCard(checklist = checklist, onClick = { onNavigateToChecklist(checklist.id.toInt()) })
                }
            }
        }
    }

    when (state.articleCategories) {
        is Result.Error -> item { ErrorStateRow() }
        Result.Loading -> loadingScreen()
        is Result.Success -> {
            item {
                ArticleHeader(
                    modifier = Modifier
                        .padding(top = GravidTheme.dimens.defaultSpacing)
                        .padding(horizontal = GravidTheme.dimens.largeSpacing),
                    title = stringResource(id = R.string.subjects_title),
                )
            }
            items(state.articleCategories.data) { wrapper ->
                GravidCardWithIcon(
                    modifier = Modifier.padding(horizontal = GravidTheme.dimens.largeSpacing),
                    title = wrapper.category.name,
                    description = wrapper.category.description,
                    icon = GravidIconType.Url(wrapper.category.icon),
                    tickerText = pluralStringResource(id = R.plurals.articles_count, count = wrapper.articleCount, wrapper.articleCount),
                    onClick = {
                        onNavigateToCategory(wrapper.category.id)
                    },
                )
            }
        }
    }

    item {
        Image(
            modifier = Modifier.fillMaxWidth(),
            painter = painterResource(id = R.drawable.ill_knowledge),
            contentDescription = null,
            contentScale = ContentScale.FillWidth,
        )
    }
}

@GravidPreview
@Composable
private fun KnowledgeScreenPreview() {
    val checklistItems = (0..5).toList().map {
        ChecklistItem(it.toString(), "Title $it", value = it % 2 == 1, order = it)
    }
    val checklists = (0..5).toList().map {
        Checklist(it.toString(), "Checklist $it", "Description $it", checklistItems)
    }

    GravidTheme {
        KnowledgeScreen(
            onNavigateToChecklist = {},
            onNavigateToCategory = {},
            state = KnowledgeScreenState.Success(
                checklists = checklists,
                articleCategories = Result.Success(
                    (1..5).map {
                        ArticleCategoryWithArticleCount(
                            ArticleCategory(
                                1,
                                1,
                                name = "Uge for uge",
                                description = "Følg med i din graviditet hver uge",
                                icon = null,
                                articles = emptyList(),
                                slug = "slug",
                            ),
                            articleCount = 4,
                        )
                    },
                ),
            ),
            onNavigateToSearch = { },
        )
    }
}
