/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021-2023 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsp.mingraviditet.ui.main.knowledge

import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import dagger.hilt.android.lifecycle.HiltViewModel
import dk.nsp.domain.knowledge.Checklist
import dk.nsp.domain.repository.ArticlesRepository
import dk.nsp.domain.repository.ChecklistRepository
import dk.nsp.mingraviditet.model.data.ArticleCategoryWithArticleCount
import dk.nsp.shared.result.Result
import kotlinx.coroutines.flow.Flow
import kotlinx.coroutines.flow.SharingStarted
import kotlinx.coroutines.flow.StateFlow
import kotlinx.coroutines.flow.combine
import kotlinx.coroutines.flow.flatMapLatest
import kotlinx.coroutines.flow.flow
import kotlinx.coroutines.flow.stateIn
import javax.inject.Inject

@HiltViewModel
class KnowledgeViewModel @Inject constructor(
    articlesRepo: ArticlesRepository,
    checklistRepository: ChecklistRepository,
) : ViewModel() {

    private val checklists = checklistRepository.getChecklists().stateIn(
        scope = viewModelScope,
        started = SharingStarted.WhileSubscribed(5_000),
        initialValue = listOf(),
    )

    private val articleCategories: Flow<Result<List<ArticleCategoryWithArticleCount>>> = articlesRepo.articleCategoriesWithCount
        .flatMapLatest {
            flow {
                if (it.isEmpty()) {
                    articlesRepo.getAllArticlesAndInsertIntoDatabase()
                    emit(Result.Loading)
                } else {
                    emit(Result.Success(it))
                }
            }
        }

    val knowledgeUiState: StateFlow<KnowledgeScreenState> = combine(checklists, articleCategories) { checklists, articles ->
        when (articles) {
            is Result.Error -> KnowledgeScreenState.Failed
            Result.Loading -> KnowledgeScreenState.Loading
            is Result.Success -> {
                KnowledgeScreenState.Success(checklists = checklists ?: emptyList(), articleCategories = articles)
            }
        }
    }.stateIn(
        scope = viewModelScope,
        started = SharingStarted.WhileSubscribed(5_000),
        initialValue = KnowledgeScreenState.Loading,
    )
}

sealed interface KnowledgeScreenState {
    data class Success(val checklists: List<Checklist>, val articleCategories: Result<List<ArticleCategoryWithArticleCount>>) : KnowledgeScreenState
    data object Loading : KnowledgeScreenState
    data object Failed : KnowledgeScreenState
}
