/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2023 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsp.mingraviditet.ui.main.knowledge.articledetails

import android.view.View
import android.webkit.WebChromeClient
import androidx.activity.compose.BackHandler
import androidx.annotation.StringRes
import androidx.compose.animation.animateColorAsState
import androidx.compose.animation.core.tween
import androidx.compose.foundation.background
import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.PaddingValues
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.lazy.LazyColumn
import androidx.compose.foundation.lazy.items
import androidx.compose.foundation.lazy.rememberLazyListState
import androidx.compose.material3.Scaffold
import androidx.compose.material3.SnackbarHost
import androidx.compose.material3.SnackbarHostState
import androidx.compose.runtime.Composable
import androidx.compose.runtime.LaunchedEffect
import androidx.compose.runtime.collectAsState
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableIntStateOf
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.saveable.rememberSaveable
import androidx.compose.runtime.setValue
import androidx.compose.ui.Modifier
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.res.stringResource
import androidx.compose.ui.text.style.TextAlign
import androidx.compose.ui.tooling.preview.Preview
import androidx.compose.ui.unit.dp
import androidx.hilt.navigation.compose.hiltViewModel
import androidx.lifecycle.compose.collectAsStateWithLifecycle
import dk.nsp.mingraviditet.R
import dk.nsp.mingraviditet.extensions.toMediumFormat
import dk.nsp.mingraviditet.gravidcompose.component.GravidText
import dk.nsp.mingraviditet.gravidcompose.component.GravidTopBar
import dk.nsp.mingraviditet.gravidcompose.theme.GravidTheme
import dk.nsp.mingraviditet.ui.common.compose.EmptyStateRow
import dk.nsp.mingraviditet.ui.common.compose.FoldableCard
import dk.nsp.mingraviditet.ui.common.compose.GravidSnackbar
import dk.nsp.mingraviditet.ui.common.compose.LoadingScreen

@Composable
internal fun KnowledgeArticleDetailsRoute(
    onBackPressed: () -> Unit,
    navigateToOtherArticle: (articleId: Int, sectionId: Int) -> Unit,
    viewModel: KnowledgeArticleDetailsViewModel = hiltViewModel(),
) {
    val uiState by viewModel.article.collectAsStateWithLifecycle()

    val navigateToInternalLink by viewModel.navigateToInternalLink.collectAsState(null)

    val fullScreenVideoView by remember { mutableStateOf(FullScreenVideoView()) }

    var showingVideoInFullScreen by remember {
        mutableStateOf(false)
    }
    BackHandler(enabled = showingVideoInFullScreen) {
        fullScreenVideoView.hideViewFromFullScreen()
    }

    @StringRes val toastTextRes: Int? by remember(navigateToInternalLink) {
        if (navigateToInternalLink != null && navigateToInternalLink?.first == null) {
            mutableIntStateOf(R.string.article_internal_link_not_found)
        } else {
            mutableStateOf(null)
        }
    }

    LaunchedEffect(navigateToInternalLink) {
        val articleId = navigateToInternalLink?.first
        if (articleId != null) {
            navigateToOtherArticle(articleId, navigateToInternalLink?.second ?: -1)
        }
    }

    KnowledgeArticleDetailsScreen(
        modifier = Modifier.fillMaxSize(),
        onBackPressed = onBackPressed,
        uiState = uiState,
        onInternalLinkPressed = {
            viewModel.findLinkedArticleById(it)
        },
        onShowFullScreen = { videoView, callback ->
            showingVideoInFullScreen = true
            fullScreenVideoView.shownViewInFullScreen(videoView, callback)
        },
        onCloseFullScreen = {
            showingVideoInFullScreen = false
            fullScreenVideoView.hideViewFromFullScreen()
        },
        toastTextResId = toastTextRes,
    )
}

@Composable
internal fun KnowledgeArticleDetailsScreen(
    modifier: Modifier = Modifier,
    uiState: ArticleDetailsUiState,
    onBackPressed: () -> Unit,
    onInternalLinkPressed: (String) -> Unit,
    onShowFullScreen: (view: View, callback: WebChromeClient.CustomViewCallback) -> Unit,
    onCloseFullScreen: () -> Unit,
    @StringRes toastTextResId: Int? = null,
) {
    val snackbarHostState = remember { SnackbarHostState() }
    val toastText = toastTextResId?.let { stringResource(id = it) }
    LaunchedEffect(toastText) {
        toastText?.let { text ->
            snackbarHostState.showSnackbar(message = text, withDismissAction = true)
        }
    }

    Scaffold(
        modifier = modifier,
        containerColor = GravidTheme.colorScheme.background,
        snackbarHost = {
            SnackbarHost(hostState = snackbarHostState) { snackbarData ->
                GravidSnackbar(snackbarData = snackbarData)
            }
        },
        topBar = {
            GravidTopBar(
                title = if (uiState is ArticleDetailsUiState.Success) uiState.article.title else "",
                onNavigationClick = onBackPressed,
            )
        },
    ) { innerPadding ->

        when (uiState) {
            ArticleDetailsUiState.Failure -> EmptyStateRow(
                modifier = Modifier.fillMaxSize(),
                title = stringResource(id = R.string.common_alert_title_error),
                description = stringResource(id = R.string.common_alert_message_generic),
            )

            ArticleDetailsUiState.Loading -> LoadingScreen()
            is ArticleDetailsUiState.Success -> {
                val sectionScrollState = rememberLazyListState()

                LaunchedEffect(Unit) {
                    val highlightSectionIndex = uiState.article.sections.indexOfFirst { it.id == uiState.sectionId }

                    if (highlightSectionIndex != -1) {
                        // +1 because the first element is the articles description
                        sectionScrollState.animateScrollToItem(highlightSectionIndex + 1)
                    }
                }

                LazyColumn(
                    modifier = Modifier
                        .fillMaxSize()
                        .padding(top = innerPadding.calculateTopPadding()),
                    state = sectionScrollState,
                    contentPadding = PaddingValues(
                        vertical = GravidTheme.dimens.defaultSpacing,
                    ),
                ) {
                    item {
                        GravidText(
                            modifier = Modifier.padding(
                                horizontal = GravidTheme.dimens.largeSpacing,
                                vertical = GravidTheme.dimens.defaultSpacing,
                            ),
                            text = uiState.article.description,
                        )
                    }

                    items(uiState.article.sections) { section ->
                        val colors = GravidTheme.colorScheme

                        var highlightShown by rememberSaveable {
                            mutableStateOf(false)
                        }

                        var initialBackgroundColor: Color by remember(highlightShown) {
                            mutableStateOf(
                                if (section.id == uiState.sectionId && !highlightShown) {
                                    colors.primary
                                } else {
                                    Color.Transparent
                                },
                            )
                        }

                        LaunchedEffect(Unit) {
                            if (!highlightShown && section.id == uiState.sectionId) {
                                initialBackgroundColor = Color.Transparent
                            }
                        }

                        val backgroundColorAnimation by animateColorAsState(
                            animationSpec = tween(durationMillis = 2000, delayMillis = 1000),
                            targetValue = initialBackgroundColor,
                            label = "Highlight animation",
                            finishedListener = {
                                highlightShown = true
                            },
                        )

                        val backgroundModifier = if (highlightShown) {
                            Modifier.background(Color.Transparent)
                        } else {
                            Modifier.background(backgroundColorAnimation)
                        }

                        Box(modifier = backgroundModifier.then(Modifier.padding(horizontal = 16.dp, vertical = 8.dp))) {
                            FoldableCard(
                                title = section.title,
                                content = section.content,
                                isExpanded = section.id == uiState.sectionId,
                                onInternalLinkClicked = onInternalLinkPressed,
                                onShowFullScreen = onShowFullScreen,
                                onCloseFullScreen = onCloseFullScreen,
                            )
                        }
                    }

                    item {
                        GravidText(
                            modifier = Modifier
                                .fillMaxWidth()
                                .padding(top = GravidTheme.dimens.largeSpacing),
                            text = stringResource(id = R.string.article_last_updated_on, uiState.article.lastUpdated.toMediumFormat() ?: ""),
                            style = GravidTheme.typography.Info,
                            textAlign = TextAlign.Center,
                        )
                    }
                }
            }
        }
    }
}

@Preview
@Composable
private fun KnowledgeArticleDetailsPreview() {
    GravidTheme {
        KnowledgeArticleDetailsScreen(
            onBackPressed = {},
            uiState = ArticleDetailsUiState.Loading,
            onInternalLinkPressed = {},
            onCloseFullScreen = {},
            onShowFullScreen = { _, _ -> },
        )
    }
}
