/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2023 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsp.mingraviditet.ui.main.knowledge.articledetails

import androidx.lifecycle.SavedStateHandle
import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import dagger.hilt.android.lifecycle.HiltViewModel
import dk.nsp.domain.repository.ArticlesRepository
import dk.nsp.mingraviditet.model.data.Article
import dk.nsp.mingraviditet.ui.main.knowledge.navigation.articleIdArg
import dk.nsp.mingraviditet.ui.main.knowledge.navigation.articleSectionIdArg
import kotlinx.coroutines.flow.Flow
import kotlinx.coroutines.flow.MutableSharedFlow
import kotlinx.coroutines.flow.SharingStarted
import kotlinx.coroutines.flow.StateFlow
import kotlinx.coroutines.flow.combine
import kotlinx.coroutines.flow.stateIn
import kotlinx.coroutines.launch
import javax.inject.Inject

typealias ArticleArgs = Pair<Int?, Int>?

@HiltViewModel
class KnowledgeArticleDetailsViewModel @Inject constructor(
    savedStateHandle: SavedStateHandle,
    private val articlesRepository: ArticlesRepository,
) : ViewModel() {

    private val articleId = savedStateHandle.getStateFlow(articleIdArg, -1)
    private val sectionId = savedStateHandle.getStateFlow(articleSectionIdArg, -1)

    private val _navigateToInternalLink = MutableSharedFlow<ArticleArgs>(replay = 0)
    val navigateToInternalLink: Flow<ArticleArgs> = _navigateToInternalLink

    val article: StateFlow<ArticleDetailsUiState> = combine(articleId, sectionId) { articleId, sectionId ->
        if (articleId != -1) {
            ArticleDetailsUiState.Success(articlesRepository.getPopulatedArticle(articleId), sectionId = sectionId)
        } else {
            ArticleDetailsUiState.Failure
        }
    }
        .stateIn(
            scope = viewModelScope,
            started = SharingStarted.WhileSubscribed(5_000),
            initialValue = ArticleDetailsUiState.Loading,
        )

    fun findLinkedArticleById(internalLink: String) = viewModelScope.launch {
        val (article, sectionId) = articlesRepository.getInternalArticleByLink(internalLink)
        _navigateToInternalLink.emit(article?.id to sectionId)
    }
}

sealed interface ArticleDetailsUiState {
    data object Loading : ArticleDetailsUiState
    data class Success(val article: Article, val sectionId: Int? = null) : ArticleDetailsUiState
    data object Failure : ArticleDetailsUiState
}
