/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2023 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsp.mingraviditet.ui.main.knowledge.articles

import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.PaddingValues
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.lazy.LazyColumn
import androidx.compose.foundation.lazy.items
import androidx.compose.material3.Scaffold
import androidx.compose.runtime.Composable
import androidx.compose.runtime.getValue
import androidx.compose.ui.Modifier
import androidx.compose.ui.res.stringResource
import androidx.hilt.navigation.compose.hiltViewModel
import androidx.lifecycle.compose.collectAsStateWithLifecycle
import dk.nsp.mingraviditet.R
import dk.nsp.mingraviditet.gravidcompose.component.GravidTopBar
import dk.nsp.mingraviditet.gravidcompose.theme.GravidTheme
import dk.nsp.mingraviditet.gravidcompose.utils.GravidIconType
import dk.nsp.mingraviditet.gravidcompose.utils.GravidPreview
import dk.nsp.mingraviditet.model.data.Article
import dk.nsp.mingraviditet.model.data.ArticleCategory
import dk.nsp.mingraviditet.ui.common.compose.EmptyStateRow
import dk.nsp.mingraviditet.ui.common.compose.HeaderWithIcon
import dk.nsp.mingraviditet.ui.common.compose.loadingScreen
import dk.nsp.mingraviditet.ui.main.common.compose.GravidCardWithIcon
import java.time.LocalDateTime

@Composable
fun CategoryRoute(
    onBackPressed: () -> Unit,
    onNavigateToArticle: (Article) -> Unit,
    articlesViewModel: KnowledgeArticlesViewModel = hiltViewModel(),
) {
    val category by articlesViewModel.populatedCategory.collectAsStateWithLifecycle()

    ArticlesScreen(
        onBackPressed = onBackPressed,
        category = category,
        onNavigateToArticle = onNavigateToArticle,
    )
}

@Composable
fun ArticlesScreen(
    onBackPressed: () -> Unit,
    category: ArticleCategory?,
    onNavigateToArticle: (Article) -> Unit,
) {
    Scaffold(
        containerColor = GravidTheme.colorScheme.background,
        topBar = {
            GravidTopBar(
                title = category?.name ?: "",
                onNavigationClick = onBackPressed,
            )
        },
    ) { innerPadding ->
        LazyColumn(
            modifier = Modifier
                .fillMaxSize()
                .padding(top = innerPadding.calculateTopPadding()),
            contentPadding = PaddingValues(
                vertical = GravidTheme.dimens.largerSpacing,
                horizontal = GravidTheme.dimens.largeSpacing,
            ),
            verticalArrangement = Arrangement.spacedBy(GravidTheme.dimens.largeSpacing),
        ) {
            if (category == null) {
                loadingScreen()
            } else {
                item {
                    HeaderWithIcon(
                        icon = GravidIconType.Url(category.icon),
                        title = category.name,
                        description = category.description,
                    )
                }
                when {
                    category.articles.isEmpty() -> item {
                        EmptyStateRow(
                            title = stringResource(id = R.string.articles_empty_state_title),
                            description = stringResource(id = R.string.articles_empty_state_description),
                        )
                    }

                    else -> {
                        items(category.articles) { article ->
                            GravidCardWithIcon(
                                title = article.title,
                                description = article.description,
                                tickerText = stringResource(id = R.string.common_cta_read_more),
                            ) {
                                onNavigateToArticle(article)
                            }
                        }
                    }
                }
            }
        }
    }
}

@GravidPreview
@Composable
private fun ArticlesScreenPreview() {
    GravidTheme {
        ArticlesScreen(
            onBackPressed = {},
            category = ArticleCategory(
                0,
                order = 0,
                name = "Uge for uge",
                description = "Følg med i din graviditet hver uge",
                icon = null,
                articles = (0 until 5).map {
                    Article(
                        id = 1,
                        title = "Tillykke",
                        searchOrder = null,
                        lastUpdated = LocalDateTime.now(),
                        description = "I den næste tid vil...",
                        content = null,
                        sections = emptyList(),
                    )
                },
                slug = "",
            ),
            onNavigateToArticle = {},
        )
    }
}
