/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2023 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsp.mingraviditet.ui.main.knowledge.checklist

import androidx.compose.animation.animateContentSize
import androidx.compose.animation.core.animateFloatAsState
import androidx.compose.animation.core.tween
import androidx.compose.foundation.background
import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.PaddingValues
import androidx.compose.foundation.layout.Spacer
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.height
import androidx.compose.foundation.layout.heightIn
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.lazy.LazyColumn
import androidx.compose.foundation.lazy.items
import androidx.compose.foundation.lazy.itemsIndexed
import androidx.compose.material3.Scaffold
import androidx.compose.material3.SnackbarDuration
import androidx.compose.material3.SnackbarHost
import androidx.compose.material3.SnackbarHostState
import androidx.compose.material3.SnackbarResult
import androidx.compose.runtime.Composable
import androidx.compose.runtime.LaunchedEffect
import androidx.compose.runtime.collectAsState
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableFloatStateOf
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.setValue
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.layout.onSizeChanged
import androidx.compose.ui.res.stringResource
import androidx.compose.ui.text.style.TextAlign
import androidx.compose.ui.text.style.TextOverflow
import androidx.compose.ui.unit.IntSize
import androidx.compose.ui.unit.dp
import androidx.hilt.navigation.compose.hiltViewModel
import dk.nsp.domain.knowledge.Checklist
import dk.nsp.domain.knowledge.ChecklistItem
import dk.nsp.domain.knowledge.completeness
import dk.nsp.mingraviditet.R
import dk.nsp.mingraviditet.extensions.nullIfBlank
import dk.nsp.mingraviditet.gravidcompose.component.GravidButton
import dk.nsp.mingraviditet.gravidcompose.component.GravidDivider
import dk.nsp.mingraviditet.gravidcompose.component.GravidText
import dk.nsp.mingraviditet.gravidcompose.component.GravidTextButton
import dk.nsp.mingraviditet.gravidcompose.component.GravidTopBar
import dk.nsp.mingraviditet.gravidcompose.extensions.toDp
import dk.nsp.mingraviditet.gravidcompose.theme.GravidTheme
import dk.nsp.mingraviditet.gravidcompose.utils.GravidPreview
import dk.nsp.mingraviditet.ui.common.compose.GravidSnackbar
import dk.nsp.mingraviditet.ui.main.common.compose.InnerCircularCard
import dk.nsp.mingraviditet.ui.main.overview.compose.widget.CircularPregnancyProgressIndicator
import dk.nsp.mingraviditet.ui.main.overview.compose.widget.OuterFetusWidgetCircle

@Composable
fun ChecklistDetailsRoute(
    onBackPressed: () -> Unit,
    checklistViewModel: ChecklistDetailsViewModel = hiltViewModel(),
) {
    val checklist by checklistViewModel.checklist.collectAsState(initial = null)
    val latestDeletionItem = checklistViewModel.pendingDeletions.map { it.title }
    val shouldShowDeleteGestureAnimation by checklistViewModel.shouldShowDeleteAnimation.collectAsState(initial = false)

    checklist?.let {
        ChecklistDetailsScreen(
            checklist = it,
            showSwipeToDeleteGesture = shouldShowDeleteGestureAnimation,
            onBackPressed = onBackPressed,
            onItemToggled = { toggledItem -> checklistViewModel.toggleChecklistItem(toggledItem) },
            onRequestDeleteItem = { requestDeletionOfItem -> checklistViewModel.deleteChecklistItem(requestDeletionOfItem) },
            onUndoDeletion = { checklistViewModel.undoDeletion() },
            onEditItem = { editedItem -> checklistViewModel.editItem(editedItem) },
            recoverChecklist = { checklistViewModel.recoverChecklist() },
            addTemporaryItem = { checklistViewModel.addTemporaryItem() },
            pendingDeletionItem = latestDeletionItem,
        )
    }
}

@Composable
fun ChecklistDetailsScreen(
    checklist: Checklist,
    showSwipeToDeleteGesture: Boolean,
    onBackPressed: () -> Unit,
    onItemToggled: (ChecklistItem) -> Unit,
    onRequestDeleteItem: (ChecklistItem) -> Unit,
    onUndoDeletion: () -> Unit,
    addTemporaryItem: () -> Unit,
    onEditItem: (ChecklistItem) -> Unit,
    recoverChecklist: () -> Unit,
    pendingDeletionItem: List<String>?,
) {
    var descriptionExpanded by remember {
        mutableStateOf(false)
    }

    var snackbarSize by remember {
        mutableStateOf(IntSize.Zero)
    }

    val progress by remember(checklist.items) {
        mutableFloatStateOf(checklist.items.completeness())
    }

    val animatedProgress by animateFloatAsState(
        targetValue = progress,
        animationSpec = tween(
            durationMillis = 500,
        ),
        label = "Animated progress",
    )

    val uncheckedItems = checklist.items.filterNot { it.value }
    val checkedItems = checklist.items.filter { it.value }

    val snackbarHostState = remember { SnackbarHostState() }

    val mostRecentlyDeleted = pendingDeletionItem?.lastOrNull()?.trim()
    val snackbarText = if (mostRecentlyDeleted != null) stringResource(id = R.string.checklist_deleted_item, mostRecentlyDeleted) else null
    val snackbarActionLabel = stringResource(id = R.string.common_regret)

    LaunchedEffect(pendingDeletionItem) {
        if (snackbarText != null) {
            val snackbarResult = snackbarHostState.showSnackbar(
                message = snackbarText,
                actionLabel = snackbarActionLabel,
                duration = SnackbarDuration.Short,
            )
            when (snackbarResult) {
                SnackbarResult.Dismissed -> Unit
                SnackbarResult.ActionPerformed -> onUndoDeletion()
            }
        }
    }

    Scaffold(
        modifier = Modifier.fillMaxSize(),
        containerColor = GravidTheme.colorScheme.background,
        topBar = {
            GravidTopBar(
                title = stringResource(id = R.string.checklist_title),
                onNavigationClick = onBackPressed,
            )
        },
        snackbarHost = {
            SnackbarHost(hostState = snackbarHostState) { snackbarData ->
                GravidSnackbar(
                    snackbarData = snackbarData,
                    modifier = Modifier.onSizeChanged { snackbarSize = it },
                )
            }
        },
    ) { innerPadding ->
        LazyColumn(
            modifier = Modifier.fillMaxSize(),
            contentPadding = PaddingValues(top = innerPadding.calculateTopPadding()),
        ) {
            item {
                Column(
                    Modifier
                        .fillMaxWidth()
                        .padding(
                            vertical = GravidTheme.dimens.largeSpacing,
                            horizontal = GravidTheme.dimens.largeSpacing,
                        ),
                    horizontalAlignment = Alignment.CenterHorizontally,
                ) {
                    val outerCircleSize = 100.dp
                    val strokeWidth = 4.dp
                    val spacingBetweenOuterAndInnerCircle = 6.dp + strokeWidth
                    val innerCircleSize = outerCircleSize - spacingBetweenOuterAndInnerCircle * 2 - strokeWidth * 2

                    OuterFetusWidgetCircle(
                        modifier = Modifier
                            .padding(top = GravidTheme.dimens.defaultSpacing)
                            .align(Alignment.CenterHorizontally),
                        size = outerCircleSize,
                    ) {
                        Box(
                            modifier = Modifier
                                .fillMaxSize()
                                .background(GravidTheme.colorScheme.tertiaryContainer),
                            contentAlignment = Alignment.Center,
                        ) {
                            CircularPregnancyProgressIndicator(
                                modifier = Modifier.padding(spacingBetweenOuterAndInnerCircle / 2),
                                progress = animatedProgress,
                                strokeWidth = strokeWidth,
                            )

                            InnerCircularCard(
                                size = innerCircleSize,
                                containerColor = GravidTheme.colorScheme.secondaryContainer,
                            ) {
                                Box(Modifier.fillMaxSize(), contentAlignment = Alignment.Center) {
                                    GravidText(
                                        text = checklist.getPercentageAsString(),
                                        style = GravidTheme.typography.Header1,
                                        textAlign = TextAlign.Center,
                                        color = GravidTheme.colorScheme.primary,
                                    )
                                }
                            }
                        }
                    }

                    GravidText(
                        modifier = Modifier
                            .align(Alignment.CenterHorizontally),
                        text = checklist.title,
                        style = GravidTheme.typography.Header1,
                    )

                    GravidText(
                        modifier = Modifier
                            .align(Alignment.CenterHorizontally),
                        text = checklist.completenessAsDivision(),
                    )

                    checklist.description?.nullIfBlank?.let { description ->
                        GravidText(
                            modifier = Modifier
                                .padding(top = GravidTheme.dimens.largerSpacing)
                                .animateContentSize(animationSpec = tween()),
                            maxLines = if (descriptionExpanded) Int.MAX_VALUE else 2,
                            overflow = TextOverflow.Ellipsis,
                            text = description,
                        )

                        GravidDivider(modifier = Modifier.padding(top = GravidTheme.dimens.largeSpacing))

                        val foldButtonText = if (descriptionExpanded) {
                            stringResource(id = R.string.common_cta_collapse)
                        } else {
                            stringResource(id = R.string.common_cta_expand)
                        }

                        GravidTextButton(
                            modifier = Modifier.fillMaxWidth(),
                            contentPadding = PaddingValues(vertical = GravidTheme.dimens.trumpSpacing),
                            text = foldButtonText,
                            onClick = { descriptionExpanded = !descriptionExpanded },
                        )
                    }
                }
            }

            if (checklist.items.isEmpty()) {
                item {
                    Column(
                        modifier = Modifier.fillMaxWidth(),
                        horizontalAlignment = Alignment.CenterHorizontally,
                    ) {
                        GravidText(
                            text = stringResource(id = R.string.checklist_recover_original_description),
                            style = GravidTheme.typography.BodyStandard,
                            textAlign = TextAlign.Center,
                        )
                        GravidButton(
                            onClick = recoverChecklist,
                            text = stringResource(id = R.string.checklist_recover_original_button),
                            containerColor = GravidTheme.colorScheme.secondary,
                        )
                    }
                }
            } else {
                itemsIndexed(
                    items = uncheckedItems,
                    key = { _, checklistItem -> checklistItem.id + checklistItem.title },
                    itemContent = { i, checklistItem ->
                        ChecklistItemCard(
                            fakeDrag = i == 0 && showSwipeToDeleteGesture,
                            forceFocus = i == uncheckedItems.size - 1 && checklistItem.title.isBlank(),
                            checklistItem = checklistItem,
                            onRequestToDeleteItem = {
                                onRequestDeleteItem(checklistItem)
                            },
                            onItemToggled = onItemToggled,
                            onEditItem = { item, newText ->
                                onEditItem(item.copy(title = newText))
                            },
                        )
                    },
                )
            }

            item {
                GravidButton(
                    modifier = Modifier
                        .fillMaxWidth()
                        .padding(GravidTheme.dimens.largeSpacing)
                        .heightIn(min = 50.dp),
                    onClick = addTemporaryItem,
                    text = stringResource(id = R.string.checklist_add_item, checklist.title.replaceFirstChar { it.lowercase().trim() }),
                )
            }

            if (checkedItems.isNotEmpty()) {
                item {
                    GravidText(
                        modifier = Modifier
                            .padding(horizontal = GravidTheme.dimens.largeSpacing)
                            .padding(top = GravidTheme.dimens.largeSpacing),
                        style = GravidTheme.typography.Header4,
                        text = stringResource(id = R.string.checklist_checked_items_title),
                    )
                }
            }

            items(
                checkedItems,
                key = { checklistItem -> checklistItem.id + checklistItem.title },
            ) { checkedItem ->
                ChecklistItemCard(
                    fakeDrag = false,
                    forceFocus = false,
                    checklistItem = checkedItem,
                    onRequestToDeleteItem = {
                        onRequestDeleteItem(checkedItem)
                    },
                    onItemToggled = onItemToggled,
                    onEditItem = null,
                )
            }

            // Bottom padding for entire view when snackbar is showing
            item {
                val additionalHeight = if (pendingDeletionItem?.isNotEmpty() == true) {
                    snackbarSize.height.toDp
                } else {
                    0.dp
                }
                Spacer(Modifier.height(GravidTheme.dimens.largeSpacing + additionalHeight))
            }
        }
    }
}

@GravidPreview
@Composable
private fun ChecklistDetailsScreenPreview() {
    val checklistItems = (0..5).toList().map {
        ChecklistItem(it.toString(), "Title $it", value = it % 2 == 1, order = it)
    }
    val checklist = Checklist(
        "0",
        "Fødetasken",
        "Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris nisi ut aliquip ex ea commodo consequat. Duis aute irure dolor in reprehenderit in voluptate velit esse cillum dolore eu fugiat nulla pariatur. ",
        checklistItems,
    )
    GravidTheme {
        ChecklistDetailsScreen(
            checklist = checklist,
            onBackPressed = {},
            onItemToggled = {},
            showSwipeToDeleteGesture = true,
            onEditItem = {},
            recoverChecklist = {},
            onUndoDeletion = {},
            onRequestDeleteItem = {},
            pendingDeletionItem = null,
            addTemporaryItem = {},
        )
    }
}
