/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2023 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsp.mingraviditet.ui.main.knowledge.checklist

import androidx.lifecycle.SavedStateHandle
import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import dagger.hilt.android.lifecycle.HiltViewModel
import dk.nsp.domain.knowledge.ChecklistItem
import dk.nsp.domain.repository.ChecklistRepository
import dk.nsp.mingraviditet.ui.main.knowledge.navigation.checklistIdArg
import dk.nsp.shared.network.Dispatcher
import dk.nsp.shared.network.MGDispatchers
import kotlinx.coroutines.CoroutineDispatcher
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.delay
import kotlinx.coroutines.flow.MutableStateFlow
import kotlinx.coroutines.flow.combine
import kotlinx.coroutines.flow.map
import kotlinx.coroutines.flow.mapNotNull
import kotlinx.coroutines.flow.update
import kotlinx.coroutines.launch
import java.util.UUID
import javax.inject.Inject

@HiltViewModel
class ChecklistDetailsViewModel @Inject constructor(
    savedStateHandle: SavedStateHandle,
    private val checklistRepository: ChecklistRepository,
    @Dispatcher(MGDispatchers.IO) private val ioDispatcher: CoroutineDispatcher,
) : ViewModel() {

    private val checklistId = savedStateHandle.getStateFlow(checklistIdArg, -1)

    private val _pendingDeletions = MutableStateFlow<List<ChecklistItem>>(emptyList())

    val pendingDeletions: List<ChecklistItem> get() = _pendingDeletions.value

    private val userAddedChecklistItem = MutableStateFlow<ChecklistItem?>(null)

    val checklist = combine(checklistId, checklistRepository.getChecklists()) { checklistId, checklists ->
        checklists?.firstOrNull { it.id == checklistId.toString() }
    }
        .mapNotNull { it }
        .combine(_pendingDeletions) { checklist, delete ->
            val filteredForPendingDeletions = checklist.items.filterNot { it.id in delete.map { it.id } }
            checklist.copy(items = filteredForPendingDeletions)
        }
        .combine(userAddedChecklistItem) { checklist, newItem ->
            if (newItem != null) {
                checklist.copy(items = checklist.items.toMutableList().apply { add(newItem) })
            } else {
                checklist
            }
        }
        .map { it.copy(items = it.items.sortedBy { it.order }) }

    val shouldShowDeleteAnimation = checklistRepository.shouldShowDeleteChecklistItemAnimation

    init {
        viewModelScope.launch {
            checklistRepository.incrementChecklistItemDeleteAnimationShownCount()
        }
    }

    fun deleteChecklistItem(checklistItem: ChecklistItem) = viewModelScope.launch {
        // Disable animation, because user knows how to do it now.
        checklistRepository.disableChecklistItemDeleteAnimation()

        if (checklistItem.title.isNotBlank()) {
            _pendingDeletions.update { it + checklistItem }
            delay(4000L) // Align time with the SnackBars' duration
            if (_pendingDeletions.value.contains(checklistItem)) {
                checklistRepository.delete(checklistId.value, checklistItem)

                _pendingDeletions.update {
                    it.filterNot { it.id == checklistItem.id }
                }
            }
        } else {
            checklistRepository.delete(checklistId.value, checklistItem)
        }
    }

    fun undoDeletion() {
        val latestDeletion = _pendingDeletions.value.lastOrNull()
        if (_pendingDeletions.value.contains(latestDeletion)) {
            _pendingDeletions.update { it.filterNot { it.id == latestDeletion?.id } }
        }
    }

    fun addTemporaryItem() = viewModelScope.launch {
        val currentSize = 100 + pendingDeletions.size
        userAddedChecklistItem.value = ChecklistItem(id = UUID.randomUUID().toString(), title = "", value = false, currentSize)
    }

    fun toggleChecklistItem(checklistItem: ChecklistItem) = viewModelScope.launch {
        checklistRepository.toggle(checklistItem)
    }

    fun editItem(checklistItem: ChecklistItem) = viewModelScope.launch {
        if (checklistItem.title.isNotBlank()) {
            checklistRepository.update(checklistId.value, checklistItem)
        }
        // Reset user added, so we don't get a empty row after submitting
        userAddedChecklistItem.value = null
    }

    fun recoverChecklist() = viewModelScope.launch {
        // Clear pending deletions
        _pendingDeletions.value.forEach {
            checklistRepository.delete(checklistId.value, it)
        }
        _pendingDeletions.update {
            emptyList()
        }

        checklistRepository.recoverChecklist(checklistId.value)
    }

    override fun onCleared() {
        // Spawn new threads to delete items in the background
        CoroutineScope(ioDispatcher).launch {
            _pendingDeletions.value.map { checklistItem ->
                checklistRepository.delete(checklistId.value, checklistItem)
            }
        }
        super.onCleared()
    }
}
