/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2023 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsp.mingraviditet.ui.main.knowledge.checklist

import androidx.compose.animation.Crossfade
import androidx.compose.animation.core.animateDpAsState
import androidx.compose.foundation.ExperimentalFoundationApi
import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.heightIn
import androidx.compose.foundation.layout.offset
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.size
import androidx.compose.foundation.layout.wrapContentHeight
import androidx.compose.foundation.lazy.LazyColumn
import androidx.compose.foundation.lazy.LazyItemScope
import androidx.compose.foundation.lazy.items
import androidx.compose.foundation.text.KeyboardActions
import androidx.compose.material3.DismissDirection
import androidx.compose.material3.DismissValue
import androidx.compose.material3.SwipeToDismiss
import androidx.compose.material3.rememberDismissState
import androidx.compose.runtime.Composable
import androidx.compose.runtime.LaunchedEffect
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableFloatStateOf
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.rememberUpdatedState
import androidx.compose.runtime.setValue
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.focus.onFocusChanged
import androidx.compose.ui.hapticfeedback.HapticFeedbackType
import androidx.compose.ui.platform.LocalFocusManager
import androidx.compose.ui.platform.LocalHapticFeedback
import androidx.compose.ui.platform.LocalSoftwareKeyboardController
import androidx.compose.ui.text.input.TextFieldValue
import androidx.compose.ui.unit.dp
import dk.nsp.domain.knowledge.ChecklistItem
import dk.nsp.mingraviditet.gravidcompose.component.GravidCard
import dk.nsp.mingraviditet.gravidcompose.component.GravidCheckbox
import dk.nsp.mingraviditet.gravidcompose.component.GravidText
import dk.nsp.mingraviditet.gravidcompose.component.GravidTextField
import dk.nsp.mingraviditet.gravidcompose.extensions.bold
import dk.nsp.mingraviditet.gravidcompose.theme.GravidTheme
import dk.nsp.mingraviditet.gravidcompose.utils.GravidPreview
import kotlinx.coroutines.delay

enum class ChecklistCardMode {
    Present, Edit
}

@OptIn(ExperimentalFoundationApi::class)
@Composable
fun LazyItemScope.ChecklistItemCard(
    fakeDrag: Boolean,
    forceFocus: Boolean,
    checklistItem: ChecklistItem,
    maxLength: Int = 80,
    onRequestToDeleteItem: (ChecklistItem) -> Unit,
    onItemToggled: (ChecklistItem) -> Unit,
    onEditItem: ((ChecklistItem, newText: String) -> Unit)?,
) {
    val focusManager = LocalFocusManager.current
    val keyboardController = LocalSoftwareKeyboardController.current

    val currentItem by rememberUpdatedState(checklistItem)
    val dismissThreshold = 0.33f
    var currentFraction by remember { mutableFloatStateOf(0f) }

    val dismissState = rememberDismissState(
        confirmValueChange = {
            // Make sure that the users don't accidentally swipe vertically while in "dismiss" state
            if (it == DismissValue.DismissedToStart && currentFraction >= dismissThreshold && currentFraction < 1.0f) {
                onRequestToDeleteItem(currentItem)
                true
            } else {
                false
            }
        },
        positionalThreshold = { it / 3 },
    )

    var fakeDragOffsetXDP by remember {
        mutableStateOf(0.dp)
    }

    val fakeDragState by animateDpAsState(targetValue = fakeDragOffsetXDP, label = "fakedrag")
    LaunchedEffect(fakeDrag) {
        if (fakeDrag) {
            delay(500)
            fakeDragOffsetXDP = (-50).dp
            delay(2_000)
            fakeDragOffsetXDP = 0.dp
        }
    }

    var cardMode by remember {
        mutableStateOf(if (forceFocus) ChecklistCardMode.Edit else ChecklistCardMode.Present)
    }

    val haptic = LocalHapticFeedback.current
    LaunchedEffect(dismissState.targetValue) {
        if (dismissState.targetValue == DismissValue.DismissedToStart) {
            haptic.performHapticFeedback(HapticFeedbackType.LongPress)
        }
    }

    SwipeToDismiss(
        directions = setOf(DismissDirection.EndToStart),
        modifier = Modifier.animateItemPlacement(),
        state = dismissState,
        background = {
            currentFraction = dismissState.progress
            ChecklistItemSwipeBackground(dismissState)
        },
        dismissContent = {
            GravidCard(
                modifier = Modifier
                    .fillMaxWidth()
                    .offset(x = fakeDragState)
                    .padding(horizontal = GravidTheme.dimens.largeSpacing)
                    .padding(top = GravidTheme.dimens.defaultSpacing),
                onClick = {
                    if (onEditItem != null) {
                        cardMode = when (cardMode) {
                            ChecklistCardMode.Present -> ChecklistCardMode.Edit
                            ChecklistCardMode.Edit -> {
                                if (checklistItem.title.isNotBlank()) {
                                    ChecklistCardMode.Present
                                } else {
                                    ChecklistCardMode.Edit
                                }
                            }
                        }
                    }
                },
            ) {
                Row(
                    modifier = Modifier.fillMaxWidth(),
                    horizontalArrangement = Arrangement.spacedBy(GravidTheme.dimens.largeSpacing),
                    verticalAlignment = Alignment.CenterVertically,
                ) {
                    GravidCheckbox(
                        modifier = Modifier.size(28.dp),
                        checked = checklistItem.value,
                        onCheckedChanged = {
                            onItemToggled(checklistItem)
                        },
                    )

                    Crossfade(targetState = cardMode, label = "") { mode ->
                        when (mode) {
                            ChecklistCardMode.Present -> {
                                GravidText(
                                    modifier = Modifier
                                        .heightIn(min = 55.dp)
                                        .wrapContentHeight(),
                                    text = checklistItem.title.bold,
                                )
                            }

                            ChecklistCardMode.Edit -> {
                                var checklistText by remember {
                                    mutableStateOf(checklistItem.title)
                                }

                                var hasHadFocus by remember {
                                    mutableStateOf(false)
                                }

                                GravidTextField(
                                    keyboardActions = KeyboardActions(onDone = {
                                        focusManager.clearFocus()
                                        keyboardController?.hide()
                                    }),
                                    maxLength = maxLength,
                                    modifier = Modifier
                                        .heightIn(min = 55.dp)
                                        .onFocusChanged {
                                            if (it.isFocused) {
                                                hasHadFocus = true
                                            } else {
                                                if (hasHadFocus) {
                                                    cardMode = ChecklistCardMode.Present
                                                    onEditItem?.invoke(checklistItem, checklistText)
                                                }
                                            }
                                        },
                                    value = TextFieldValue(checklistText),
                                    onValueChange = {
                                        checklistText = it.text
                                    },
                                )
                            }
                        }
                    }
                }
            }
        },
    )
}

@GravidPreview
@Composable
private fun ChecklistItemCardPreview() {
    val checklistItems = (0..5).toList().map {
        ChecklistItem(
            id = it.toString(),
            title = "Title $it",
            value = false,
            order = it,
        )
    }

    GravidTheme {
        LazyColumn {
            item {
                ChecklistItemCard(
                    fakeDrag = false,
                    forceFocus = true,
                    checklistItem = ChecklistItem("", "focused thing", false, -1),
                    onRequestToDeleteItem = {},
                    onItemToggled = {},
                    onEditItem = { _, _ -> },
                )
            }
            items(checklistItems) {
                ChecklistItemCard(
                    fakeDrag = false,
                    forceFocus = false,
                    checklistItem = it,
                    onRequestToDeleteItem = {},
                    onItemToggled = {},
                    onEditItem = { _, _ -> },
                )
            }
        }
    }
}
