/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2023 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsp.mingraviditet.ui.main.knowledge.navigation

import androidx.navigation.NavController
import androidx.navigation.NavGraphBuilder
import androidx.navigation.NavOptions
import androidx.navigation.NavType
import androidx.navigation.compose.composable
import androidx.navigation.navArgument
import androidx.navigation.navigation
import dk.nsp.mingraviditet.ui.main.common.navigation.animatedComposable
import dk.nsp.mingraviditet.ui.main.knowledge.KnowledgeRoute
import dk.nsp.mingraviditet.ui.main.knowledge.articledetails.KnowledgeArticleDetailsRoute
import dk.nsp.mingraviditet.ui.main.knowledge.articles.CategoryRoute
import dk.nsp.mingraviditet.ui.main.knowledge.checklist.ChecklistDetailsRoute

const val knowledgeGraphRoute = "knowledge_graph"
const val knowledgeRoute = "knowledge_route"

const val articleIdArg = "articleId"
const val articleSectionIdArg = "sectionIdArg"
const val articleRoute = "article_route/{$articleIdArg}?sectionId={$articleSectionIdArg}"

const val categoryIdArg = "categoryId"
const val categoryRoute = "category_route/{$categoryIdArg}"

const val checklistIdArg = "checklistId"
const val checklistRoute = "checklist_route/{$checklistIdArg}"

fun NavController.navigateToChecklist(checklistId: Int) {
    this.navigate("checklist_route/$checklistId") {
        launchSingleTop = true
    }
}

fun NavController.navigateToCategory(categoryId: Int) {
    this.navigate("category_route/$categoryId")
}

fun NavController.navigateToArticleDetailsScreen(articleId: Int, relevantId: Int? = null) {
    this.navigate("article_route/$articleId?sectionId=$relevantId")
}

fun NavController.navigateToKnowledgeGraph(navOptions: NavOptions? = null) {
    this.navigate(knowledgeGraphRoute, navOptions)
}

fun NavGraphBuilder.knowledgeGraph(navController: NavController) {
    navigation(
        startDestination = knowledgeRoute,
        route = knowledgeGraphRoute,
    ) {
        composable(route = knowledgeRoute) {
            KnowledgeRoute(
                onNavigateToChecklist = navController::navigateToChecklist,
                onNavigateToCategory = navController::navigateToCategory,
                onNavigateToSearch = navController::navigateToSearch,
            )
        }

        searchScreen(
            onBackClick = navController::navigateUp,
            navigateToArticleDetails = { articleId, sectionId ->
                navController.navigateToArticleDetailsScreen(articleId, sectionId ?: -1)
            },
        )

        animatedComposable(
            route = categoryRoute,
            arguments = listOf(
                navArgument(categoryIdArg) { type = NavType.IntType },
            ),

        ) {
            CategoryRoute(
                onBackPressed = navController::navigateUp,
                onNavigateToArticle = { id -> navController.navigateToArticleDetailsScreen(id.id) },
            )
        }

        animatedComposable(
            route = checklistRoute,
            arguments = listOf(
                navArgument(checklistIdArg) { type = NavType.IntType },
            ),
        ) {
            ChecklistDetailsRoute(onBackPressed = navController::navigateUp)
        }

        animatedComposable(
            route = articleRoute,
            arguments = listOf(
                navArgument(articleIdArg) { type = NavType.IntType },
                navArgument(articleSectionIdArg) {
                    type = NavType.IntType
                    defaultValue = -1
                },
            ),
        ) {
            KnowledgeArticleDetailsRoute(
                onBackPressed = navController::navigateUp,
                navigateToOtherArticle = navController::navigateToArticleDetailsScreen,
            )
        }
    }
}
