/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2023 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsp.mingraviditet.ui.main.more.compose

import androidx.compose.foundation.Image
import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.PaddingValues
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.lazy.LazyColumn
import androidx.compose.foundation.lazy.items
import androidx.compose.material3.Scaffold
import androidx.compose.material3.Text
import androidx.compose.material3.TopAppBarDefaults
import androidx.compose.runtime.Composable
import androidx.compose.ui.Modifier
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.input.nestedscroll.nestedScroll
import androidx.compose.ui.layout.ContentScale
import androidx.compose.ui.res.painterResource
import androidx.compose.ui.res.stringResource
import dk.nsp.mingraviditet.R
import dk.nsp.mingraviditet.gravidcompose.component.GravidExpandableTopBar
import dk.nsp.mingraviditet.gravidcompose.theme.GravidTheme
import dk.nsp.mingraviditet.gravidcompose.utils.GravidPreview
import dk.nsp.mingraviditet.gravidcompose.utils.rememberForeverLazyListState
import dk.nsp.mingraviditet.gravidcompose.utils.rememberForeverTopAppBarState
import dk.nsp.mingraviditet.ui.main.common.compose.GravidCardWithIcon
import dk.nsp.mingraviditet.ui.main.more.MoreSection

@Composable
fun MoreRoute(
    navigateToSupport: () -> Unit,
    navigateToSettings: () -> Unit,
    navigateToWorkInProgress: () -> Unit,
) {
    MoreScreen(onNavigateToSection = {
        when (it) {
            MoreSection.Support -> navigateToSupport()
            MoreSection.Settings -> navigateToSettings()
            MoreSection.WorkInProgress -> navigateToWorkInProgress()
        }
    })
}

@Composable
fun MoreScreen(
    onNavigateToSection: (MoreSection) -> Unit,
) {
    val lazyState = rememberForeverLazyListState(key = "MoreScreen")
    val barState = rememberForeverTopAppBarState("MoreScreen")
    val scrollBehavior = TopAppBarDefaults.exitUntilCollapsedScrollBehavior(
        state = barState,
        canScroll = { true },
    )
    val scrollConnection = scrollBehavior.nestedScrollConnection

    Scaffold(
        modifier = Modifier
            .fillMaxSize()
            .nestedScroll(scrollConnection),
        topBar = {
            GravidExpandableTopBar(
                title = { Text(stringResource(id = R.string.bottom_navigation_title_more)) },
                scrollBehavior = scrollBehavior,
            )
        },
        containerColor = Color.Transparent,
    ) { innerPadding ->
        LazyColumn(
            modifier = Modifier
                .padding(top = innerPadding.calculateTopPadding()),
            state = lazyState,
            verticalArrangement = Arrangement.spacedBy(GravidTheme.dimens.largeSpacing),
            contentPadding = PaddingValues(top = GravidTheme.dimens.largerSpacing),
        ) {
            items(MoreSection.values()) { section ->
                GravidCardWithIcon(
                    modifier = Modifier.padding(horizontal = GravidTheme.dimens.largeSpacing),
                    title = stringResource(id = section.title),
                    description = stringResource(id = section.description),
                    icon = section.icon,
                ) {
                    onNavigateToSection(section)
                }
            }

            item {
                Image(
                    modifier = Modifier.fillMaxWidth(),
                    painter = painterResource(id = R.drawable.ill_more),
                    contentDescription = null,
                    contentScale = ContentScale.FillWidth,
                )
            }
        }
    }
}

@GravidPreview
@Composable
fun MoreScreenPreview() {
    GravidTheme {
        MoreScreen(onNavigateToSection = {})
    }
}
