/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2023 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsp.mingraviditet.ui.main.more.navigation

import androidx.navigation.NavController
import androidx.navigation.NavGraphBuilder
import androidx.navigation.NavOptions
import androidx.navigation.NavType
import androidx.navigation.compose.composable
import androidx.navigation.navArgument
import androidx.navigation.navigation
import dk.nsp.mingraviditet.model.misc.SettingSubSectionUrl
import dk.nsp.mingraviditet.ui.main.common.navigation.animatedComposable
import dk.nsp.mingraviditet.ui.main.more.compose.MoreRoute
import dk.nsp.mingraviditet.ui.main.more.settings.SettingsConsentRoute
import dk.nsp.mingraviditet.ui.main.more.settings.SettingsRoute
import dk.nsp.mingraviditet.ui.main.more.settings.SettingsSubPageWebViewRoute
import dk.nsp.mingraviditet.ui.main.more.support.SupportOverviewScreen
import dk.nsp.mingraviditet.ui.main.more.support.faq.FAQRoute
import dk.nsp.mingraviditet.ui.useradmission.onboarding.navigation.navigateToOnboarding

private const val MORE_GRAPH_ROUTE = "more_graph"
const val moreNavigationRoute = "more_route"

const val settingsRoute = "settings_route"

const val faqRoute = "faq_route"
const val supportRoute = "support_route"

const val webViewArg = "pagePath"
const val webViewRoute = "webview_route/{$webViewArg}"

const val consentRoute = "consent_route"

fun NavController.navigateToMore(navOptions: NavOptions? = null) {
    this.navigate(MORE_GRAPH_ROUTE, navOptions)
}

fun NavController.navigateToSupport() {
    this.navigate(supportRoute)
}

fun NavController.navigateToFaq() {
    this.navigate(faqRoute)
}

fun NavController.navigateToSettings() {
    this.navigate(settingsRoute)
}

fun NavController.navigateToWebView(pagePath: String) {
    this.navigate("webview_route/$pagePath")
}

fun NavController.navigateToConsentRoute() {
    this.navigate(consentRoute)
}

fun NavGraphBuilder.moreGraph(
    navController: NavController,
) {
    navigation(
        route = MORE_GRAPH_ROUTE,
        startDestination = moreNavigationRoute,
    ) {
        composable(route = moreNavigationRoute) {
            MoreRoute(
                navigateToSettings = navController::navigateToSettings,
                navigateToSupport = navController::navigateToSupport,
                navigateToWorkInProgress = { navController.navigateToWebView(SettingSubSectionUrl.WorkInProgress.path) },
            )
        }

        animatedComposable(route = faqRoute) {
            FAQRoute(onBackPressed = navController::navigateUp)
        }

        animatedComposable(
            route = webViewRoute,
            arguments = listOf(
                navArgument(webViewArg) { type = NavType.StringType },
            ),
        ) {
            SettingsSubPageWebViewRoute(onBackPressed = navController::navigateUp)
        }

        animatedComposable(route = consentRoute) {
            SettingsConsentRoute(
                onBackPressed = navController::navigateUp,
                onConsentRemoved = {
                    navController.navigateToOnboarding()
                },
            )
        }

        animatedComposable(supportRoute) {
            SupportOverviewScreen(
                onBackPressed = navController::navigateUp,
                navigateToFaq = navController::navigateToFaq,
                navigateToSupport = { navController.navigateToWebView(SettingSubSectionUrl.Support.path) },
            )
        }

        animatedComposable(settingsRoute) {
            SettingsRoute(
                navigateToSubPage = { navController.navigateToWebView(pagePath = it.path) },
                navigateToConsent = { navController.navigateToConsentRoute() },
                onBackPressed = navController::navigateUp,
            )
        }
    }
}
