/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2023 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsp.mingraviditet.ui.main.more.settings

import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.fillMaxHeight
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.rememberScrollState
import androidx.compose.foundation.verticalScroll
import androidx.compose.material3.Scaffold
import androidx.compose.runtime.Composable
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.setValue
import androidx.compose.ui.Modifier
import androidx.compose.ui.layout.onSizeChanged
import androidx.compose.ui.res.stringResource
import androidx.compose.ui.unit.IntSize
import androidx.hilt.navigation.compose.hiltViewModel
import androidx.lifecycle.compose.collectAsStateWithLifecycle
import dk.nsp.mingraviditet.R
import dk.nsp.mingraviditet.gravidcompose.component.GravidButton
import dk.nsp.mingraviditet.gravidcompose.component.GravidTopBar
import dk.nsp.mingraviditet.gravidcompose.theme.GravidTheme
import dk.nsp.mingraviditet.gravidcompose.utils.GravidPreview
import dk.nsp.mingraviditet.ui.common.compose.EmptyStateRow
import dk.nsp.mingraviditet.ui.common.compose.GravidDialog
import dk.nsp.mingraviditet.ui.common.compose.LoadingScreen
import dk.nsp.mingraviditet.ui.main.common.compose.LocalWebView
import dk.nsp.mingraviditet.util.android.WebViewHelper

@Composable
internal fun SettingsConsentRoute(
    onBackPressed: () -> Unit,
    onConsentRemoved: () -> Unit,
    viewModel: ConsentViewModel = hiltViewModel(),
) {
    val content by viewModel.pageResult.collectAsStateWithLifecycle()

    SettingsConsentScreen(
        onBackPressed = onBackPressed,
        onRemoveConsent = {
            viewModel.clearPregnancy(onConsentRemoved)
        },
        state = content,
    )
}

@Composable
internal fun SettingsConsentScreen(
    onBackPressed: () -> Unit,
    onRemoveConsent: () -> Unit,
    state: SettingsPageUiState,
    modifier: Modifier = Modifier,
) {
    var showConfirmDialog by remember {
        mutableStateOf(false)
    }

    Scaffold(
        modifier = modifier.fillMaxSize(),
        containerColor = GravidTheme.colorScheme.background,
        topBar = {
            GravidTopBar(
                title = stringResource(id = R.string.settings_consent_title),
                onNavigationClick = onBackPressed,
            )
        },
    ) { padding ->

        when (state) {
            SettingsPageUiState.Failed -> {
                EmptyStateRow(
                    title = stringResource(id = R.string.common_alert_title_error),
                    description = stringResource(id = R.string.common_alert_message_failed_to_load_data),
                )
            }

            SettingsPageUiState.Loading -> LoadingScreen()

            is SettingsPageUiState.Success -> {
                var bottomButtonSize by remember {
                    mutableStateOf(IntSize.Zero)
                }

                var showDeleteButton by remember {
                    mutableStateOf(false)
                }

                Column(
                    modifier = Modifier
                        .padding(top = padding.calculateTopPadding())
                        .padding(horizontal = GravidTheme.dimens.largeSpacing)
                        .verticalScroll(rememberScrollState()),
                ) {
                    LocalWebView(
                        modifier = Modifier
                            .fillMaxHeight(),
                        content = state.content,
                        webViewType = WebViewHelper.Type.Subpage,
                        onPageFinishedLoading = { showDeleteButton = true },
                    )

                    if (showDeleteButton) {
                        GravidButton(
                            modifier = Modifier
                                .fillMaxWidth()
                                .padding(vertical = GravidTheme.dimens.largeSpacing)
                                .onSizeChanged { bottomButtonSize = it },
                            onClick = { showConfirmDialog = true },
                            text = stringResource(id = R.string.settings_consent_remove_title),
                        )
                    }
                }
            }
        }
    }

    if (showConfirmDialog) {
        GravidDialog(
            title = stringResource(id = R.string.settings_consent_remove_title),
            text = stringResource(id = R.string.settings_consent_remove_description_v1),
            onDismissRequest = { showConfirmDialog = false },
            confirmButtonText = stringResource(id = R.string.common_delete),
            onConfirmButtonClicked = { onRemoveConsent() },
        )
    }
}

@GravidPreview
@Composable
private fun SettingsConsentPreview() {
    GravidTheme {
        SettingsConsentScreen(
            onBackPressed = {},
            onRemoveConsent = {},
            state = SettingsPageUiState.Loading,
        )
    }
}
