/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021-2023 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsp.mingraviditet.ui.main.more.settings

import androidx.lifecycle.SavedStateHandle
import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import dagger.hilt.android.lifecycle.HiltViewModel
import dk.nsp.domain.repository.SettingsPagesRepository
import dk.nsp.mingraviditet.R
import dk.nsp.mingraviditet.model.misc.SettingSubSectionUrl
import dk.nsp.mingraviditet.ui.main.more.navigation.webViewArg
import dk.nsp.shared.network.Failed
import dk.nsp.shared.network.Success
import kotlinx.coroutines.flow.MutableStateFlow
import kotlinx.coroutines.flow.SharingStarted
import kotlinx.coroutines.flow.StateFlow
import kotlinx.coroutines.flow.combine
import kotlinx.coroutines.flow.map
import kotlinx.coroutines.flow.stateIn
import javax.inject.Inject

@HiltViewModel
class SettingsPageViewModel @Inject constructor(
    private val pagesRepository: SettingsPagesRepository,
    savedStateHandle: SavedStateHandle,
) : ViewModel() {

    private var pageArg = savedStateHandle.getStateFlow(webViewArg, "")

    val pagePath = MutableStateFlow<String?>(null)

    private val titleMap = SettingSubSectionUrl.entries.associate { subSection ->
        subSection.path to when (subSection) {
            SettingSubSectionUrl.AboutTheApp -> R.string.settings_about_the_app_title
            SettingSubSectionUrl.WhoIsBehindTheApp -> R.string.settings_who_is_behind_the_app_title
            SettingSubSectionUrl.Consent -> R.string.settings_consent_title
            SettingSubSectionUrl.Support -> R.string.more_support_contact_title
            SettingSubSectionUrl.WorkInProgress -> R.string.more_work_in_progress_title
            SettingSubSectionUrl.DataProtection -> R.string.settings_data_protection_title
        }
    }

    val titleResId = pageArg.map { arg ->
        titleMap[arg]
    }

    val pageResult: StateFlow<SettingsPageUiState> = combine(pageArg, pagePath) { arg, explicit ->
        explicit ?: arg
    }.map { page ->
        when (val pageResult = pagesRepository.getPage(page)) {
            is Failed -> SettingsPageUiState.Failed
            is Success -> SettingsPageUiState.Success(pageResult.value.content)
        }
    }.stateIn(
        scope = viewModelScope,
        started = SharingStarted.WhileSubscribed(5_000),
        initialValue = SettingsPageUiState.Loading,
    )
}

sealed interface SettingsPageUiState {
    data object Failed : SettingsPageUiState
    data class Success(val content: String) : SettingsPageUiState
    data object Loading : SettingsPageUiState
}
