/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2023 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsp.mingraviditet.ui.main.more.settings

import android.content.Intent
import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.PaddingValues
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.rememberScrollState
import androidx.compose.foundation.verticalScroll
import androidx.compose.material3.Scaffold
import androidx.compose.runtime.Composable
import androidx.compose.runtime.collectAsState
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.setValue
import androidx.compose.ui.Modifier
import androidx.compose.ui.res.stringResource
import androidx.compose.ui.text.style.TextAlign
import androidx.hilt.navigation.compose.hiltViewModel
import dk.nsp.mingraviditet.BuildConfig
import dk.nsp.mingraviditet.R
import dk.nsp.mingraviditet.gravidcompose.component.GravidDivider
import dk.nsp.mingraviditet.gravidcompose.component.GravidText
import dk.nsp.mingraviditet.gravidcompose.component.GravidTopBar
import dk.nsp.mingraviditet.gravidcompose.theme.GravidTheme
import dk.nsp.mingraviditet.gravidcompose.utils.GravidIconType
import dk.nsp.mingraviditet.gravidcompose.utils.GravidPreview
import dk.nsp.mingraviditet.model.misc.SettingSubSectionUrl
import dk.nsp.mingraviditet.ui.common.compose.ChevronListItem
import dk.nsp.mingraviditet.ui.common.compose.HeaderWithIcon
import dk.nsp.mingraviditet.ui.common.compose.ToggleListItem
import dk.nsp.mingraviditet.ui.main.common.compose.Section
import dk.nsp.mingraviditet.ui.main.more.settings.compose.BiometricPromptViewModel
import dk.nsp.mingraviditet.ui.main.more.settings.compose.GravidBiometricPrompt

@Composable
fun SettingsRoute(
    settingsViewModel: SettingsViewModel = hiltViewModel(),
    biometricPromptViewModel: BiometricPromptViewModel = hiltViewModel(),
    navigateToConsent: () -> Unit,
    navigateToSubPage: (SettingSubSectionUrl) -> Unit,
    onBackPressed: () -> Unit,
) {
    val allowScreenshots by settingsViewModel.allowScreenshots.collectAsState(initial = false)
    val useBiometrics by settingsViewModel.biometricEnabled.collectAsState(initial = false)

    SettingsScreen(
        allowScreenshots = allowScreenshots,
        onAllowScreenshotsCheckedChanged = {
            settingsViewModel.toggleScreenSecurity(it)
        },
        useBiometrics = useBiometrics,
        onUseBiometricsCheckedChanged = {
            settingsViewModel.toggleBiometrics(it)
        },
        onBackPressed = onBackPressed,
        onNavigateToSubPage = {
            if (it == SettingSubSectionUrl.Consent) navigateToConsent() else navigateToSubPage(it)
        },
        isDeviceSecure = biometricPromptViewModel.isDeviceSecure(),
        biometricsEnrollIntent = biometricPromptViewModel.biometricsEnrollIntent,
    )
}

@Composable
fun SettingsScreen(
    allowScreenshots: Boolean,
    onAllowScreenshotsCheckedChanged: (Boolean) -> Unit,
    useBiometrics: Boolean,
    onUseBiometricsCheckedChanged: (Boolean) -> Unit,
    onBackPressed: () -> Unit,
    onNavigateToSubPage: (SettingSubSectionUrl) -> Unit,
    isDeviceSecure: Boolean,
    biometricsEnrollIntent: Intent,
) {
    var showBiometricPrompt by remember { mutableStateOf(false) }

    Scaffold(
        containerColor = GravidTheme.colorScheme.background,
        topBar = {
            GravidTopBar(
                title = stringResource(id = R.string.more_settings_title),
                onNavigationClick = onBackPressed,
            )
        },

    ) { innerPadding ->
        Column(
            modifier = Modifier
                .verticalScroll(state = rememberScrollState())
                .padding(innerPadding)
                .padding(GravidTheme.dimens.largeSpacing),
            verticalArrangement = Arrangement.spacedBy(GravidTheme.dimens.largeSpacing),
        ) {
            HeaderWithIcon(
                icon = GravidIconType.Drawable(R.drawable.ic_settings),
                title = stringResource(id = R.string.more_settings_title),
                description = stringResource(id = R.string.more_settings_description),
            )

            Section(
                modifier = Modifier.padding(top = GravidTheme.dimens.defaultSpacing),
                innerCardPaddingValues = PaddingValues(vertical = GravidTheme.dimens.defaultSpacing),
                title = stringResource(id = R.string.settings_privacy_title),
            ) {
                ChevronListItem(
                    modifier = Modifier.padding(horizontal = GravidTheme.dimens.largeSpacing),
                    title = stringResource(id = R.string.settings_data_protection_title),
                    subtitle = stringResource(id = R.string.settings_data_protection_description),
                    onClick = { onNavigateToSubPage(SettingSubSectionUrl.DataProtection) },
                )
            }

            Section(title = stringResource(id = R.string.settings_security_title)) {
                ToggleListItem(
                    title = stringResource(id = R.string.settings_protect_app_title),
                    checked = useBiometrics,
                    onCheckedChanged = {
                        showBiometricPrompt = true
                    },
                )

                GravidDivider()

                ToggleListItem(
                    title = stringResource(id = R.string.settings_allow_screenshots_title),
                    checked = allowScreenshots,
                    onCheckedChanged = onAllowScreenshotsCheckedChanged,
                )
            }

            Section(
                title = stringResource(id = R.string.settings_about_the_app_title),
                innerCardPaddingValues = PaddingValues(vertical = GravidTheme.dimens.defaultSpacing),
            ) {
                ChevronListItem(
                    modifier = Modifier.padding(horizontal = GravidTheme.dimens.largeSpacing),
                    title = stringResource(id = R.string.settings_read_about_the_app_title),
                    onClick = { onNavigateToSubPage(SettingSubSectionUrl.AboutTheApp) },
                )

                GravidDivider(modifier = Modifier.padding(horizontal = GravidTheme.dimens.largeSpacing))

                ChevronListItem(
                    modifier = Modifier.padding(horizontal = GravidTheme.dimens.largeSpacing),
                    title = stringResource(id = R.string.settings_who_is_behind_the_app_title),
                    onClick = { onNavigateToSubPage(SettingSubSectionUrl.WhoIsBehindTheApp) },
                )

                GravidDivider(modifier = Modifier.padding(horizontal = GravidTheme.dimens.largeSpacing))

                ChevronListItem(
                    modifier = Modifier.padding(horizontal = GravidTheme.dimens.largeSpacing),
                    title = stringResource(id = R.string.settings_consent_title),
                    onClick = { onNavigateToSubPage(SettingSubSectionUrl.Consent) },
                )
            }

            GravidText(
                modifier = Modifier
                    .fillMaxWidth()
                    .padding(top = GravidTheme.dimens.largeSpacing),
                textAlign = TextAlign.Center,
                text = stringResource(
                    id = R.string.settings_version,
                    BuildConfig.VERSION_NAME,
                    BuildConfig.VERSION_CODE.toString(),
                ),
            )
        }
    }

    GravidBiometricPrompt(
        showBiometricPrompt = showBiometricPrompt,
        title = if (useBiometrics) {
            stringResource(id = R.string.settings_protect_app_prompt_remove_title)
        } else {
            stringResource(id = R.string.settings_protect_app_prompt_add_title)
        },
        onSuccess = {
            showBiometricPrompt = false
            onUseBiometricsCheckedChanged(!useBiometrics)
        },
        onError = {
            showBiometricPrompt = false
        },
        isDeviceSecure = isDeviceSecure,
        biometricsEnrollIntent = biometricsEnrollIntent,
    )
}

@GravidPreview
@Composable
fun SettingsScreenPreview() {
    var allowScreenshots by remember { mutableStateOf(false) }
    GravidTheme {
        SettingsScreen(
            allowScreenshots = allowScreenshots,
            onAllowScreenshotsCheckedChanged = { allowScreenshots = it },
            onBackPressed = {},
            onNavigateToSubPage = {},
            useBiometrics = false,
            onUseBiometricsCheckedChanged = {},
            isDeviceSecure = false,
            biometricsEnrollIntent = Intent(),

        )
    }
}
