/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2023 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsp.mingraviditet.ui.main.more.settings.compose

import android.app.Activity
import android.app.KeyguardManager
import android.content.Intent
import android.os.Build
import android.provider.Settings
import androidx.activity.compose.rememberLauncherForActivityResult
import androidx.activity.result.contract.ActivityResultContracts
import androidx.biometric.BiometricManager
import androidx.biometric.BiometricPrompt
import androidx.compose.runtime.Composable
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.setValue
import androidx.compose.ui.res.stringResource
import androidx.hilt.navigation.compose.hiltViewModel
import androidx.lifecycle.ViewModel
import dagger.hilt.android.lifecycle.HiltViewModel
import dk.nsp.mingraviditet.R
import dk.nsp.mingraviditet.ui.common.compose.GravidDialog
import dk.nsp.mingraviditet.ui.main.common.compose.BiometricPromptDialog
import timber.log.Timber
import javax.inject.Inject

@Composable
fun GravidBiometricPrompt(
    vm: BiometricPromptViewModel = hiltViewModel(),
    showBiometricPrompt: Boolean,
    title: CharSequence,
    onSuccess: (BiometricPrompt.AuthenticationResult) -> Unit,
    onError: (Int) -> Unit,
    isDeviceSecure: Boolean = vm.isDeviceSecure(),
    biometricsEnrollIntent: Intent = vm.biometricsEnrollIntent,
) {
    GravidBiometricPrompt(
        showBiometricPrompt,
        title,
        onSuccess,
        onError,
        isDeviceSecure,
        biometricsEnrollIntent,
    )
}

@Composable
fun GravidBiometricPrompt(
    showBiometricPrompt: Boolean,
    title: CharSequence,
    onSuccess: (BiometricPrompt.AuthenticationResult) -> Unit,
    onError: (Int) -> Unit,
    isDeviceSecure: Boolean,
    biometricsEnrollIntent: Intent,
) {
    var showBiometricPromptDialog by remember(showBiometricPrompt) { mutableStateOf(showBiometricPrompt) }
    var showEnrollDialog by remember { mutableStateOf(false) }

    val resultLauncher = rememberLauncherForActivityResult(
        contract = ActivityResultContracts.StartActivityForResult(),
        onResult = {
            showEnrollDialog = false

            showBiometricPromptDialog = isDeviceSecure

            if (it.resultCode == Activity.RESULT_CANCELED) {
                onError(BiometricPrompt.ERROR_USER_CANCELED)
            }
        },
    )

    if (showBiometricPromptDialog) {
        BiometricPromptDialog(
            title = title,
            onSuccess = onSuccess,
            onError = {
                when (it) {
                    BiometricPrompt.ERROR_NO_DEVICE_CREDENTIAL,
                    BiometricPrompt.ERROR_NO_BIOMETRICS,
                    -> {
                        showBiometricPromptDialog = false
                        showEnrollDialog = true
                    }

                    else -> {
                        onError(it)
                        Timber.w("Biometric prompt returned an error: $it")
                    }
                }
            },
        )
    }

    if (showEnrollDialog) {
        GravidDialog(
            title = stringResource(id = R.string.settings_protect_app_enroll_dialog_title),
            text = stringResource(id = R.string.settings_protect_app_enroll_dialog_description),
            onDismissRequest = { showEnrollDialog = false },
            confirmButtonText = stringResource(id = R.string.settings_protect_app_enroll_dialog_confirm_button_title),
            onConfirmButtonClicked = { resultLauncher.launch(biometricsEnrollIntent) },
            onDismissButtonClicked = { showEnrollDialog = false },
        )
    }
}

@HiltViewModel
class BiometricPromptViewModel @Inject constructor(
    private val keyguardManager: KeyguardManager,
) : ViewModel() {
    fun isDeviceSecure(): Boolean = keyguardManager.isDeviceSecure

    val biometricsEnrollIntent: Intent = when {
        Build.VERSION.SDK_INT >= Build.VERSION_CODES.R -> Intent(Settings.ACTION_BIOMETRIC_ENROLL).apply {
            putExtra(
                Settings.EXTRA_BIOMETRIC_AUTHENTICATORS_ALLOWED,
                BiometricManager.Authenticators.BIOMETRIC_WEAK or BiometricManager.Authenticators.DEVICE_CREDENTIAL,
            )
        }

        Build.VERSION.SDK_INT >= Build.VERSION_CODES.P -> {
            @Suppress("DEPRECATION")
            Intent(Settings.ACTION_FINGERPRINT_ENROLL)
        }

        else -> {
            Intent(Settings.ACTION_SECURITY_SETTINGS)
        }
    }
}
