/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2023 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsp.mingraviditet.ui.main.more.support.faq

import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.PaddingValues
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.lazy.LazyColumn
import androidx.compose.foundation.lazy.items
import androidx.compose.material3.Scaffold
import androidx.compose.runtime.Composable
import androidx.compose.runtime.getValue
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.res.stringResource
import androidx.compose.ui.tooling.preview.Preview
import androidx.hilt.navigation.compose.hiltViewModel
import androidx.lifecycle.compose.collectAsStateWithLifecycle
import dk.nsp.domain.faq.FAQItem
import dk.nsp.mingraviditet.R
import dk.nsp.mingraviditet.gravidcompose.component.GravidTopBar
import dk.nsp.mingraviditet.gravidcompose.theme.GravidTheme
import dk.nsp.mingraviditet.gravidcompose.utils.GravidPreview
import dk.nsp.mingraviditet.ui.common.compose.EmptyStateRow
import dk.nsp.mingraviditet.ui.common.compose.ErrorStateRow
import dk.nsp.mingraviditet.ui.common.compose.ExpandableCard
import dk.nsp.mingraviditet.ui.common.compose.LoadingScreen
import dk.nsp.shared.result.Result
import java.time.LocalDateTime

@Composable
fun FAQRoute(
    onBackPressed: () -> Unit,
    faqViewModel: FAQViewModel = hiltViewModel(),
) {
    val faqResult by faqViewModel.faqs.collectAsStateWithLifecycle(initialValue = Result.Loading)

    FAQScreen(
        onBackPressed = onBackPressed,
        faqResult = faqResult,
    )
}

@Composable
fun FAQScreen(
    onBackPressed: () -> Unit,
    faqResult: Result<List<FAQItem>>,
) {
    Scaffold(
        containerColor = GravidTheme.colorScheme.background,
        topBar = {
            GravidTopBar(
                title = stringResource(id = R.string.more_support_faq_title),
                onNavigationClick = onBackPressed,
            )
        },
    ) { innerPadding ->
        when (faqResult) {
            is Result.Error -> {
                Box(
                    modifier = Modifier
                        .padding(innerPadding)
                        .fillMaxSize(),
                    contentAlignment = Alignment.Center,
                ) {
                    ErrorStateRow()
                }
            }

            Result.Loading -> LoadingScreen()

            is Result.Success -> {
                if (faqResult.data.isNotEmpty()) {
                    LazyColumn(
                        modifier = Modifier
                            .fillMaxWidth()
                            .padding(top = innerPadding.calculateTopPadding()),
                        contentPadding = PaddingValues(
                            horizontal = GravidTheme.dimens.largeSpacing,
                            vertical = GravidTheme.dimens.largerSpacing,
                        ),
                        verticalArrangement = Arrangement.spacedBy(GravidTheme.dimens.largeSpacing),
                    ) {
                        items(faqResult.data) { faq ->
                            ExpandableCard(
                                modifier = Modifier,
                                title = faq.title,
                                description = faq.text,
                                lastUpdated = faq.updated,
                            )
                        }
                    }
                } else {
                    Box(
                        modifier = Modifier
                            .padding(innerPadding)
                            .fillMaxSize(),
                        contentAlignment = Alignment.Center,
                    ) {
                        EmptyStateRow(
                            title = stringResource(id = R.string.faq_empty_state_title),
                            description = stringResource(id = R.string.faq_empty_state_description),
                        )
                    }
                }
            }
        }
    }
}

@GravidPreview
@Composable
private fun FAQScreenPreview() {
    GravidTheme {
        FAQScreen(
            onBackPressed = {},
            faqResult = Result.Success(
                (0..10).map {
                    FAQItem(
                        title = "Hvor er mit spørgeskema?",
                        text = "I forbindelse med bla bla bla",
                        updated = LocalDateTime.now(),
                    )
                },
            ),
        )
    }
}

@Preview
@Composable
private fun FAQScreenLoadingPreview() {
    GravidTheme {
        FAQScreen(
            onBackPressed = {},
            faqResult = Result.Loading,
        )
    }
}

@Preview
@Composable
private fun FAQScreenErrorPreview() {
    GravidTheme {
        FAQScreen(
            onBackPressed = {},
            faqResult = Result.Error(),
        )
    }
}

@Preview
@Composable
private fun FAQScreenEmptyStatePreview() {
    GravidTheme {
        FAQScreen(
            onBackPressed = {},
            faqResult = Result.Success(listOf()),
        )
    }
}
