/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2020-2023 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsp.mingraviditet.ui.main.overview

import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import dagger.hilt.android.lifecycle.HiltViewModel
import dk.nsp.domain.articles.OverviewArticles
import dk.nsp.domain.repository.ArticlesRepository
import dk.nsp.domain.repository.PregnancyRepository
import dk.nsp.mingraviditet.ui.main.overview.widget.OverviewPregnancyState
import dk.nsp.shared.network.asResult
import dk.nsp.shared.result.Result
import kotlinx.coroutines.flow.SharingStarted
import kotlinx.coroutines.flow.StateFlow
import kotlinx.coroutines.flow.flatMapLatest
import kotlinx.coroutines.flow.flow
import kotlinx.coroutines.flow.map
import kotlinx.coroutines.flow.stateIn
import kotlinx.coroutines.launch
import java.time.LocalDate
import java.time.temporal.ChronoUnit
import javax.inject.Inject

@HiltViewModel
class OverviewViewModel @Inject constructor(
    private val articlesRepository: ArticlesRepository,
    private val pregnancyRepository: PregnancyRepository,
) : ViewModel() {

    private val pregnancyInfo = pregnancyRepository.pregnancyInfo
        .stateIn(
            scope = viewModelScope,
            started = SharingStarted.Eagerly,
            initialValue = null,
        )

    val articles: StateFlow<Result<OverviewArticles>> = pregnancyInfo
        .flatMapLatest { pregnancyInfo ->
            flow {
                emit(Result.Loading)

                val overviewArticles = if (pregnancyInfo?.dateOfBirth != null) {
                    val weeksSinceBirth = ChronoUnit.WEEKS.between(pregnancyInfo.dateOfBirth, LocalDate.now()).toInt() + 1
                    articlesRepository.getAfterBirthArticles(weeksSinceBirth)
                } else {
                    articlesRepository.getOverviewArticles(pregnancyInfo?.pregnancyWeek)
                }

                emit(overviewArticles.asResult())
            }
        }
        .stateIn(
            scope = viewModelScope,
            started = SharingStarted.Lazily,
            initialValue = Result.Loading,
        )

    val overviewWidgetState = pregnancyInfo.map { data ->
        when {
            data == null -> OverviewPregnancyState.None
            data.dateOfBirth != null -> {
                OverviewPregnancyState.GaveBirth(
                    dateOfBirth = data.dateOfBirth!!,
                )
            }

            else -> {
                OverviewPregnancyState.WaitingForBirth(
                    monthsIntoPregnancy = data.monthsIntoPregnancy,
                    expectedDueDate = data.expectedDueDate,
                    week = data.pregnancyWeek,
                    gestationAge = data.gestationAge,
                    trimester = data.trimester,
                    numberOfFetuses = null,
                )
            }
        }
    }.stateIn(
        scope = viewModelScope,
        started = SharingStarted.Lazily,
        initialValue = OverviewPregnancyState.None,
    )

    fun clearPregnancy() = viewModelScope.launch {
        pregnancyRepository.clearPregnancy()
    }
}
