/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2023 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsp.mingraviditet.ui.main.overview.compose

import androidx.compose.foundation.layout.PaddingValues
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.lazy.LazyColumn
import androidx.compose.foundation.lazy.items
import androidx.compose.material3.IconButton
import androidx.compose.material3.Scaffold
import androidx.compose.material3.Text
import androidx.compose.material3.TopAppBarDefaults
import androidx.compose.runtime.Composable
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.setValue
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.draw.paint
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.input.nestedscroll.nestedScroll
import androidx.compose.ui.layout.ContentScale
import androidx.compose.ui.res.painterResource
import androidx.compose.ui.res.stringResource
import androidx.compose.ui.unit.dp
import androidx.hilt.navigation.compose.hiltViewModel
import androidx.lifecycle.compose.collectAsStateWithLifecycle
import dk.nsp.domain.articles.OverviewArticles
import dk.nsp.domain.pregnancymodels.GestationAge
import dk.nsp.mingraviditet.R
import dk.nsp.mingraviditet.extensions.toMediumFormat
import dk.nsp.mingraviditet.gravidcompose.component.GravidExpandableTopBar
import dk.nsp.mingraviditet.gravidcompose.component.GravidIcon
import dk.nsp.mingraviditet.gravidcompose.component.GravidOutlinedButton
import dk.nsp.mingraviditet.gravidcompose.component.GravidText
import dk.nsp.mingraviditet.gravidcompose.theme.GravidTheme
import dk.nsp.mingraviditet.gravidcompose.utils.GravidIconType
import dk.nsp.mingraviditet.gravidcompose.utils.GravidPreview
import dk.nsp.mingraviditet.gravidcompose.utils.rememberForeverLazyListState
import dk.nsp.mingraviditet.gravidcompose.utils.rememberForeverTopAppBarState
import dk.nsp.mingraviditet.model.data.Article
import dk.nsp.mingraviditet.ui.common.compose.EmptyStateRow
import dk.nsp.mingraviditet.ui.common.compose.ErrorStateRow
import dk.nsp.mingraviditet.ui.common.compose.GravidDialog
import dk.nsp.mingraviditet.ui.common.compose.loadingRow
import dk.nsp.mingraviditet.ui.main.common.compose.GravidCardWithIcon
import dk.nsp.mingraviditet.ui.main.common.compose.InnerCircularCard
import dk.nsp.mingraviditet.ui.main.overview.OverviewViewModel
import dk.nsp.mingraviditet.ui.main.overview.compose.widget.FetusProgressWidget
import dk.nsp.mingraviditet.ui.main.overview.compose.widget.OverviewPregnancyInformationCardsRow
import dk.nsp.mingraviditet.ui.main.overview.datechooser.model.DateChooserMethod
import dk.nsp.mingraviditet.ui.main.overview.widget.OverviewPregnancyState
import dk.nsp.models.ArticleSearchResult
import dk.nsp.shared.result.Result
import java.time.LocalDate
import java.time.LocalDateTime

@Composable
fun OverviewRoute(
    overviewViewModel: OverviewViewModel = hiltViewModel(),
    hasShownProgressAnimation: Boolean = false,
    onNavigateToKnowledge: () -> Unit,
    onNavigateToArticleDetails: (articleId: Int, sectionId: Int?) -> Unit,
    onOpenDateChooserDialog: (DateChooserMethod) -> Unit,
) {
    val overviewArticles by overviewViewModel.articles.collectAsStateWithLifecycle()
    val overviewWidgetState by overviewViewModel.overviewWidgetState.collectAsStateWithLifecycle()

    OverviewScreen(
        state = overviewWidgetState,
        overviewArticles = overviewArticles,
        onNavigateToKnowledge = onNavigateToKnowledge,
        onNavigateToArticleDetails = onNavigateToArticleDetails,
        hasShownProgressAnimation = hasShownProgressAnimation,
        openCalendarDialog = onOpenDateChooserDialog,
        clearCurrentPregnancy = {
            overviewViewModel.clearPregnancy()
            onOpenDateChooserDialog(DateChooserMethod.ExpectedDateOfDelivery)
        },
    )
}

@Composable
fun OverviewScreen(
    state: OverviewPregnancyState,
    overviewArticles: Result<OverviewArticles>,
    onNavigateToKnowledge: () -> Unit,
    onNavigateToArticleDetails: (articleId: Int, sectionId: Int?) -> Unit,
    openCalendarDialog: (DateChooserMethod) -> Unit,
    clearCurrentPregnancy: () -> Unit,
    hasShownProgressAnimation: Boolean = false,
) {
    var internalHasShowProgress by remember {
        mutableStateOf(hasShownProgressAnimation)
    }

    var showClearDialog by remember {
        mutableStateOf(false)
    }

    val lazyState = rememberForeverLazyListState(key = "OverviewLazy")
    val barState = rememberForeverTopAppBarState("Overview")
    val scrollBehavior = TopAppBarDefaults.exitUntilCollapsedScrollBehavior(
        state = barState,
        canScroll = { true },
    )
    val scrollConnection = scrollBehavior.nestedScrollConnection

    Scaffold(
        modifier = Modifier
            .fillMaxSize()
            .nestedScroll(scrollConnection)
            .paint(
                painter = painterResource(id = R.drawable.ill_overview_circles),
                alignment = Alignment.TopCenter,
                contentScale = ContentScale.FillWidth,
            ),
        containerColor = Color.Transparent,
        topBar = {
            GravidExpandableTopBar(
                title = { Text(text = stringResource(id = R.string.bottom_navigation_title_overview)) },
                subtitle = { Text(text = stringResource(id = R.string.overview_welcome_message_unauthenticated)) },
                scrollBehavior = scrollBehavior,
                expandedBackgroundTransparent = true,
                actions = {
                    if (state is OverviewPregnancyState.GaveBirth) {
                        InnerCircularCard(
                            containerColor = GravidTheme.colorScheme.surface,
                            modifier = Modifier
                                .padding(
                                    end = GravidTheme.dimens.largeSpacing,
                                    top = GravidTheme.dimens.largeSpacing * (1f - scrollBehavior.state.collapsedFraction),
                                ),
                        ) {
                            IconButton(onClick = { showClearDialog = true }) {
                                GravidIcon(
                                    painterResource(id = R.drawable.ic_calendar),
                                    tint = GravidTheme.colorScheme.tertiary,
                                )
                            }
                        }
                    }
                },
            )
        },
    ) { innerPadding ->

        LazyColumn(
            state = lazyState,
            modifier = Modifier
                .fillMaxWidth()
                .padding(top = innerPadding.calculateTopPadding()),
            horizontalAlignment = Alignment.CenterHorizontally,
            contentPadding = PaddingValues(
                vertical = GravidTheme.dimens.largeSpacing,
                horizontal = GravidTheme.dimens.largeSpacing,
            ),
        ) {
            item {
                FetusProgressWidget(
                    modifier = Modifier.fillMaxWidth(),
                    size = 250.dp,
                    monthsIntoPregnancy = state.monthsIntoPregnancy,
                    animateProgress = !internalHasShowProgress,
                    onProgressAnimationFinished = { internalHasShowProgress = true },
                )
            }

            item {
                when (state) {
                    is OverviewPregnancyState.GaveBirth -> {
                        GravidText(
                            modifier = Modifier.padding(top = GravidTheme.dimens.defaultSpacing),
                            text = stringResource(id = R.string.overview_date_of_birth),
                            style = GravidTheme.typography.Header4,
                        )

                        GravidOutlinedButton(
                            modifier = Modifier.padding(top = GravidTheme.dimens.largeSpacing),
                            trailingIcon = R.drawable.ic_edit,
                            contentPadding = PaddingValues(
                                horizontal = GravidTheme.dimens.largeSpacing,
                                vertical = 13.dp,
                            ),
                            onClick = {
                                openCalendarDialog(DateChooserMethod.DateOfBirth)
                            },
                            text = state.dateOfBirth.toMediumFormat(),
                        )
                    }

                    is OverviewPregnancyState.WaitingForBirth -> {
                        Row(verticalAlignment = Alignment.CenterVertically) {
                            GravidText(
                                text = stringResource(id = R.string.overview_your_pregnancy_due_date, state.expectedDueDate.toMediumFormat()),
                                style = GravidTheme.typography.Header4,
                            )

                            IconButton(
                                onClick = {
                                    openCalendarDialog(DateChooserMethod.ExpectedDateOfDelivery)
                                },
                            ) {
                                GravidIcon(
                                    painter = painterResource(id = R.drawable.ic_edit),
                                    tint = GravidTheme.colorScheme.tertiary,
                                )
                            }
                        }

                        val dateOfBirthComposable: (@Composable () -> Unit)? = if (state.week >= 36) {
                            {
                                GravidOutlinedButton(
                                    contentPadding = PaddingValues(
                                        horizontal = GravidTheme.dimens.largeSpacing,
                                        vertical = 13.dp,
                                    ),
                                    onClick = {
                                        openCalendarDialog(DateChooserMethod.DateOfBirth)
                                    },
                                    text = stringResource(id = R.string.overview_did_give_birth_button_title),
                                )
                            }
                        } else {
                            null
                        }

                        OverviewPregnancyInformationCardsRow(
                            modifier = Modifier.padding(top = GravidTheme.dimens.largeSpacing),
                            pregnancyWeek = state.week,
                            gestationAge = state.gestationAge,
                            trimester = state.trimester,
                            trailingComponent = dateOfBirthComposable,
                        )
                    }

                    is OverviewPregnancyState.None -> {
                        GravidText(
                            modifier = Modifier.padding(top = GravidTheme.dimens.defaultSpacing),
                            text = stringResource(id = R.string.overview_your_pregnancy_title),
                            style = GravidTheme.typography.Header4,
                        )

                        GravidOutlinedButton(
                            modifier = Modifier.padding(top = GravidTheme.dimens.largeSpacing),
                            onClick = {
                                openCalendarDialog(DateChooserMethod.ExpectedDateOfDelivery)
                            },
                            text = stringResource(id = R.string.overview_your_pregnancy_select_due_date),
                            trailingIcon = R.drawable.ic_calendar,
                        )
                    }
                }
            }

            when (overviewArticles) {
                is Result.Error -> item { ErrorStateRow() }
                Result.Loading -> loadingRow()

                is Result.Success -> {
                    if (overviewArticles.data.currentWeek.isNotEmpty()) {
                        item {
                            ArticleHeader(
                                modifier = Modifier.padding(top = GravidTheme.dimens.largerSpacing),
                                title = stringResource(id = R.string.overview_relevant_right_now_title),
                                trailingTitle = stringResource(id = R.string.overview_find_more_knowledge_button),
                                trailingOnClick = onNavigateToKnowledge,
                            )
                        }

                        items(overviewArticles.data.currentWeek) { article ->
                            ArticleCard(
                                modifier = Modifier.padding(bottom = GravidTheme.dimens.defaultSpacing),
                                article = article,
                                onClick = {
                                    onNavigateToArticleDetails(article.article.id, article.articleSectionId)
                                },
                            )
                        }
                    } else {
                        item {
                            EmptyStateRow(
                                title = stringResource(id = R.string.overview_empty_state_title),
                                description = stringResource(id = R.string.overview_empty_state_text),
                            )
                        }
                    }

                    if (overviewArticles.data.nextWeek?.isNotEmpty() == true) {
                        item {
                            ArticleHeader(
                                modifier = Modifier.padding(top = GravidTheme.dimens.defaultSpacing),
                                title = stringResource(id = R.string.overview_relevant_next_week_title),
                            )
                        }

                        items(overviewArticles.data.nextWeek ?: emptyList()) { article ->
                            ArticleCard(
                                modifier = Modifier.padding(bottom = GravidTheme.dimens.defaultSpacing),
                                article = article,
                                onClick = {
                                    onNavigateToArticleDetails(article.article.id, article.articleSectionId)
                                },
                            )
                        }
                    }
                }
            }
        }
    }

    if (showClearDialog) {
        GravidDialog(
            title = stringResource(id = R.string.overview_start_new_pregnancy_button_title),
            text = stringResource(id = R.string.overview_start_new_pregnancy_alert_description),
            onDismissRequest = { showClearDialog = false },
            confirmButtonText = stringResource(id = R.string.common_delete),
            onConfirmButtonClicked = {
                clearCurrentPregnancy()
                showClearDialog = false
            },
            onDismissButtonClicked = { showClearDialog = false },
        )
    }
}

@Composable
private fun ArticleCard(
    modifier: Modifier = Modifier,
    article: ArticleSearchResult,
    onClick: () -> Unit,
) {
    GravidCardWithIcon(
        title = article.title,
        description = article.description,
        tickerText = article.subtitle,
        icon = GravidIconType.Url(article.iconUrl),
        onClick = onClick,
        modifier = modifier,
    )
}

@GravidPreview
@Composable
fun OverviewScreenPreview() {
    GravidTheme {
        OverviewScreen(
            overviewArticles = Result.Success(
                OverviewArticles(
                    currentWeek = listOf(
                        ArticleSearchResult(
                            title = "38. uge (37+0 - 37+6)",
                            description = "Ved fulde 37 uger (37+0) er barnet omkring 47 cm langt og vejer ca. 3000 gram.",
                            subtitle = "Uge for uge - 3. trimester",
                            iconUrl = null,
                            article = Article(0, "", 1, LocalDateTime.now(), "", null, emptyList()),
                            searchOrder = 1,
                            articleSectionId = null,
                        ),
                    ),
                    nextWeek = listOf(
                        ArticleSearchResult(
                            title = "39. uge (38+0 - 38+6)",
                            description = "Ved fulde 38 uger (38+0) er barnet omkring 48 cm langt og vejer knap  3200 gram.",
                            subtitle = "Uge for uge - 3. trimester",
                            iconUrl = null,
                            article = Article(0, "", 1, LocalDateTime.now(), "", null, emptyList()),
                            searchOrder = 1,
                            articleSectionId = null,
                        ),
                    ),
                ),
            ),
            onNavigateToKnowledge = { /*TODO*/ },
            onNavigateToArticleDetails = { _, _ -> },
            clearCurrentPregnancy = {},
            hasShownProgressAnimation = true,
            state = OverviewPregnancyState.WaitingForBirth(
                expectedDueDate = LocalDate.now().plusMonths(1),
                week = 38,
                trimester = 3,
                gestationAge = GestationAge(35, 5),
                monthsIntoPregnancy = 8.4,
            ),
            openCalendarDialog = {},
        )
    }
}

@GravidPreview
@Composable
fun NoEDDOverviewScreenPreview() {
    GravidTheme {
        OverviewScreen(
            onNavigateToKnowledge = { },
            onNavigateToArticleDetails = { _, _ -> },
            clearCurrentPregnancy = {},
            overviewArticles = Result.Loading,
            hasShownProgressAnimation = true,
            state = OverviewPregnancyState.None,
            openCalendarDialog = {},
        )
    }
}

@GravidPreview
@Composable
fun GaveBirthOverviewScreenPreview() {
    GravidTheme {
        OverviewScreen(
            state = OverviewPregnancyState.GaveBirth(dateOfBirth = LocalDate.now()),
            overviewArticles = Result.Success(OverviewArticles(emptyList(), emptyList())),
            onNavigateToKnowledge = { /*TODO*/ },
            onNavigateToArticleDetails = { _, _ -> },
            hasShownProgressAnimation = true,
            clearCurrentPregnancy = {},
            openCalendarDialog = {},
        )
    }
}
