/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2023 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsp.mingraviditet.ui.main.overview.compose.widget

import androidx.annotation.DrawableRes
import androidx.compose.animation.Crossfade
import androidx.compose.animation.core.LinearEasing
import androidx.compose.animation.core.RepeatMode
import androidx.compose.animation.core.animateFloat
import androidx.compose.animation.core.animateFloatAsState
import androidx.compose.animation.core.infiniteRepeatable
import androidx.compose.animation.core.rememberInfiniteTransition
import androidx.compose.animation.core.tween
import androidx.compose.foundation.Image
import androidx.compose.foundation.background
import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.aspectRatio
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.size
import androidx.compose.foundation.shape.CircleShape
import androidx.compose.runtime.Composable
import androidx.compose.runtime.LaunchedEffect
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableFloatStateOf
import androidx.compose.runtime.mutableIntStateOf
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.setValue
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.draw.clip
import androidx.compose.ui.graphics.Brush
import androidx.compose.ui.res.painterResource
import androidx.compose.ui.tooling.preview.Preview
import androidx.compose.ui.unit.Dp
import androidx.compose.ui.unit.dp
import dk.nsp.mingraviditet.R
import dk.nsp.mingraviditet.gravidcompose.theme.GravidTheme
import dk.nsp.mingraviditet.gravidcompose.utils.innerShadow

@Composable
fun FetusProgressWidget(
    modifier: Modifier = Modifier,
    animateProgress: Boolean = false,
    size: Dp = 200.dp,
    monthsIntoPregnancy: Double? = null,
    onProgressAnimationFinished: (() -> Unit)? = null,
) {
    Crossfade(targetState = monthsIntoPregnancy != null, label = "Crossfade") { hasDueDate ->
        if (hasDueDate) {
            FetusWidgetDots(
                modifier = Modifier
                    .size(size + 30.dp)
                    .clip(CircleShape)
                    .then(modifier),
            ) {
                OuterFetusWidgetCircle(
                    size = size,
                ) {
                    ProgressFetusWidget(
                        outerSize = size,
                        monthsIntoPregnancy = monthsIntoPregnancy ?: 0.0,
                        animate = animateProgress,
                        onProgressAnimationFinished = onProgressAnimationFinished,
                    )
                }
            }
        } else {
            OuterFetusWidgetCircle(
                modifier = Modifier
                    .padding(10.dp)
                    .size(size)
                    .then(modifier),
                size = size,
            ) {
                EmptyFetusWidget()
            }
        }
    }
}

@Composable
fun OuterFetusWidgetCircle(
    modifier: Modifier = Modifier,
    size: Dp = 200.dp,
    content: @Composable () -> Unit,
) {
    Box(
        modifier = modifier
            .size(size)
            .clip(CircleShape)
            .background(color = GravidTheme.colorScheme.background)
            .innerShadow(
                color = GravidTheme.gravidColorSystem.overviewWidget.innerDropShadow,
                blur = size / 33,
                cornersRadius = size / 2,
                spread = size / 40,
            )
            .aspectRatio(ratio = 1f),
        contentAlignment = Alignment.Center,
    ) {
        content()
    }
}

@Composable
fun EmptyFetusWidget() {
    Image(
        modifier = Modifier.fillMaxSize(),
        painter = painterResource(
            id = R.drawable.ill_dashboard_woman_square,
        ),
        contentDescription = null,
    )
}

@Composable
fun ProgressFetusWidget(
    outerSize: Dp,
    monthsIntoPregnancy: Double,
    animate: Boolean = false,
    onProgressAnimationFinished: (() -> Unit)? = null,
) {
    @DrawableRes
    val fetusDrawable by remember(monthsIntoPregnancy) {
        mutableIntStateOf(
            when (monthsIntoPregnancy) {
                in (9.0..Double.MAX_VALUE) -> R.drawable.ic_9_0_months
                in (8.5..9.0) -> R.drawable.ic_8_5_months
                in (8.0..8.5) -> R.drawable.ic_8_0_months
                in (7.5..8.0) -> R.drawable.ic_7_5_months
                in (7.0..7.5) -> R.drawable.ic_7_0_months
                in (6.5..7.0) -> R.drawable.ic_6_5_months
                in (6.0..6.5) -> R.drawable.ic_6_0_months
                in (5.5..6.0) -> R.drawable.ic_5_5_months
                in (5.0..5.5) -> R.drawable.ic_5_0_months
                in (4.5..5.0) -> R.drawable.ic_4_5_months
                in (4.0..4.5) -> R.drawable.ic_4_0_months
                in (3.5..4.0) -> R.drawable.ic_3_5_months
                in (3.0..3.5) -> R.drawable.ic_3_0_months
                in (2.5..3.0) -> R.drawable.ic_2_5_months
                in (2.0..2.5) -> R.drawable.ic_2_0_months
                in (1.5..2.0) -> R.drawable.ic_1_5_months
                in (1.0..1.5) -> R.drawable.ic_1_0_month
                in (0.0..1.0) -> R.drawable.ic_0_5_month
                else -> throw IllegalStateException("Months into pregnancy cannot be $monthsIntoPregnancy")
            },
        )
    }

    val targetMonths = 9f

    var progress by remember {
        if (animate) {
            mutableFloatStateOf(0f)
        } else {
            mutableFloatStateOf((monthsIntoPregnancy / targetMonths).toFloat())
        }
    }

    LaunchedEffect(monthsIntoPregnancy) {
        progress = (monthsIntoPregnancy / targetMonths).toFloat()
    }

    val animatedProgress by animateFloatAsState(
        targetValue = progress,
        animationSpec = tween(
            durationMillis = 500,
            delayMillis = 250,
        ),
        finishedListener = {
            onProgressAnimationFinished?.invoke()
        },
        label = "progress bar progression",
    )

    val paddingToOuterRing by remember { mutableStateOf(outerSize / 10) }
    val progressStrokePaddingToBorders by remember { mutableStateOf(paddingToOuterRing / 5) }
    val progressStrokeWidth by remember { mutableStateOf(paddingToOuterRing - progressStrokePaddingToBorders * 2) }

    Box(
        modifier = Modifier
            .background(GravidTheme.colorScheme.background)
            .padding(paddingToOuterRing)
            .fillMaxSize()
            .clip(CircleShape)
            .background(
                brush = Brush.verticalGradient(
                    0.0f to GravidTheme.gravidColorSystem.overviewWidget.wombTop,
                    1.0f to GravidTheme.gravidColorSystem.overviewWidget.wombBottom,
                ),
            )
            .innerShadow(
                color = GravidTheme.gravidColorSystem.overviewWidget.innerDropShadow,
                blur = outerSize / 33,
                cornersRadius = outerSize / 2,
                spread = outerSize / 40,
            ),
        contentAlignment = Alignment.Center,
    ) {
        NoiseAnimatedImage(
            modifier = Modifier.fillMaxSize(),
            imageDrawable = fetusDrawable,
        )
    }

    CircularPregnancyProgressIndicator(
        progress = if (animate) animatedProgress else progress,
        strokeWidth = progressStrokeWidth,
        modifier = Modifier
            .padding(progressStrokePaddingToBorders)
            .fillMaxSize(),
    )
}

@Preview
@Composable
fun EmptyFetusWidgetPreview() {
    GravidTheme {
        FetusProgressWidget()
    }
}

@Preview(heightDp = 2000, showBackground = true)
@Composable
fun FetusProgressWidgetPreview() {
    val infiniteTransition = rememberInfiniteTransition(label = "Infinite")
    val monthAnimation by infiniteTransition.animateFloat(
        initialValue = 0.5f,
        targetValue = 9.0f,
        animationSpec = infiniteRepeatable(
            animation = tween(durationMillis = 10000, easing = LinearEasing),
            repeatMode = RepeatMode.Restart,
        ),
        label = "Speed baby progress",
    )

    GravidTheme {
        Column(
            horizontalAlignment = Alignment.CenterHorizontally,
            verticalArrangement = Arrangement.spacedBy(16.dp),
        ) {
            FetusProgressWidget(
                animateProgress = true,
                monthsIntoPregnancy = monthAnimation.toDouble(),
            )

            generateSequence(0.6) { it + 0.5 }
                .takeWhile { it < 9.0 }
                .forEach {
                    FetusProgressWidget(
                        size = 100.dp,
                        monthsIntoPregnancy = it,
                    )
                }
        }
    }
}
