/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2023 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsp.mingraviditet.ui.main.overview.datechooser

import androidx.annotation.StringRes
import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.ColumnScope
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.Spacer
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.shape.RoundedCornerShape
import androidx.compose.material3.Scaffold
import androidx.compose.runtime.Composable
import androidx.compose.runtime.LaunchedEffect
import androidx.compose.runtime.collectAsState
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.setValue
import androidx.compose.ui.Modifier
import androidx.compose.ui.draw.clip
import androidx.compose.ui.res.stringResource
import androidx.hilt.navigation.compose.hiltViewModel
import dk.nsp.mingraviditet.R
import dk.nsp.mingraviditet.gravidcompose.component.GravidButton
import dk.nsp.mingraviditet.gravidcompose.component.GravidDialogTopAppBar
import dk.nsp.mingraviditet.gravidcompose.component.GravidText
import dk.nsp.mingraviditet.gravidcompose.component.GravidTextButton
import dk.nsp.mingraviditet.gravidcompose.theme.GravidTheme
import dk.nsp.mingraviditet.gravidcompose.utils.GravidPreview
import dk.nsp.mingraviditet.ui.common.compose.GravidDialog
import dk.nsp.mingraviditet.ui.main.overview.datechooser.calendar.Calendar
import dk.nsp.mingraviditet.ui.main.overview.datechooser.calendar.model.CalendarState
import dk.nsp.mingraviditet.ui.main.overview.datechooser.model.DateChooserData
import java.time.LocalDate

@Composable
fun DateChooserDialog(
    onDismissRequest: () -> Unit,
    viewModel: DateChooserViewModel = hiltViewModel(),
    calendarViewModel: CalendarViewModel = hiltViewModel(),
) {
    val uiState by viewModel.dateChooserDataFlow.collectAsState(initial = null)

    val calendarState = calendarViewModel.apply {
        uiState?.let(calendarViewModel::setupWithDateChooserData)
    }.calendarState

    // Select initial date once per uiState (and reset it when changing between modes)
    LaunchedEffect(uiState) {
        calendarState.setSelectedDay(uiState?.initialDate ?: LocalDate.now())
    }

    val dateUpdatedCallback by viewModel.onDateUpdated.collectAsState(initial = null)
    LaunchedEffect(dateUpdatedCallback) {
        if (dateUpdatedCallback != null) {
            onDismissRequest()
        }
    }

    val title = when (uiState) {
        is DateChooserData.CalculateExpectedDateOfDelivery -> R.string.overview_pregnancy_calculate_expected_delivery_date_title
        is DateChooserData.DateOfBirth -> R.string.overview_pick_date_of_birth_title
        is DateChooserData.ExpectedDateOfDelivery -> if (uiState?.initialDate != null) R.string.overview_pregnancy_select_new_expected_delivery_date_title else R.string.overview_pregnancy_select_expected_delivery_date_title
        else -> R.string.common_missing_data
    }

    var showClearDialog by remember {
        mutableStateOf(false)
    }

    Scaffold(
        modifier = Modifier
            .fillMaxSize()
            .padding(top = GravidTheme.dimens.largerSpacing)
            .clip(RoundedCornerShape(topStart = GravidTheme.dimens.largeCardCornerRadius, topEnd = GravidTheme.dimens.largeCardCornerRadius)),
        topBar = {
            GravidDialogTopAppBar(title = stringResource(id = title), onNavigationClick = onDismissRequest)
        },
        containerColor = GravidTheme.colorScheme.background,
    ) { padding ->

        Box(
            modifier = Modifier
                .padding(padding)
                .padding(vertical = GravidTheme.dimens.largeSpacing),
        ) {
            when (uiState) {
                is DateChooserData.CalculateExpectedDateOfDelivery -> {
                    DefaultDateChooserScreen(
                        description = stringResource(id = R.string.overview_select_calculated_expected_due_date_description),
                        calendarState = calendarState,
                        onUpdateDate = calendarViewModel::onDaySelected,
                        onConfirmDate = {
                            calendarState.calendarUiState.value.selectedDate?.let {
                                viewModel.setDate(it)
                            }
                        },
                        confirmButtonText = stringResource(id = R.string.overview_pregnancy_calculate_expected_delivery_date_button_title),
                    )
                }

                is DateChooserData.DateOfBirth -> {
                    DefaultDateChooserScreen(
                        description = stringResource(id = R.string.overview_pick_date_of_birth_description),
                        calendarState = calendarState,
                        onUpdateDate = calendarViewModel::onDaySelected,
                        onConfirmDate = {
                            calendarState.calendarUiState.value.selectedDate?.let {
                                viewModel.setDate(it)
                            }
                        },
                        confirmButtonText = stringResource(id = R.string.overview_pick_date_of_birth_button),
                        bottomContent = {
                            if (calendarState.initialSelectedDate != null) {
                                GravidButton(
                                    containerColor = GravidTheme.colorScheme.onPrimaryContainer,
                                    textColor = GravidTheme.colorScheme.primaryContainer,
                                    modifier = Modifier
                                        .fillMaxWidth()
                                        .padding(horizontal = GravidTheme.dimens.largeSpacing),
                                    onClick = { showClearDialog = true },
                                    text = stringResource(id = R.string.overview_reset_date_of_birth_button),
                                )
                            }
                        },
                    )
                }

                is DateChooserData.ExpectedDateOfDelivery -> {
                    val descriptionText = if (calendarState.initialSelectedDate == null) {
                        stringResource(id = R.string.overview_select_expected_delivery_date_description)
                    } else {
                        stringResource(id = R.string.overview_select_new_expected_delivery_date_description)
                    }

                    val confirmButtonText = if (calendarState.initialSelectedDate == null) {
                        stringResource(id = R.string.overview_pregnancy_select_expected_delivery_date_button_title)
                    } else {
                        stringResource(id = R.string.overview_pregnancy_select_new_expected_delivery_date_button_title)
                    }
                    DefaultDateChooserScreen(
                        description = descriptionText,
                        confirmButtonText = confirmButtonText,
                        calendarState = calendarState,
                        onUpdateDate = calendarViewModel::onDaySelected,
                        onConfirmDate = {
                            calendarState.calendarUiState.value.selectedDate?.let {
                                viewModel.setDate(it)
                            }
                        },
                        bottomContent = {
                            if (calendarState.initialSelectedDate == null) {
                                GravidButton(
                                    modifier = Modifier
                                        .fillMaxWidth()
                                        .padding(horizontal = GravidTheme.dimens.largeSpacing),
                                    onClick = viewModel::changeToCalculateDueDate,
                                    containerColor = GravidTheme.colorScheme.secondary,
                                    text = stringResource(id = R.string.overview_pregnancy_calculate_expected_delivery_date_button_title),
                                )
                            } else {
                                Row(
                                    modifier = Modifier
                                        .fillMaxWidth()
                                        .padding(horizontal = GravidTheme.dimens.largeSpacing),
                                    horizontalArrangement = Arrangement.End,
                                ) {
                                    GravidTextButton(
                                        onClick = { showClearDialog = true },
                                        text = stringResource(id = R.string.overview_end_pregnancy_button_title),
                                    )
                                }
                            }
                        },
                    )
                }

                null -> Unit
            }
        }
    }

    if (showClearDialog) {
        @StringRes val titleResId: Int

        @StringRes val descriptionResId: Int

        @StringRes val clearButtonTitleResId: Int

        when (uiState) {
            is DateChooserData.ExpectedDateOfDelivery -> {
                titleResId = R.string.overview_reset_date_of_birth_alert_title
                descriptionResId = R.string.overview_reset_expected_due_date_alert_description
                clearButtonTitleResId = R.string.common_delete
            }

            is DateChooserData.DateOfBirth -> {
                titleResId = R.string.overview_reset_date_of_birth_alert_title
                descriptionResId = R.string.overview_reset_date_of_birth_alert_description
                clearButtonTitleResId = R.string.common_cta_reset
            }

            // This is not a possibility at the moment, so strings are placeholders
            else -> {
                titleResId = R.string.overview_reset_date_of_birth_alert_title
                descriptionResId = R.string.common_no
                clearButtonTitleResId = R.string.common_cta_reset
            }
        }

        GravidDialog(
            onDismissRequest = { showClearDialog = false },
            title = stringResource(id = titleResId),
            text = stringResource(id = descriptionResId),
            confirmButtonText = stringResource(id = clearButtonTitleResId),
            onConfirmButtonClicked = viewModel::clearDate,
            onDismissButtonClicked = { showClearDialog = false },
            dismissButtonText = stringResource(id = R.string.common_regret),
        )
    }
}

@Composable
internal fun DefaultDateChooserScreen(
    description: String,
    calendarState: CalendarState,
    onUpdateDate: (LocalDate) -> Unit,
    onConfirmDate: () -> Unit,
    confirmButtonText: String,
    bottomContent: @Composable ColumnScope.() -> Unit = {},
) {
    Column(Modifier.fillMaxSize()) {
        GravidText(modifier = Modifier.padding(horizontal = GravidTheme.dimens.largeSpacing), text = description)

        Calendar(
            modifier = Modifier.padding(top = GravidTheme.dimens.largeSpacing),
            calendarState = calendarState,
            onDayClicked = onUpdateDate,
        )

        Spacer(Modifier.weight(1f))

        GravidButton(
            modifier = Modifier
                .fillMaxWidth()
                .padding(horizontal = GravidTheme.dimens.largeSpacing),
            onClick = onConfirmDate,
            text = confirmButtonText,
        )

        bottomContent()
    }
}

@GravidPreview
@Composable
private fun DefaultDateChooserScreenPreview() {
    GravidTheme {
        DefaultDateChooserScreen(
            description = "Description",
            calendarState = CalendarState(),
            onUpdateDate = {},
            onConfirmDate = {},
            confirmButtonText = "OK",
        )
    }
}
