/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2023 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsp.mingraviditet.ui.main.overview.datechooser

import androidx.lifecycle.SavedStateHandle
import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import dagger.hilt.android.lifecycle.HiltViewModel
import dk.nsp.domain.repository.PregnancyRepository
import dk.nsp.mingraviditet.ui.main.overview.datechooser.model.DateChooserData
import dk.nsp.mingraviditet.ui.main.overview.datechooser.model.DateChooserMethod
import kotlinx.coroutines.flow.Flow
import kotlinx.coroutines.flow.MutableSharedFlow
import kotlinx.coroutines.flow.collectLatest
import kotlinx.coroutines.flow.combine
import kotlinx.coroutines.flow.map
import kotlinx.coroutines.launch
import timber.log.Timber
import java.time.LocalDate
import javax.inject.Inject

private const val DATE_CHOOSER_METHOD = "dateChooserMethodArgKey"

@HiltViewModel
class DateChooserViewModel @Inject constructor(
    private val savedStateHandle: SavedStateHandle,
    private val pregnancyRepository: PregnancyRepository,
) : ViewModel() {

    private val methodArg = savedStateHandle.getStateFlow(DATE_CHOOSER_METHOD, DateChooserMethod.ExpectedDateOfDelivery.name)

    private val _method = methodArg.map {
        DateChooserMethod.valueOf(it)
    }

    private val pregnancy = pregnancyRepository.pregnancyInfo.map { it }

    val dateChooserDataFlow = pregnancy.combine(_method) { pregnancyInfo, method ->
        when (method) {
            DateChooserMethod.ExpectedDateOfDelivery -> {
                DateChooserData.ExpectedDateOfDelivery(
                    initialDate = pregnancyInfo?.expectedDueDate,
                    minDate = LocalDate.now(),
                    maxDate = LocalDate.now().plusWeeks(40),
                )
            }

            DateChooserMethod.CalculateExpectedDateOfDelivery -> {
                DateChooserData.CalculateExpectedDateOfDelivery(
                    minDate = LocalDate.now().minusWeeks(40),
                    maxDate = LocalDate.now(),
                )
            }

            DateChooserMethod.DateOfBirth -> {
                if (pregnancyInfo == null) throw IllegalStateException("We should never be able to set date of birth, if pregnancy doesn't have a due date")

                Timber.d("Date of birth data: ${LocalDate.now().coerceAtMost(pregnancyInfo.dueDateMethod.expectedDueDate.plusWeeks(2))} -> ${pregnancyInfo.dueDateMethod.expectedDueDate.plusWeeks(2)}")
                DateChooserData.DateOfBirth(
                    initialDate = pregnancyInfo.dateOfBirth,
                    minDate = pregnancyInfo.dueDateMethod.lastMenstruationPeriod,
                    maxDate = LocalDate.now().coerceAtMost(pregnancyInfo.dueDateMethod.expectedDueDate.plusWeeks(2)),
                )
            }
        }
    }

    private val _onDateUpdated = MutableSharedFlow<Unit?>(replay = 0)
    val onDateUpdated: Flow<Unit?> = _onDateUpdated

    fun changeToCalculateDueDate() = viewModelScope.launch {
        savedStateHandle[DATE_CHOOSER_METHOD] = DateChooserMethod.CalculateExpectedDateOfDelivery.name
    }

    fun setDate(date: LocalDate) = viewModelScope.launch {
        _method.collectLatest {
            when (it) {
                DateChooserMethod.ExpectedDateOfDelivery -> pregnancyRepository.setExpectedDueDate(date)
                DateChooserMethod.DateOfBirth -> pregnancyRepository.setDateOfBirth(date)
                DateChooserMethod.CalculateExpectedDateOfDelivery -> pregnancyRepository.setLastMenstruationPeriod(date)
            }
            _onDateUpdated.emit(Unit)
        }
    }

    fun clearDate() = viewModelScope.launch {
        _method.collectLatest {
            when (it) {
                DateChooserMethod.ExpectedDateOfDelivery -> pregnancyRepository.clearPregnancy()
                DateChooserMethod.DateOfBirth -> pregnancyRepository.setDateOfBirth(null)
                DateChooserMethod.CalculateExpectedDateOfDelivery -> Unit // N/A
            }
            _onDateUpdated.emit(Unit)
        }
    }
}
