/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2023 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsp.mingraviditet.ui.main.overview.datechooser.calendar

import androidx.compose.foundation.ExperimentalFoundationApi
import androidx.compose.foundation.gestures.snapping.rememberSnapFlingBehavior
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.ColumnScope
import androidx.compose.foundation.layout.ExperimentalLayoutApi
import androidx.compose.foundation.layout.PaddingValues
import androidx.compose.foundation.layout.consumeWindowInsets
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.wrapContentWidth
import androidx.compose.foundation.lazy.LazyRow
import androidx.compose.foundation.lazy.items
import androidx.compose.foundation.lazy.rememberLazyListState
import androidx.compose.runtime.Composable
import androidx.compose.runtime.LaunchedEffect
import androidx.compose.runtime.derivedStateOf
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.rememberCoroutineScope
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import dk.nsp.mingraviditet.gravidcompose.theme.GravidTheme
import dk.nsp.mingraviditet.gravidcompose.utils.GravidPreview
import dk.nsp.mingraviditet.ui.main.overview.datechooser.calendar.model.CalendarState
import dk.nsp.mingraviditet.ui.main.overview.datechooser.calendar.model.CalendarUiState
import dk.nsp.mingraviditet.ui.main.overview.datechooser.calendar.model.Month
import kotlinx.coroutines.launch
import java.time.LocalDate
import java.time.temporal.WeekFields

@OptIn(ExperimentalLayoutApi::class, ExperimentalFoundationApi::class)
@Composable
fun Calendar(
    calendarState: CalendarState,
    onDayClicked: (date: LocalDate) -> Unit,
    modifier: Modifier = Modifier,
    contentPadding: PaddingValues = PaddingValues(),
) {
    val calendarUiState = calendarState.calendarUiState.value

    val lazyListState = rememberLazyListState()
    val coroutineScope = rememberCoroutineScope()

    LaunchedEffect(calendarState) {
        val indexOfMonthWithSelectedDate = calendarState.initialSelectedDate?.let { initial ->
            calendarState.listMonths.indexOfFirst { month -> month.yearMonth.monthValue == initial.monthValue }
        } ?: -1

        val indexOfCurrentMonth = calendarState.listMonths.indexOfFirst { month -> LocalDate.now().monthValue == month.yearMonth.monthValue }

        if (indexOfMonthWithSelectedDate != -1) {
            lazyListState.scrollToItem(indexOfMonthWithSelectedDate)
        } else if (indexOfCurrentMonth != -1) {
            lazyListState.scrollToItem(indexOfCurrentMonth)
        }
    }

    val index by remember { derivedStateOf { lazyListState.firstVisibleItemIndex } }
    val month by remember(index) {
        mutableStateOf(calendarState.listMonths[index])
    }

    Column(modifier = modifier.consumeWindowInsets(contentPadding)) {
        MonthHeader(
            modifier = Modifier
                .fillMaxWidth()
                .wrapContentWidth(Alignment.CenterHorizontally)
                .padding(horizontal = GravidTheme.dimens.largestSpacing),
            month = month.yearMonth.month.name,
            year = month.yearMonth.year.toString(),
            onNextMonth = if (lazyListState.canScrollForward) {
                {
                    coroutineScope.launch {
                        lazyListState.animateScrollToItem(lazyListState.firstVisibleItemIndex + 1)
                    }
                }
            } else {
                null
            },
            onPreviousMonth = if (lazyListState.canScrollBackward) {
                {
                    coroutineScope.launch {
                        lazyListState.animateScrollToItem(lazyListState.firstVisibleItemIndex - 1)
                    }
                }
            } else {
                null
            },
        )
        // Expanding width and centering horizontally
        val contentModifier = Modifier
            .fillMaxWidth()
            .wrapContentWidth(Alignment.CenterHorizontally)

        DaysOfWeek(modifier = contentModifier)

        LazyRow(
            state = lazyListState,
            flingBehavior = rememberSnapFlingBehavior(lazyListState = lazyListState),
        ) {
            items(calendarState.listMonths) { month ->
                Column(
                    modifier = Modifier.fillParentMaxWidth(),
                ) {
                    itemsCalendarMonth(
                        calendarUiState,
                        onDayClicked,
                        month,
                    )
                }
            }
        }
    }
}

@Composable
private fun ColumnScope.itemsCalendarMonth(
    calendarUiState: CalendarUiState,
    onDayClicked: (LocalDate) -> Unit,
    month: Month,
) {
    month.weeks.forEach { week ->
        Week(
            calendarUiState = calendarUiState,
            modifier = Modifier
                .fillMaxWidth()
                .wrapContentWidth(Alignment.CenterHorizontally),
            week = week,
            onDayClicked = onDayClicked,
        )
    }
}

internal val CALENDAR_STARTS_ON = WeekFields.ISO

@GravidPreview
@Composable
fun DayPreview() {
    GravidTheme {
        Calendar(CalendarState().apply { this.setSelectedDay(LocalDate.now().plusDays(1)) }, onDayClicked = { })
    }
}
