/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2023 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsp.mingraviditet.ui.useradmission.onboarding

import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.padding
import androidx.compose.runtime.Composable
import androidx.compose.runtime.LaunchedEffect
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.setValue
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.res.stringResource
import androidx.hilt.navigation.compose.hiltViewModel
import androidx.lifecycle.compose.collectAsStateWithLifecycle
import dk.nsp.mingraviditet.R
import dk.nsp.mingraviditet.gravidcompose.component.GravidButton
import dk.nsp.mingraviditet.gravidcompose.component.GravidCircularProgressIndicator
import dk.nsp.mingraviditet.gravidcompose.theme.GravidTheme
import dk.nsp.mingraviditet.gravidcompose.utils.GravidPreview
import dk.nsp.mingraviditet.model.misc.SettingSubSectionUrl
import dk.nsp.mingraviditet.ui.common.compose.EmptyStateRow
import dk.nsp.mingraviditet.ui.main.common.compose.LocalWebView
import dk.nsp.mingraviditet.ui.main.more.settings.SettingsPageUiState
import dk.nsp.mingraviditet.ui.main.more.settings.SettingsPageViewModel
import dk.nsp.mingraviditet.ui.widgets.compose.BottomSheetDialog
import dk.nsp.mingraviditet.util.android.WebViewHelper

@Composable
internal fun AgreementDialog(
    onDismissRequest: () -> Unit,
    onAgreementAccepted: () -> Unit,
    viewModel: AgreementViewModel = hiltViewModel(),
    webViewViewModel: SettingsPageViewModel = hiltViewModel(),
) {
    val state by webViewViewModel.pageResult.collectAsStateWithLifecycle()

    LaunchedEffect(Unit) {
        webViewViewModel.pagePath.value = SettingSubSectionUrl.Consent.path
    }

    BottomSheetDialog(
        title = stringResource(id = R.string.agreement_title),
        onDismissRequest = { onDismissRequest() },
        contentVerticalArrangement = Arrangement.Top,
    ) {
        AgreementScreen(
            onAgreementAccepted = {
                viewModel.setHasSeenAgreement()
                onDismissRequest()
                onAgreementAccepted()
            },
            state = state,
        )
    }
}

@Composable
internal fun AgreementScreen(
    onAgreementAccepted: () -> Unit,
    state: SettingsPageUiState,
    modifier: Modifier = Modifier,
) {
    Column(
        modifier = modifier
            .fillMaxSize()
            .padding(horizontal = GravidTheme.dimens.largeSpacing),
    ) {
        when (state) {
            SettingsPageUiState.Failed -> EmptyStateRow(
                title = stringResource(id = R.string.common_alert_title_error),
                description = stringResource(id = R.string.common_alert_message_failed_to_load_data),
            )

            SettingsPageUiState.Loading -> {
                Box(
                    modifier = Modifier.fillMaxSize(),
                    contentAlignment = Alignment.Center,
                ) {
                    GravidCircularProgressIndicator()
                }
            }

            is SettingsPageUiState.Success -> {
                var showAcceptButton by remember {
                    mutableStateOf(false)
                }
                LocalWebView(
                    content = state.content,
                    webViewType = WebViewHelper.Type.Subpage,
                    onPageFinishedLoading = {
                        showAcceptButton = true
                    },
                )

                if (showAcceptButton) {
                    GravidButton(
                        modifier = Modifier
                            .fillMaxWidth()
                            .padding(bottom = GravidTheme.dimens.largestSpacing),
                        onClick = onAgreementAccepted,
                        text = stringResource(id = R.string.agreement_button_text),
                    )
                }
            }
        }
    }
}

@GravidPreview
@Composable
private fun AgreementPreview() {
    GravidTheme {
        AgreementScreen(
            onAgreementAccepted = {},
            state = SettingsPageUiState.Loading,
        )
    }
}
