/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2023 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsp.mingraviditet.ui.useradmission.onboarding

import androidx.compose.animation.animateColorAsState
import androidx.compose.foundation.ExperimentalFoundationApi
import androidx.compose.foundation.Image
import androidx.compose.foundation.background
import androidx.compose.foundation.gestures.Orientation
import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.PaddingValues
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.Spacer
import androidx.compose.foundation.layout.fillMaxHeight
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.size
import androidx.compose.foundation.lazy.grid.GridCells
import androidx.compose.foundation.lazy.grid.LazyVerticalGrid
import androidx.compose.foundation.lazy.grid.items
import androidx.compose.foundation.pager.HorizontalPager
import androidx.compose.foundation.pager.PageSize
import androidx.compose.foundation.pager.PagerDefaults
import androidx.compose.foundation.pager.rememberPagerState
import androidx.compose.foundation.shape.CircleShape
import androidx.compose.runtime.Composable
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.saveable.rememberSaveable
import androidx.compose.runtime.setValue
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.draw.clip
import androidx.compose.ui.graphics.ColorFilter
import androidx.compose.ui.layout.ContentScale
import androidx.compose.ui.res.painterResource
import androidx.compose.ui.res.stringResource
import androidx.compose.ui.text.style.TextAlign
import androidx.compose.ui.unit.dp
import androidx.hilt.navigation.compose.hiltViewModel
import androidx.lifecycle.compose.collectAsStateWithLifecycle
import dk.nsp.mingraviditet.R
import dk.nsp.mingraviditet.gravidcompose.component.GravidCard
import dk.nsp.mingraviditet.gravidcompose.component.GravidText
import dk.nsp.mingraviditet.gravidcompose.component.GravidTextButton
import dk.nsp.mingraviditet.gravidcompose.extensions.bold
import dk.nsp.mingraviditet.gravidcompose.theme.GravidTheme
import dk.nsp.mingraviditet.gravidcompose.utils.GravidPreview
import dk.nsp.mingraviditet.ui.useradmission.onboarding.models.OnboardingPage

@Composable
fun OnboardingRoute(
    onAgreementAccepted: () -> Unit,
    onboardingViewModel: OnboardingViewModel = hiltViewModel(),
) {
    val onboardingPages by onboardingViewModel.onboardingPages.collectAsStateWithLifecycle()

    OnboardingScreen(onGetStartedClicked = onAgreementAccepted, onboardingPages = onboardingPages)
}

@OptIn(ExperimentalFoundationApi::class)
@Composable
private fun OnboardingScreen(
    onGetStartedClicked: () -> Unit,
    onboardingPages: List<OnboardingPage>,
) {
    val pagerState = rememberPagerState(pageCount = { onboardingPages.size })

    var showAgreementDialog by rememberSaveable {
        mutableStateOf(false)
    }

    Column(
        modifier = Modifier
            .fillMaxSize()
            .padding(vertical = GravidTheme.dimens.largeSpacing),
    ) {
        HorizontalPager(
            modifier = Modifier.weight(0.8f),
            state = pagerState,
            pageSpacing = 0.dp,
            userScrollEnabled = true,
            reverseLayout = false,
            contentPadding = PaddingValues(top = GravidTheme.dimens.largerSpacing),
            beyondBoundsPageCount = 3,
            pageSize = PageSize.Fill,
            key = null,
            pageNestedScrollConnection = PagerDefaults.pageNestedScrollConnection(
                Orientation.Horizontal,
            ),
            pageContent = { page ->
                val onboardingPage = onboardingPages[page]
                Column {
                    Column(modifier = Modifier.weight(0.2f)) {
                        GravidText(
                            modifier = Modifier
                                .fillMaxWidth()
                                .padding(bottom = GravidTheme.dimens.defaultSpacing)
                                .padding(horizontal = GravidTheme.dimens.largerSpacing),
                            textAlign = TextAlign.Center,
                            text = stringResource(id = onboardingPage.title),
                            style = GravidTheme.typography.Hero,
                        )

                        GravidText(
                            text = stringResource(id = onboardingPage.text),
                            modifier = Modifier
                                .fillMaxWidth()
                                .padding(horizontal = GravidTheme.dimens.largerSpacing),
                            textAlign = TextAlign.Center,
                        )
                    }

                    Box(modifier = Modifier.weight(0.8f)) {
                        Image(
                            modifier = Modifier.fillMaxSize(),
                            painter = painterResource(id = onboardingPage.backgroundImageRes),
                            contentDescription = null,
                            contentScale = ContentScale.FillWidth,
                        )

                        when (onboardingPage) {
                            is OnboardingPage.Default -> Spacer(modifier = Modifier.fillMaxHeight())
                            is OnboardingPage.Logos -> {
                                LazyVerticalGrid(
                                    modifier = Modifier.fillMaxHeight(),
                                    columns = GridCells.Fixed(2),
                                    contentPadding = PaddingValues(horizontal = GravidTheme.dimens.largeSpacing),
                                    horizontalArrangement = Arrangement.spacedBy(GravidTheme.dimens.largeSpacing),
                                    verticalArrangement = Arrangement.spacedBy(GravidTheme.dimens.largeSpacing, Alignment.CenterVertically),
                                ) {
                                    items(onboardingPage.logos) { logo ->
                                        GravidCard {
                                            Image(
                                                modifier = Modifier.fillMaxSize(),
                                                painter = painterResource(id = logo),
                                                contentDescription = "Logo",
                                                colorFilter = ColorFilter.tint(GravidTheme.colorScheme.onSurface),
                                            )
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            },
        )

        Row(
            modifier = Modifier
                .fillMaxWidth()
                .weight(0.05f),
            verticalAlignment = Alignment.CenterVertically,
            horizontalArrangement = Arrangement.Center,
        ) {
            repeat(onboardingPages.size) { iteration ->
                val color by animateColorAsState(
                    targetValue = if (pagerState.currentPage == iteration) {
                        GravidTheme.colorScheme.onBackground
                    } else {
                        GravidTheme.colorScheme.onBackground.copy(alpha = 0.25f)
                    },
                    label = "DotColor animation",
                )

                Box(
                    modifier = Modifier
                        .padding(horizontal = GravidTheme.dimens.trumpSpacing)
                        .clip(CircleShape)
                        .background(color)
                        .size(8.dp),
                )
            }
        }

        Box(modifier = Modifier.weight(0.15f), contentAlignment = Alignment.Center) {
            GravidTextButton(
                modifier = Modifier
                    .fillMaxWidth()
                    .padding(horizontal = GravidTheme.dimens.largeSpacing),
                onClick = { showAgreementDialog = true },
                textStyle = GravidTheme.typography.BodyStandard.bold,
                text = stringResource(id = R.string.onboarding_login_get_started_button),
            )
        }
    }

    if (showAgreementDialog) {
        AgreementDialog(
            onDismissRequest = { showAgreementDialog = false },
            onAgreementAccepted = onGetStartedClicked,
        )
    }
}

@GravidPreview
@Composable
private fun OnboardingScreenPreview() {
    GravidTheme {
        OnboardingScreen(
            onGetStartedClicked = {},
            onboardingPages = listOf(
                OnboardingPage.Logos(
                    logos = listOf(
                        R.drawable.logo_danske_regioner,
                        R.drawable.logo_sd,
                        R.drawable.logo_plo,
                        R.drawable.logo_kl,
                        R.drawable.logo_sds,
                        R.drawable.logo_ministeriet,
                    ),
                    R.string.onboarding_overview_title,
                    R.string.onboarding_overview_text,
                    R.drawable.ill_onboarding_01,
                ),
                OnboardingPage.Default(
                    R.string.onboarding_appointments_title,
                    R.string.onboarding_appointments_text,
                    R.drawable.ill_onboarding_02,
                ),
                OnboardingPage.Default(
                    R.string.onboarding_knowledge_title,
                    R.string.onboarding_knowledge_text,
                    R.drawable.ill_onboarding_03,
                ),
            ),
        )
    }
}
