/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2023 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsp.domain.pregnancymodels

import dk.nsp.testassertions.assertEquals
import org.junit.jupiter.api.Test
import java.time.LocalDate

class PregnancyInfoTest {

    private val activeLastMenstruationPeriodPregnancyInfo = PregnancyInfo(
        dueDateMethod = DueDateMethod.LastMenstruationPeriod(LocalDate.now()),
        dateOfBirth = null,
    )

    private val activeExpectedDueDatePregnancyInfo = PregnancyInfo(
        dueDateMethod = DueDateMethod.ExpectedDueDate(LocalDate.now()),
        dateOfBirth = null,
    )

    @Test
    fun getExpectedDueDate() {
        activeLastMenstruationPeriodPregnancyInfo.expectedDueDate.assertEquals(LocalDate.now().plusDays(280))
        activeExpectedDueDatePregnancyInfo.expectedDueDate.assertEquals(LocalDate.now())
    }

    @Test
    fun getPregnancyWeek() {
        activeLastMenstruationPeriodPregnancyInfo.pregnancyWeek.assertEquals(1, "When LMP is today, we are in week 1")
        val daysIntoWeek1 = PregnancyInfo(
            dueDateMethod = DueDateMethod.LastMenstruationPeriod(LocalDate.now().minusDays(2)),
            dateOfBirth = null,
        )
        daysIntoWeek1.pregnancyWeek.assertEquals(1)

        val secondTrimester = PregnancyInfo(
            dueDateMethod = DueDateMethod.LastMenstruationPeriod(LocalDate.now().minusWeeks(12)),
            dateOfBirth = null,
        )
        secondTrimester.pregnancyWeek.assertEquals(13)

        val userForgotToBirthBaby = PregnancyInfo(
            dueDateMethod = DueDateMethod.LastMenstruationPeriod(LocalDate.now().minusWeeks(133)),
            dateOfBirth = null,
        )
        userForgotToBirthBaby.pregnancyWeek.assertEquals(MAX_PREGNANCY_WEEK + 1)
    }

    @Test
    fun getGestationAge() {
        val twoDaysIntoWeek1 = PregnancyInfo(
            dueDateMethod = DueDateMethod.LastMenstruationPeriod(LocalDate.now().minusDays(2)),
            dateOfBirth = null,
        )
        twoDaysIntoWeek1.gestationAge.gestationDays.assertEquals(2)

        val userForgotToBirthBaby = PregnancyInfo(
            dueDateMethod = DueDateMethod.LastMenstruationPeriod(LocalDate.now().minusWeeks(133)),
            dateOfBirth = null,
        )
        userForgotToBirthBaby.gestationAge.apply {
            gestationDays.assertEquals(6)
            gestationWeek.assertEquals(MAX_PREGNANCY_WEEK)
        }
    }

    @Test
    fun getTrimester() {
        /**
         * First trimester: 0+0-6 to 11+0-6 weeks
         * Second trimester: from week 12+0-6 to 27+0-6
         * Third trimester: from week 28+0-6
         */
        val firstTrimester = PregnancyInfo(
            dueDateMethod = DueDateMethod.LastMenstruationPeriod(LocalDate.now()),
            dateOfBirth = null,
        )
        firstTrimester.trimester.assertEquals(1, "First trimester should be within gestation 0+0-6 to 11+0-6: ${firstTrimester.pregnancyWeek - 1}")

        val secondTrimester = PregnancyInfo(
            dueDateMethod = DueDateMethod.LastMenstruationPeriod(LocalDate.now().minusWeeks(12)),
            dateOfBirth = null,
        )
        secondTrimester.trimester.assertEquals(2, "Second trimester should be within gestation 12+0-6 to 27+0-6: ${secondTrimester.pregnancyWeek - 1}")

        val thirdTrimester = PregnancyInfo(
            dueDateMethod = DueDateMethod.LastMenstruationPeriod(LocalDate.now().minusWeeks(28)),
            dateOfBirth = null,
        )
        thirdTrimester.trimester.assertEquals(3, "Third trimester should be from gestation 28+0-6 to end: ${thirdTrimester.pregnancyWeek - 1}")

        // Expected Due date variant
        val firstEDDTrimester = PregnancyInfo(
            dueDateMethod = DueDateMethod.ExpectedDueDate(LocalDate.now().plusWeeks(30)),
            dateOfBirth = null,
        )
        firstEDDTrimester.trimester.assertEquals(1, "First trimester should be within gestation 0+0-6 to 11+0-6: ${firstTrimester.pregnancyWeek - 1}")

        val secondEDDTrimester = PregnancyInfo(
            dueDateMethod = DueDateMethod.ExpectedDueDate(LocalDate.now().plusWeeks(15)),
            dateOfBirth = null,
        )
        secondEDDTrimester.trimester.assertEquals(2, "Second trimester should be within gestation 12+0-6 to 27+0-6: ${secondTrimester.pregnancyWeek - 1}")

        val thirdEDDTrimester = PregnancyInfo(
            dueDateMethod = DueDateMethod.ExpectedDueDate(LocalDate.now()),
            dateOfBirth = null,
        )
        thirdEDDTrimester.trimester.assertEquals(3, "Third trimester should be from gestation 28+0-6 to end: ${thirdTrimester.pregnancyWeek - 1}")
    }

    @Test
    fun getPregnancyMonth() {
    }

    @Test
    fun getPregnancyMonthDouble() {
    }

    @Test
    fun getDueDateMethod() {
    }

    @Test
    fun getDateOfBirthDate() {
    }
}
