/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2023 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsp.mingraviditet


import com.android.build.api.variant.AndroidComponentsExtension
import com.android.build.gradle.BaseExtension
import groovy.xml.XmlSlurper
import groovy.xml.slurpersupport.NodeChild
import org.gradle.api.GradleException
import org.gradle.api.Project
import org.gradle.api.tasks.testing.Test
import org.gradle.api.tasks.testing.logging.TestExceptionFormat
import org.gradle.api.tasks.testing.logging.TestLogEvent
import org.gradle.kotlin.dsl.configure
import org.gradle.kotlin.dsl.dependencies
import org.gradle.kotlin.dsl.extra
import org.gradle.kotlin.dsl.register
import org.gradle.kotlin.dsl.withType
import org.gradle.testing.jacoco.plugins.JacocoPluginExtension
import org.gradle.testing.jacoco.plugins.JacocoTaskExtension
import org.gradle.testing.jacoco.tasks.JacocoReport
import java.io.File
import java.util.Locale
import kotlin.math.roundToInt

private val coverageExclusions = listOf(
    // Android
    "**/R.class",
    "**/R\$*.class",
    "**/BuildConfig.*",
    "**/Manifest*.*",

    // App
    "**/dk/nsp/mingraviditet/databinding/*", // The autogenerated android XML binding implementation
    "**/dk/nsp/mingraviditet/di/", // Exclude Dependency Injection modules from test coverage
    // Excluding custom application, since it"s currently only debug and Firebase setup
    "**/dk/nsp/mingraviditet/App*",

    // Currently no UI testing, so exclude fragments and viewmodels, since they are mainly presentational components
    "**/**ViewModel*",
    "**/**Fragment*",

    // Android
    "**/androidx/databinding/*",
    "**/DataBinderMapperImpl*.*",
    "**/DataBindingTriggerClass*.*",
    "**/BR.*",
    // For some reason one of the autogenerated Android Navigation directions was found for coverage
    "**/dk/nsp/mingraviditet/JournalDirections*.*",

    // Dagger/Hilt (Mainly autogenerated classes for DI)
    "**/dagger/hilt/internal/*",
    "**/DaggerApp_*.*",
    "**/App_ComponentTreeDeps*",

    "**/hilt_aggregated_deps/*",
    "**/App_HiltComponents*.*",
    "**/Hilt_*",
    "**/*HiltModule*",

    // Shared
    "**/dk/nsp/models/*", // Exclude all the models generated by the OpenAPI script

    // Exclude models (No need for testing constructors and get/set for data classes)
    "**/dk/nsp/api/cms/models/*",
    "**/dk/nsp/api/auth/OpenIdCredentials.class",
    "**/dk/nsp/api/auth/AuthException*",

    // Repo
    // Exclude models (No need for testing constructors and get/set for data classes)
    "**/dk/nsp/domain/careplan/CareplanActivities.class",
    "**/dk/nsp/domain/SettingSubSectionUrl.class",
    "**/dk/nsp/models/ArticleSearchResult.class",
)

private val Project.jacoco: JacocoPluginExtension
    get() = extensions.findByName("jacoco") as? JacocoPluginExtension
        ?: error("Not a Jacoco module: $name")

private val Project.android: BaseExtension
    get() = extensions.findByName("android") as? BaseExtension
        ?: error("Not an Android module: $name")

private fun String.capitalize() = replaceFirstChar { if (it.isLowerCase()) it.titlecase(Locale.ENGLISH) else it.toString() }

internal fun Project.configureJacoco(
    androidComponentsExtension: AndroidComponentsExtension<*, *, *>,
) {
    configure<JacocoPluginExtension> {
        toolVersion = libs.findVersion("jacoco").get().toString()
    }

    val jacocoTestReport = tasks.create("jacocoTestReport")

    androidComponentsExtension.onVariants { variant ->
        if (variant.name.contains("${MinGraviditetFlavor.projectEnv.name}Release")) {

            val testTaskName = "test${variant.name.capitalize()}UnitTest"
            val versionName = rootProject.extra["versionName"] as? String
            val versionCode = rootProject.extra["versionCode"] as? Int

            val reportTask = tasks.register("jacoco${testTaskName.capitalize()}Report", JacocoReport::class) {
                dependsOn(testTaskName)

                reports {
                    xml.required.set(true)
                    csv.required.set(false)
                    html.required.set(true)

                    html.outputLocation.set(file("${project.buildDir}/../../qa-reports/coverage-reports-$versionName-$versionCode/${project.name}"))
                }

                classDirectories.setFrom(
                    fileTree("$buildDir/tmp/kotlin-classes/${variant.name}") {
                        exclude(coverageExclusions)
                    },
                )

                sourceDirectories.setFrom(files("$projectDir/src/main/java", "$projectDir/src/main/kotlin"))
                executionData.setFrom(file("$buildDir/jacoco/$testTaskName.exec"))
            }

            project.android.testOptions {
                unitTests.all {
                    it.testLogging {
                        events(TestLogEvent.PASSED, TestLogEvent.SKIPPED, TestLogEvent.FAILED)
                        exceptionFormat = TestExceptionFormat.FULL
                    }
                    it.reports.html.outputLocation.set(file("${project.buildDir}/../../qa-reports/unit-tests-$versionName-$versionCode/${project.name}"))
                }
            }

            jacocoTestReport.apply {
                dependsOn(reportTask)
                doLast {
                    println("Wrote '${project.name}' coverage report to ${jacoco.reportsDirectory.get()}/index.html")
                    jacocoTestReportPrint("jacoco${testTaskName.capitalize()}Report")
                }
            }
        }
    }

    tasks.withType<Test>().configureEach {
        configure<JacocoTaskExtension> {
            // Required for JaCoCo + Robolectric
            // https://github.com/robolectric/robolectric/issues/2230
            // TODO: Consider removing if not we don't add Robolectric
            isIncludeNoLocationClasses = true

            // Required for JDK 11 with the above
            // https://github.com/gradle/gradle/issues/5184#issuecomment-391982009
            excludes = listOf("jdk.internal.*")
        }
    }

    dependencies {
        add("testImplementation", project(":TestAssertions"))
    }
}

private fun Project.jacocoTestReportPrint(testTaskName: String) {
    val report = file("$buildDir/reports/jacoco/$testTaskName/$testTaskName.xml")

    logger.lifecycle("Checking coverage results: $testTaskName")

    val metrics = report.extractTestsCoveredByType()
    val limits = mapOf(
        "instruction" to 0,
        "branch" to 0,
        "line" to 0,
        "complexity" to 0,
        "method" to 0,
        "class" to 0,
    )

    val failures = metrics.filter { entry ->
        entry.value < limits[entry.key]!!
    }.map { entry ->
        "- ${entry.key} coverage rate is: ${entry.value}%, minimum is ${limits[entry.key]}%"
    }

    if (failures.isNotEmpty()) {
        logger.quiet("------------------ Code Coverage Failed -----------------------")
        failures.forEach { logger.quiet(it) }
        logger.quiet("---------------------------------------------------------------")
        throw GradleException("Code coverage failed")
    }

    logger.quiet("------------------ Code Coverage Success -----------------------")
    metrics.forEach { entry ->
        logger.quiet("- ${entry.key} coverage rate is: ${entry.value}%")
    }
    logger.quiet("---------------------------------------------------------------")
}

@Suppress("UNCHECKED_CAST")
private fun File.extractTestsCoveredByType(): Map<String, Double> {
    val xmlReader = XmlSlurper().apply {
        setFeature("http://apache.org/xml/features/nonvalidating/load-external-dtd", false)
        setFeature("http://apache.org/xml/features/disallow-doctype-decl", false)
    }

    val counterNodes: List<NodeChild> = xmlReader
        .parse(this).parent()
        .children()
        .filter {
            (it as NodeChild).name() == "counter"
        } as List<NodeChild>

    return counterNodes.associate { nodeChild ->
        val type = nodeChild.attributes()["type"].toString().lowercase(Locale.ENGLISH)

        val covered = nodeChild.attributes()["covered"].toString().toDouble()
        val missed = nodeChild.attributes()["missed"].toString().toDouble()
        val percentage = ((covered / (covered + missed)) * 10000.0).roundToInt() / 100.0

        Pair(type, percentage)
    }
}
