/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2023 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsp.mingraviditet.datastore

import androidx.datastore.core.DataStore
import dk.nsp.mingraviditet.datastore.extensions.toLocalDate
import dk.nsp.mingraviditet.datastore.extensions.toTimestamp
import dk.nsp.mingraviditet.model.data.MGPreferences
import kotlinx.coroutines.flow.Flow
import kotlinx.coroutines.flow.map
import java.time.LocalDate
import javax.inject.Inject

class MGPreferencesDataSource @Inject constructor(
    private val preferences: DataStore<ProtoMGPreferences>,
) {
    val data = preferences.data.map {
        MGPreferences(
            checklistDeleteAnimationShownCount = it.checklistDeleteAnimationShownCount,
            useBiometrics = it.useBiometrics,
            allowScreenshots = it.allowScreenshots,
            userAcceptedAgreement = it.userAcceptedAgreementOrNull?.toLocalDate(),
        )
    }

    suspend fun setUseBiometrics(value: Boolean) {
        preferences.updateData {
            it.copy {
                useBiometrics = value
            }
        }
    }

    suspend fun setAllowScreenshots(value: Boolean) {
        preferences.updateData {
            it.copy {
                allowScreenshots = value
            }
        }
    }

    suspend fun setUserAcceptedAgreement(value: LocalDate) {
        preferences.updateData {
            it.copy {
                userAcceptedAgreement = value.toTimestamp()
            }
        }
    }

    // <editor-fold desc="Checklist details fake drag counter">
    val shouldShowChecklistItemDeleteAnimation: Flow<Boolean> = preferences.data.map {
        it.checklistDeleteAnimationShownCount < TimesToShowChecklistDeleteAnimation
    }

    suspend fun incrementChecklistItemDeleteAnimationShownCount() {
        preferences.updateData {
            it.copy {
                if (checklistDeleteAnimationShownCount < TimesToShowChecklistDeleteAnimation) {
                    checklistDeleteAnimationShownCount += 1
                }
            }
        }
    }

    suspend fun disableChecklistItemDeleteAnimation() {
        preferences.updateData {
            it.copy {
                checklistDeleteAnimationShownCount = TimesToShowChecklistDeleteAnimation
            }
        }
    }
    // </editor-fold>

    suspend fun clearPreferences() {
        preferences.updateData {
            it.toBuilder()
                .clear()
                .build()
        }
    }

    companion object {
        const val TimesToShowChecklistDeleteAnimation = 10
    }
}
