/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2023 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsp.mingraviditet.datastore

import androidx.annotation.VisibleForTesting
import androidx.datastore.core.DataStore
import dk.nsp.mingraviditet.datastore.extensions.toLocalDate
import dk.nsp.mingraviditet.datastore.extensions.toTimestamp
import dk.nsp.mingraviditet.datastore.model.ChecklistDb
import dk.nsp.mingraviditet.datastore.model.ChecklistDb.Companion.toProto
import dk.nsp.mingraviditet.datastore.model.ChecklistItemDb
import dk.nsp.mingraviditet.model.data.PregnancyInfo
import kotlinx.coroutines.flow.combine
import kotlinx.coroutines.flow.map
import java.time.LocalDate
import javax.inject.Inject

class PregnancyPreferencesDataSource @Inject constructor(
    private val pregnancyPreferences: DataStore<Pregnancy>,
) {

    val data = pregnancyPreferences.data
        .map {
            PregnancyInfo(
                expectedDueDate = it.expectedDueDateOrNull?.toLocalDate(),
                lastMenstruationPeriod = it.lastMenstruationPeriodOrNull?.toLocalDate(),
                dateOfBirth = it.dateOfBirthOrNull?.toLocalDate(),
            )
        }

    @VisibleForTesting
    private val checkedChecklistItems = pregnancyPreferences.data.map { it.checkedChecklistItemsMap.keys }

    val checklists = pregnancyPreferences.data
        .combine(checkedChecklistItems) { data, checkedIds ->
            data.checklistsList.map { checklist ->
                ChecklistDb(
                    id = checklist.id,
                    title = checklist.title,
                    description = checklist.description,
                    items = checklist.itemsList.map { checklistItem ->
                        ChecklistItemDb(
                            id = checklistItem.id,
                            title = checklistItem.title,
                            checked = checklistItem.id in checkedIds,
                            order = checklistItem.order,
                        )
                    },
                )
            }
        }

    // <editor-fold desc="Date selections">
    suspend fun setExpectedDueDate(date: LocalDate?) {
        pregnancyPreferences.updateData {
            it.copy {
                if (date == null) {
                    clearExpectedDueDate()
                } else {
                    expectedDueDate = date.toTimestamp()
                }
            }
        }
    }

    suspend fun setLastMenstruationPeriod(date: LocalDate?) {
        pregnancyPreferences.updateData {
            it.copy {
                if (date == null) {
                    clearLastMenstruationPeriod()
                } else {
                    lastMenstruationPeriod = date.toTimestamp()
                }
            }
        }
    }

    suspend fun setDateOfBirth(date: LocalDate?) {
        pregnancyPreferences.updateData {
            it.copy {
                if (date == null) {
                    clearDateOfBirth()
                } else {
                    dateOfBirth = date.toTimestamp()
                }
            }
        }
    }
    // </editor-fold>

    // <editor-fold desc="Checklists">
    suspend fun setChecklists(checklists: List<ChecklistDb>) {
        val checklistsProto = checklists.map {
            it.toProto()
        }

        pregnancyPreferences.updateData {
            it.toBuilder().addAllChecklists(checklistsProto).build()
        }
    }

    suspend fun setChecklist(checklist: ChecklistDb) {
        val checklistProto = checklist.toProto()

        pregnancyPreferences.updateData {
            it.copy {
                val indexOfExisting = checklists.indexOfFirst { it.id == checklist.id }

                if (indexOfExisting != -1) {
                    // Make sure that the checked states are clean, even though it should be
                    checklistProto.itemsList.forEach { checklistItem ->
                        checkedChecklistItems.remove(checklistItem.id)
                    }

                    checklists[indexOfExisting] = checklistProto
                }
            }
        }
    }

    suspend fun toggleCheckedChecklistItem(checklistItemDb: ChecklistItemDb) {
        pregnancyPreferences.updateData {
            it.copy {
                val checkedItems = checkedChecklistItems.keys

                if (checklistItemDb.id in checkedItems) {
                    checkedChecklistItems.remove(checklistItemDb.id)
                } else {
                    checkedChecklistItems.put(checklistItemDb.id, true)
                }
            }
        }
    }

    suspend fun updateChecklistItem(checklistId: Int, checklistItemDb: ChecklistItemDb) {
        pregnancyPreferences.updateData { thing ->
            thing.copy {
                val checklistIndex = checklists.indexOfFirst { it.id == checklistId.toString() }
                val existingChecklist = checklists[checklistIndex]

                val newItem = with(checklistItemDb) {
                    ChecklistItem.newBuilder()
                        .setId(id)
                        .setTitle(title)
                        .setChecked(false)
                        .setOrder(order)
                        .build()
                }

                val newChecklist = if (existingChecklist.itemsList.any { it.id == checklistItemDb.id }) {
                    val indexOfItem = existingChecklist.itemsList.indexOfFirst { it.id == checklistItemDb.id }
                    existingChecklist.toBuilder().setItems(indexOfItem, newItem).build()
                } else {
                    existingChecklist.toBuilder().addItems(newItem).build()
                }

                checklists[checklistIndex] = newChecklist
            }
        }
    }

    suspend fun deleteChecklistItem(checklistId: Int, checklistItem: ChecklistItemDb) {
        pregnancyPreferences.updateData {
            it.copy {
                // Delete the checklist items "checked"
                if (checkedChecklistItems.keys.contains(checklistItem.id)) {
                    checkedChecklistItems.remove(checklistItem.id)
                }

                // Delete the checklist item
                val checklistIndex = checklists.indexOfFirst { it.id == checklistId.toString() }
                val existingChecklist = checklists[checklistIndex]

                val indexOfDeleted = existingChecklist.itemsList.indexOfFirst { it.id == checklistItem.id }

                if (indexOfDeleted != -1) {
                    val newChecklist = existingChecklist.toBuilder().removeItems(indexOfDeleted).build()
                    checklists[checklistIndex] = newChecklist
                }
            }
        }
    }
    // </editor-fold>

    suspend fun clearPregnancy() {
        pregnancyPreferences.updateData {
            it.toBuilder()
                .clear()
                .build()
        }
    }
}
