/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2023 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsp.mingraviditet.datastore

import dk.nsp.testassertions.assertEquals
import dk.nsp.testassertions.assertFalse
import dk.nsp.testassertions.assertNull
import dk.nsp.testassertions.assertTrue
import kotlinx.coroutines.flow.first
import kotlinx.coroutines.test.TestScope
import kotlinx.coroutines.test.UnconfinedTestDispatcher
import kotlinx.coroutines.test.runTest
import org.junit.jupiter.api.Assertions.*
import org.junit.jupiter.api.BeforeEach
import org.junit.jupiter.api.Test
import org.junit.jupiter.api.extension.RegisterExtension
import java.time.LocalDate

internal class MGPreferencesDataSourceTest {
    private val testScope = TestScope(UnconfinedTestDispatcher())

    private lateinit var subject: MGPreferencesDataSource

    @JvmField
    @RegisterExtension
    val tmpFolder = TemporaryFolderExtension()

    @BeforeEach
    fun setup() {
        subject = MGPreferencesDataSource(
            tmpFolder.testPregnancyPreferencesDataStore(
                filename = "mg_preferences_test.db",
                coroutineScope = testScope,
                serializer = MGPreferencesSerializer(),
            ),
        )
    }

    @Test
    fun setUseBiometrics() = testScope.runTest {
        subject.data.first().useBiometrics.assertFalse()
        subject.setUseBiometrics(true)
        subject.data.first().useBiometrics.assertTrue()
        subject.clearPreferences()
        subject.data.first().useBiometrics.assertFalse()
    }

    @Test
    fun setAllowScreenshots() = testScope.runTest {
        subject.data.first().allowScreenshots.assertFalse()
        subject.setAllowScreenshots(true)
        subject.data.first().allowScreenshots.assertTrue()
        subject.clearPreferences()
        subject.data.first().allowScreenshots.assertFalse()
    }

    @Test
    fun setUserAcceptedAgreement() = testScope.runTest {
        val today = LocalDate.now()
        subject.data.first().userAcceptedAgreement.assertNull()
        subject.setUserAcceptedAgreement(today)
        subject.data.first().userAcceptedAgreement.assertEquals(today)
        subject.clearPreferences()
        subject.data.first().userAcceptedAgreement.assertNull()
    }

    @Test
    fun getShouldShowChecklistItemDeleteAnimation() = testScope.runTest {
        subject.shouldShowChecklistItemDeleteAnimation.first().assertTrue()

        subject.disableChecklistItemDeleteAnimation()

        subject.shouldShowChecklistItemDeleteAnimation.first().assertFalse()
    }

    @Test
    fun incrementChecklistItemDeleteAnimationShownCount() = testScope.runTest {
        (0 until MGPreferencesDataSource.TimesToShowChecklistDeleteAnimation).forEach { _ ->
            subject.incrementChecklistItemDeleteAnimationShownCount()
        }

        subject.data.first().checklistDeleteAnimationShownCount.assertEquals(MGPreferencesDataSource.TimesToShowChecklistDeleteAnimation)
    }
}
