/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2023 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsp.mingraviditet.datastore

import dk.nsp.mingraviditet.datastore.model.ChecklistDb
import dk.nsp.mingraviditet.datastore.model.ChecklistItemDb
import dk.nsp.testassertions.assertEquals
import dk.nsp.testassertions.assertFalse
import dk.nsp.testassertions.assertNotNull
import dk.nsp.testassertions.assertNull
import dk.nsp.testassertions.assertTrue
import kotlinx.coroutines.flow.first
import kotlinx.coroutines.test.TestScope
import kotlinx.coroutines.test.UnconfinedTestDispatcher
import kotlinx.coroutines.test.runTest
import org.junit.jupiter.api.BeforeEach
import org.junit.jupiter.api.Test
import org.junit.jupiter.api.extension.RegisterExtension
import java.time.LocalDate

class PregnancyPreferencesDataSourceTest {
    private val testScope = TestScope(UnconfinedTestDispatcher())

    private lateinit var subject: PregnancyPreferencesDataSource

    @JvmField
    @RegisterExtension
    val tmpFolder = TemporaryFolderExtension()

    @BeforeEach
    fun setup() {
        subject = PregnancyPreferencesDataSource(
            tmpFolder.testPregnancyPreferencesDataStore(
                filename = "pregnancy_preferences_test.db",
                coroutineScope = testScope,
                serializer = PregnancySerializer(),
            ),
        )
    }

    @Test
    fun pregnancyFieldsInitiallyEmpty() = testScope.runTest {
        with(subject.data.first()) {
            dateOfBirth.assertNull()
            expectedDueDate.assertNull()
            lastMenstruationPeriod.assertNull()
        }
    }

    @Test
    fun setExpectedDueDate() = testScope.runTest {
        val today = LocalDate.now()
        subject.setExpectedDueDate(today)
        subject.data.first().expectedDueDate.assertEquals(today)

        subject.setExpectedDueDate(null)
        subject.data.first().expectedDueDate.assertNull()
    }

    @Test
    fun setLastMenstruationPeriod() = testScope.runTest {
        val lmp = LocalDate.now().minusDays(280)
        subject.setLastMenstruationPeriod(lmp)
        subject.data.first().lastMenstruationPeriod.assertEquals(lmp)

        subject.setLastMenstruationPeriod(null)
        subject.data.first().lastMenstruationPeriod.assertNull()
    }

    @Test
    fun testDateOfBirth() = testScope.runTest {
        val today = LocalDate.now()
        subject.setDateOfBirth(today)
        subject.data.first().dateOfBirth.assertEquals(today)

        subject.setDateOfBirth(null)
        subject.data.first().dateOfBirth.assertNull()
    }

    @Test
    fun testClearingPregnancy() = testScope.runTest {
        subject.setExpectedDueDate(LocalDate.now())
        subject.setDateOfBirth(LocalDate.now())
        subject.setLastMenstruationPeriod(LocalDate.now().minusDays(280))

        subject.clearPregnancy()

        subject.data.first().expectedDueDate.assertNull()
        subject.data.first().dateOfBirth.assertNull()
        subject.data.first().lastMenstruationPeriod.assertNull()
    }

    @Test
    fun testChecklistItemsAreInitiallyEmpty() = testScope.runTest {
        subject.checklists.first().isEmpty().assertTrue()
    }

    private val testChecklist = ChecklistDb(
        id = "1",
        description = "Description",
        title = "Checklist",
        items = listOf(
            ChecklistItemDb(
                id = "2",
                title = "Checklist item",
                checked = false,
                order = 0,
            ),
        ),
    )

    @Test
    fun testInsertingChecklists() = testScope.runTest {
        val testChecklists = listOf(testChecklist, testChecklist.copy(id = "newId"))
        subject.setChecklists(testChecklists)
        subject.checklists.first().size.assertEquals(testChecklists.size)
    }

    @Test
    fun testDeletingChecklistItem() = testScope.runTest {
        subject.setChecklists(listOf(testChecklist))
        subject.checklists.first().first().items.size.assertEquals(1)

        subject.deleteChecklistItem(testChecklist.id.toInt(), testChecklist.items.first())
        subject.checklists.first().first().items.size.assertEquals(0)
    }

    @Test
    fun testTogglingChecklistItem() = testScope.runTest {
        subject.setChecklists(listOf(testChecklist))
        // Initial state should be false
        subject.checklists.first().first().items.first().checked.assertFalse()

        // Toggle checked to true
        subject.toggleCheckedChecklistItem(testChecklist.items.first())
        subject.checklists.first().first().items.first().checked.assertTrue()

        // Toggle back to false
        subject.toggleCheckedChecklistItem(testChecklist.items.first())
        subject.checklists.first().first().items.first().checked.assertFalse()
    }

    @Test
    fun testRecoveringChecklistWithEmptyChecklistItems() = testScope.runTest {
        subject.setChecklists(listOf(testChecklist))
        subject.deleteChecklistItem(testChecklist.id.toInt(), testChecklist.items.first())

        subject.checklists.first().first().items.isEmpty().assertTrue()

        subject.setChecklist(testChecklist)
        subject.checklists.first().first().items.singleOrNull().assertNotNull()
    }

    @Test
    fun testUpdatingChecklistItem() = testScope.runTest {
        subject.setChecklists(listOf(testChecklist))

        subject.updateChecklistItem(testChecklist.id.toInt(), testChecklist.items.first().copy(title = "Edited checklist item"))

        subject.checklists.first().first().items.first().title.assertEquals("Edited checklist item")
    }

    @Test
    fun testInsertingNewChecklistItemIntoChecklist() = testScope.runTest {
        subject.setChecklists(listOf(testChecklist))

        subject.updateChecklistItem(testChecklist.id.toInt(), ChecklistItemDb("NewItem", "New item goes here", false, 1))

        subject.checklists.first().first().items.size.assertEquals(2)
    }
}
