/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2023 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsp.domain.repository

import dk.nsp.domain.pregnancymodels.DueDateMethod
import dk.nsp.domain.pregnancymodels.PregnancyInfo
import dk.nsp.mingraviditet.datastore.PregnancyPreferencesDataSource
import dk.nsp.shared.network.Dispatcher
import dk.nsp.shared.network.MGDispatchers
import kotlinx.coroutines.CoroutineDispatcher
import kotlinx.coroutines.flow.flowOn
import kotlinx.coroutines.flow.map
import kotlinx.coroutines.withContext
import java.time.LocalDate
import javax.inject.Inject

class PregnancyRepository @Inject constructor(
    private val pregnancyDataSource: PregnancyPreferencesDataSource,
    @Dispatcher(MGDispatchers.IO) private val ioDispatcher: CoroutineDispatcher,
) {
    val pregnancyInfo = pregnancyDataSource.data.map { data ->
        val edd = data.expectedDueDate
        val lmp = data.lastMenstruationPeriod

        val dueDateMethod = when {
            edd != null -> DueDateMethod.ExpectedDueDate(edd)
            lmp != null -> DueDateMethod.LastMenstruationPeriod(lmp)
            else -> null
        }

        dueDateMethod?.let {
            PregnancyInfo(
                dueDateMethod = dueDateMethod,
                dateOfBirth = data.dateOfBirth,
            )
        }
    }.flowOn(ioDispatcher)

    suspend fun setExpectedDueDate(date: LocalDate?) = withContext(ioDispatcher) {
        pregnancyDataSource.setExpectedDueDate(date)
    }

    suspend fun setDateOfBirth(date: LocalDate?) = withContext(ioDispatcher) {
        pregnancyDataSource.setDateOfBirth(date)
    }

    suspend fun setLastMenstruationPeriod(date: LocalDate?) = withContext(ioDispatcher) {
        pregnancyDataSource.setLastMenstruationPeriod(date)
    }

    suspend fun clearPregnancy() = withContext(ioDispatcher) { pregnancyDataSource.clearPregnancy() }
}
