/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021-2023 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsp.domain.knowledge

import dk.nsp.api.cms.models.NetworkChecklist
import dk.nsp.api.cms.models.NetworkChecklistItem
import dk.nsp.testassertions.assertEquals
import dk.nsp.testassertions.assertFalse
import dk.nsp.testassertions.assertNotNull
import dk.nsp.testassertions.assertNotNullAndEquals
import org.junit.jupiter.api.Nested
import org.junit.jupiter.api.Test

class ChecklistTest {

    private val nonCheckedChecklistItems = mutableListOf(
        ChecklistItem("0", "1", false, 0),
        ChecklistItem("1", "2", false, 1),
        ChecklistItem("2", "3", false, 2),
        ChecklistItem("3", "4", false, 3),
    )

    private val oneCheckedChecklistItems = mutableListOf(
        ChecklistItem("0", "1", true, 0),
        ChecklistItem("1", "2", false, 1),
        ChecklistItem("2", "3", false, 2),
        ChecklistItem("3", "4", false, 3),
    )

    private val sixChecklistItems = mutableListOf(
        ChecklistItem("0", "1", true, 0),
        ChecklistItem("1", "2", false, 1),
        ChecklistItem("2", "3", false, 2),
        ChecklistItem("3", "4", false, 3),
        ChecklistItem("4", "4", false, 4),
        ChecklistItem("5", "4", false, 5),
    )

    private val emptyCheckList =
        Checklist("0", "Empty items list", "Description", emptyList())
    private val noCheckItemsInChecklist =
        Checklist("0", "Test Checklist", "Test description", nonCheckedChecklistItems)
    private val oneCheckedItemInChecklist =
        Checklist("1", "Checklist with one item selected", null, oneCheckedChecklistItems)
    private val sixCheckedItemsInChecklist =
        Checklist("2", "Checklist with six items", null, sixChecklistItems)

    private val basicCheckListItemNetwork = NetworkChecklistItem(
        id = "ID",
        title = "Title",
    )

    private val basicNetworkChecklist = NetworkChecklist(
        id = 1,
        title = "Title",
        text = "Text",
        items = listOf(basicCheckListItemNetwork),
    )

    @Test
    fun completeness() {
        noCheckItemsInChecklist.completeness().assertEquals(0.0f)
        oneCheckedItemInChecklist.completeness().assertEquals(0.25F)
        sixCheckedItemsInChecklist.completeness().assertEquals(0.16666667F)

        emptyCheckList.completeness().assertEquals(0.0f)
    }

    @Test
    fun completenessAsDivision() {
        noCheckItemsInChecklist.completenessAsDivision().assertEquals("0/4")
        oneCheckedItemInChecklist.completenessAsDivision().assertEquals("1/4")
        sixCheckedItemsInChecklist.completenessAsDivision().assertEquals("1/6")
    }

    @Test
    fun getPercentageAsString() {
        noCheckItemsInChecklist.getPercentageAsString().assertEquals("0%")
        oneCheckedItemInChecklist.getPercentageAsString().assertEquals("25%")
        sixCheckedItemsInChecklist.getPercentageAsString().assertEquals("17%")
    }

    @Test
    fun checkedItems() {
        val checkedItems = Checklist(
            "1",
            "Title",
            "Description",
            listOf(
                ChecklistItem("0", "1", true, 0),
                ChecklistItem("1", "2", false, 1),
                ChecklistItem("2", "3", true, 2),
                ChecklistItem("3", "4", false, 3),
            ),
        ).checkedItems()

        checkedItems.size.assertEquals(2)
        checkedItems.getOrNull(0)?.id?.assertNotNullAndEquals("0")
        checkedItems.getOrNull(1)?.id?.assertNotNullAndEquals("2")
    }

    @Nested
    inner class ListNetworkChecklistToDomainModelListOfCheckedIds {

        val source = listOf(
            basicNetworkChecklist.copy(
                id = 10,
                items = listOf(
                    basicCheckListItemNetwork.copy(id = "1"),
                    basicCheckListItemNetwork.copy(id = "2"),
                ),
            ),
            basicNetworkChecklist.copy(
                id = 11,
                items = listOf(
                    basicCheckListItemNetwork.copy(id = "3"),
                    basicCheckListItemNetwork.copy(id = "4"),
                ),
            ),
            basicNetworkChecklist.copy(
                id = 12,
                items = listOf(
                    basicCheckListItemNetwork.copy(id = "5"),
                    basicCheckListItemNetwork.copy(id = "6"),
                    basicCheckListItemNetwork.copy(id = "7"),
                ),
            ),
        )

        @Test
        fun withCheckedList() {
            val checkedItems = listOf("2", "5", "7")
            val result = source.toDomainModel()

            result.size.assertEquals(3)

            result.getOrNull(0).let { checklist ->
                checklist.assertNotNull()
                checklist.id.assertEquals("10")
                checklist.items.let { items ->
                    items.size.assertEquals(2)

                    // Assert correct items are checked
                    items.getOrNull(0)?.value.assertNotNullAndEquals(false)
                    items.getOrNull(1)?.value.assertNotNullAndEquals(false)
                }
            }

            result.getOrNull(1).let { checklist ->
                checklist.assertNotNull()
                checklist.id.assertEquals("11")
                checklist.items.let { items ->
                    items.size.assertEquals(2)

                    // Assert correct items are checked
                    items.getOrNull(0)?.value.assertNotNullAndEquals(false)
                    items.getOrNull(1)?.value.assertNotNullAndEquals(false)
                }
            }

            result.getOrNull(2).let { checklist ->
                checklist.assertNotNull()
                checklist.id.assertEquals("12")
                checklist.items.let { items ->
                    items.size.assertEquals(3)

                    // Assert correct items are checked
                    items.getOrNull(0)?.value.assertNotNullAndEquals(false)
                    items.getOrNull(1)?.value.assertNotNullAndEquals(false)
                    items.getOrNull(2)?.value.assertNotNullAndEquals(false)
                }
            }
        }

        @Test
        fun withNullCheckedList() {
            val result = source.toDomainModel()

            result.forEach { checklist ->
                checklist.items.forEach { item ->
                    item.value.assertFalse()
                }
            }
        }
    }
}
