/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021-2023 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsp.domain.repository

import dk.nsp.api.cms.CMSDataSource
import dk.nsp.api.cms.models.NetworkArticle
import dk.nsp.api.cms.models.NetworkArticleCategory
import dk.nsp.api.cms.models.NetworkArticleSection
import dk.nsp.api.cms.models.NetworkRecommendedArticles
import dk.nsp.domain.testdoubles.TestArticleCategoryDao
import dk.nsp.domain.testdoubles.TestArticleDao
import dk.nsp.domain.testdoubles.TestArticleSectionDao
import dk.nsp.mingraviditet.model.data.ArticleCategory
import dk.nsp.shared.network.Failed
import dk.nsp.shared.network.Success
import dk.nsp.shared.network.toSuccess
import dk.nsp.testassertions.assertEquals
import dk.nsp.testassertions.assertIs
import dk.nsp.testassertions.assertIsAndApply
import dk.nsp.testassertions.assertNotNullAndApply
import io.mockk.coEvery
import io.mockk.mockk
import kotlinx.coroutines.runBlocking
import kotlinx.coroutines.test.UnconfinedTestDispatcher
import org.junit.Before
import org.junit.jupiter.api.Test
import java.time.LocalDateTime

class ArticlesRepositoryTest {

    private val defaultNetworkArticleDtosSuccess = listOf(
        NetworkArticleCategory(
            1,
            "1",
            "",
            null,
            articles = listOf(
                NetworkArticle(
                    id = 11,
                    "",
                    1,
                    LocalDateTime.now(),
                    "",
                    content = "Content",
                    sections = listOf(
                        NetworkArticleSection(
                            id = 111,
                            "111",
                            2,
                            "",
                            null,
                        ),
                    ),
                ),
            ),
            "",
        ),
        NetworkArticleCategory(
            2,
            "2",
            "",
            null,
            articles = listOf(
                NetworkArticle(
                    id = 22,
                    "",
                    3,
                    LocalDateTime.now(),
                    "",
                    content = "Content",
                    sections = listOf(
                        NetworkArticleSection(
                            id = 222,
                            "222",
                            4,
                            "",
                            null,
                        ),
                    ),
                ),
            ),
            "",
        ),
    ).toSuccess()

    private val defaultRecommendedArticleDtoSuccess = NetworkRecommendedArticles(
        currentWeek = defaultNetworkArticleDtosSuccess.value,
        nextWeek = emptyList(),
    ).toSuccess()

    private var articleCategoryDao: TestArticleCategoryDao = TestArticleCategoryDao()
    private var articleDao: TestArticleDao = TestArticleDao()
    private var articleSectionDao: TestArticleSectionDao = TestArticleSectionDao()

    private val mockedDataSource = mockk<CMSDataSource> {
        coEvery {
            getArticleSections()
        } returns defaultNetworkArticleDtosSuccess

        coEvery {
            getAfterBirthArticleCategories(any())
        } returns defaultRecommendedArticleDtoSuccess

        coEvery {
            getArticleCategories(any())
        } returns defaultRecommendedArticleDtoSuccess

        coEvery {
            searchArticles(any())
        } returns defaultNetworkArticleDtosSuccess
    }

    private val failingController = mockk<CMSDataSource> {
        coEvery {
            getArticleSections()
        } returns Failed(Exception("failed to get article sections"))

        coEvery {
            getAfterBirthArticleCategories(any())
        } returns Failed(Exception("failed to get after birth"))

        coEvery {
            getArticleCategories(any())
        } returns Failed(Exception("failed to article categories"))

        coEvery {
            searchArticles(any())
        } returns Failed(Exception("failed to search"))
    }

    @Before
    fun setup() {
        articleCategoryDao = TestArticleCategoryDao()
        articleDao = TestArticleDao()
        articleSectionDao = TestArticleSectionDao()
    }

    private fun getRepo(): ArticlesRepository {
        return ArticlesRepository(
            cmsDataSource = mockedDataSource,
            ioDispatcher = UnconfinedTestDispatcher(),
            articleCategoriesDao = articleCategoryDao,
            articlesDao = articleDao,
            articleSectionDao = articleSectionDao,
        )
    }

    //
    private fun getFailingRepo(): ArticlesRepository {
        return ArticlesRepository(
            cmsDataSource = failingController,
            ioDispatcher = UnconfinedTestDispatcher(),
            articleCategoriesDao = articleCategoryDao,
            articlesDao = articleDao,
            articleSectionDao = articleSectionDao,
        )
    }

//    @Test
//    fun getArticleSections() = runBlocking {
//        getRepo().getArticleSections()
//            .assertIsAndApply<Success<List<ArticleCategory>>> {
//                value.size.assertEquals(2)
//                value.getOrNull(0).assertNotNullAndApply {
//                    name.assertEquals("1")
//                    articles.getOrNull(0).assertNotNullAndApply {
//                        id.assertEquals(11)
//                        sections.getOrNull(0).assertNotNullAndApply {
//                            id.assertEquals(111)
//                        }
//                    }
//                }
//                value.getOrNull(1).assertNotNullAndApply {
//                    name.assertEquals("2")
//                    articles.getOrNull(0).assertNotNullAndApply {
//                        id.assertEquals(22)
//                        sections.getOrNull(0).assertNotNullAndApply {
//                            id.assertEquals(222)
//                        }
//                    }
//                }
//            }
//
//        getFailingRepo().getArticleSections()
//            .assertIs<Failed>()
//    }

    @Test
    fun getAfterBirthArticleCategories() = runBlocking {
//        with(getRepo()) { this.getAfterBirthArticles(0) }
//            .assertIsAndApply<Success<OverviewArticles>> {
//                val value = this.value.currentWeek
//                value.size.assertEquals(2)
// //                value.getOrNull(0).assertNotNullAndApply {
// //
// //                    title.assertEquals("1")
// //
// //                    articles.getOrNull(0).assertNotNullAndApply {
// //                        id.assertEquals(11)
// //                        sections.getOrNull(0).assertNotNullAndApply {
// //                            id.assertEquals(111)
// //                        }
// //                    }
// //                }
// //                value.getOrNull(1).assertNotNullAndApply {
// //                    name.assertEquals("2")
// //                    articles.getOrNull(0).assertNotNullAndApply {
// //                        id.assertEquals(22)
// //                        sections.getOrNull(0).assertNotNullAndApply {
// //                            id.assertEquals(222)
// //                        }
// //                    }
// //                }
//            }

        with(getFailingRepo()) { this.getAfterBirthArticles(0) }.assertIs<Failed>()
    }

//    @Test
//    fun getOverviewArticles() = runBlocking {
//        with(getRepo()) { this.getOverviewArticles(2) }
//            .assertIsAndApply<Success<OverviewArticles>> {
//                val value = this.value.currentWeek
//                value.size.assertEquals(2)
//                value.getOrNull(0).assertNotNullAndApply {
//
//                    name.assertEquals("1")
//                    articles.getOrNull(0).assertNotNullAndApply {
//                        id.assertEquals(11)
//                        sections.getOrNull(0).assertNotNullAndApply {
//                            id.assertEquals(111)
//                        }
//                    }
//                }
//                value.getOrNull(1).assertNotNullAndApply {
//                    name.assertEquals("2")
//                    articles.getOrNull(0).assertNotNullAndApply {
//                        id.assertEquals(22)
//                        sections.getOrNull(0).assertNotNullAndApply {
//                            id.assertEquals(222)
//                        }
//                    }
//                }
//            }
//
//        with(getFailingRepo()) { this.getArticleSections() }
//            .assertIs<Failed>()
//    }

    @Test
    fun searchArticles() = runBlocking {
        getRepo().search("")
            .assertIsAndApply<Success<List<ArticleCategory>>> {
                value.size.assertEquals(2)
                value.getOrNull(0).assertNotNullAndApply {
                    name.assertEquals("1")
                    articles.getOrNull(0).assertNotNullAndApply {
                        id.assertEquals(11)
                        sections.getOrNull(0).assertNotNullAndApply {
                            id.assertEquals(111)
                        }
                    }
                }
                value.getOrNull(1).assertNotNullAndApply {
                    name.assertEquals("2")
                    articles.getOrNull(0).assertNotNullAndApply {
                        id.assertEquals(22)
                        sections.getOrNull(0).assertNotNullAndApply {
                            id.assertEquals(222)
                        }
                    }
                }
            }

        getFailingRepo().search("")
            .assertIs<Failed>()
    }
}
