/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2023 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsp.mingraviditet.gravidcompose.component

import androidx.compose.animation.core.AnimationSpec
import androidx.compose.animation.core.AnimationState
import androidx.compose.animation.core.DecayAnimationSpec
import androidx.compose.animation.core.animateDecay
import androidx.compose.animation.core.animateTo
import androidx.compose.foundation.gestures.Orientation
import androidx.compose.foundation.gestures.draggable
import androidx.compose.foundation.gestures.rememberDraggableState
import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.RowScope
import androidx.compose.foundation.layout.WindowInsets
import androidx.compose.foundation.layout.height
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.windowInsetsPadding
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.filled.ArrowBack
import androidx.compose.material.icons.filled.Check
import androidx.compose.material3.Divider
import androidx.compose.material3.ExperimentalMaterial3Api
import androidx.compose.material3.IconButton
import androidx.compose.material3.LocalContentColor
import androidx.compose.material3.ProvideTextStyle
import androidx.compose.material3.Surface
import androidx.compose.material3.Text
import androidx.compose.material3.TopAppBarDefaults
import androidx.compose.material3.TopAppBarScrollBehavior
import androidx.compose.material3.TopAppBarState
import androidx.compose.runtime.Composable
import androidx.compose.runtime.CompositionLocalProvider
import androidx.compose.runtime.SideEffect
import androidx.compose.runtime.derivedStateOf
import androidx.compose.runtime.getValue
import androidx.compose.runtime.remember
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.graphics.toArgb
import androidx.compose.ui.layout.AlignmentLine
import androidx.compose.ui.layout.LastBaseline
import androidx.compose.ui.layout.Layout
import androidx.compose.ui.layout.layoutId
import androidx.compose.ui.platform.LocalDensity
import androidx.compose.ui.text.TextStyle
import androidx.compose.ui.text.font.FontWeight
import androidx.compose.ui.unit.Constraints
import androidx.compose.ui.unit.Dp
import androidx.compose.ui.unit.Velocity
import androidx.compose.ui.unit.dp
import androidx.compose.ui.unit.sp
import androidx.core.graphics.ColorUtils
import dk.nsp.mingraviditet.gravidcompose.theme.GravidTheme
import dk.nsp.mingraviditet.gravidcompose.utils.GravidPreview
import kotlin.math.abs
import kotlin.math.max
import kotlin.math.roundToInt

internal object GravidTopAppBarDefaults {
    val containerMaxHeight = 112.dp
    val containerCollapsedHeight = 64.dp
    val titleBottomPadding = 24.dp
    val horizontalPadding = 4.dp
    val titleInset = 16.dp - horizontalPadding

    val windowInsets: WindowInsets
        @Composable
        get() = TopAppBarDefaults.windowInsets

    fun maxHeight(collapsed: Boolean): Dp {
        return if (collapsed) {
            containerCollapsedHeight
        } else {
            containerMaxHeight
        }
    }
}

@Composable
fun GravidExpandableTopBar(
    title: @Composable () -> Unit,
    modifier: Modifier = Modifier,
    navigationIcon: @Composable () -> Unit = {},
    actions: @Composable RowScope.() -> Unit = {},
    subtitle: @Composable (() -> Unit)? = null,
    scrollBehavior: TopAppBarScrollBehavior? = null,
    collapsed: Boolean = scrollBehavior == null,
    allowBarDrag: Boolean = true,
    windowInsets: WindowInsets = GravidTopAppBarDefaults.windowInsets,
    titleBottomPadding: Dp = GravidTopAppBarDefaults.titleBottomPadding,
    maxHeight: Dp = GravidTopAppBarDefaults.maxHeight(collapsed),
    collapsedHeight: Dp = GravidTopAppBarDefaults.containerCollapsedHeight,
    titleHorizontalArrangement: Arrangement.Horizontal = Arrangement.Start, // position when collapsed
    expandedTitleStyle: TextStyle = GravidTheme.typography.Hero,
    collapsedTitleStyle: TextStyle = GravidTheme.typography.BodyStandard,
    expandedTitleColor: Color = GravidTheme.colorScheme.onBackground,
    collapsedTitleColor: Color = GravidTheme.colorScheme.onBackground,
    collapsedBorder: Dp = 1.dp,
    collapsedBorderColor: Color = GravidTheme.colorScheme.outline,
    expandedBackgroundTransparent: Boolean = false,
) {
    val colors = GravidTheme.colorScheme

    val collapsedFraction by remember {
        derivedStateOf {
            if (collapsed) {
                1f
            } else {
                scrollBehavior?.state?.collapsedFraction ?: 1f
            }
        }
    }

    val shadowCollapsedFraction by remember { derivedStateOf { scrollBehavior?.state?.collapsedFraction ?: 1f } }
    val borderColor by remember {
        derivedStateOf {
            if (expandedBackgroundTransparent) {
                collapsedBorderColor.copy(alpha = collapsedFraction)
            } else {
                val colorInt = ColorUtils.blendARGB(
//                    Color.Transparent.toArgb(),
                    colors.background.toArgb(),
                    collapsedBorderColor.toArgb(),
                    shadowCollapsedFraction,
                )
                Color(colorInt)
            }
        }
    }
    val dividerHeight by remember(collapsedBorder) {
        derivedStateOf { collapsedBorder.value * collapsedFraction }
    }

    val collapsedFontSize = collapsedTitleStyle.fontSize
    val distance = expandedTitleStyle.fontSize.value - collapsedFontSize.value

    val fontSize by remember {
        derivedStateOf { (expandedTitleStyle.fontSize.value - (distance * collapsedFraction)).sp }
    }
    val titleColor by remember {
        derivedStateOf {
            val colorInt = ColorUtils.blendARGB(
                expandedTitleColor.toArgb(),
                collapsedTitleColor.toArgb(),
                collapsedFraction,
            )
            Color(colorInt)
        }
    }

    val subtitleColor by remember {
        derivedStateOf {
            colors.onBackground.copy(alpha = 1 - (collapsedFraction * 1.5f).coerceAtMost(1f))
        }
    }

    val topAppBarBackgroundColor by remember {
        derivedStateOf {
            colors.surface.copy(alpha = if (expandedBackgroundTransparent) collapsedFraction else 1f)
        }
    }

    val pinnedHeightPx: Float
    val maxHeightPx: Float
    val titleBottomPaddingPx: Int
    val titlePadding: Int
    LocalDensity.current.run {
        pinnedHeightPx = collapsedHeight.toPx()
        maxHeightPx = maxHeight.toPx()
        titleBottomPaddingPx = titleBottomPadding.roundToPx()
        titlePadding = GravidTopAppBarDefaults.titleInset.roundToPx()
    }

    // Sets the app bar's height offset limit to hide just the bottom title area and keep top title
    // visible when collapsed.
    SideEffect {
        if (scrollBehavior?.state?.heightOffsetLimit != pinnedHeightPx - maxHeightPx) {
            scrollBehavior?.state?.heightOffsetLimit = pinnedHeightPx - maxHeightPx
        }
    }

    // Wrap the given actions in a Row.
    val actionsRow = @Composable {
        Row(
            horizontalArrangement = Arrangement.End,
            verticalAlignment = Alignment.CenterVertically,
            content = actions,
        )
    }

    // Set up support for resizing the top app bar when vertically dragging the bar itself.
    val appBarDragModifier = if (scrollBehavior != null && !scrollBehavior.isPinned && allowBarDrag) {
        Modifier.draggable(
            orientation = Orientation.Vertical,
            state = rememberDraggableState { delta ->
                scrollBehavior.state.heightOffset = scrollBehavior.state.heightOffset + delta
            },
            onDragStopped = { velocity ->
                settleAppBar(
                    scrollBehavior.state,
                    velocity,
                    scrollBehavior.flingAnimationSpec,
                    scrollBehavior.snapAnimationSpec,
                )
            },
        )
    } else {
        Modifier
    }

    val height = with(LocalDensity.current) {
        scrollBehavior?.state?.let {
            (maxHeightPx + (it.heightOffset)).toDp()
        } ?: collapsedHeight
    }

    Column(modifier = modifier.then(appBarDragModifier)) {
        Surface(modifier = Modifier, color = topAppBarBackgroundColor) {
            Layout(
                modifier = Modifier
                    .windowInsetsPadding(windowInsets)
                    .height(height),
                content =
                {
                    Box(
                        Modifier
                            .layoutId("navigationIcon")
                            .padding(start = GravidTopAppBarDefaults.horizontalPadding),
                    ) {
                        CompositionLocalProvider(
                            LocalContentColor provides colors.primary,
                            content = navigationIcon,
                        )
                    }
                    Box(
                        Modifier
                            .layoutId("subtitle")
                            .padding(horizontal = GravidTopAppBarDefaults.horizontalPadding),
                    ) {
                        ProvideTextStyle(
                            value = GravidTheme.typography.BodyStandard.copy(color = subtitleColor),
                        ) {
                            subtitle?.let {
                                CompositionLocalProvider(
                                    LocalContentColor provides titleColor,
                                    content = subtitle,
                                )
                            }
                        }
                    }
                    Box(
                        Modifier
                            .layoutId("title")
                            .padding(horizontal = GravidTopAppBarDefaults.horizontalPadding),
                    ) {
                        ProvideTextStyle(
                            value = expandedTitleStyle.copy(
                                fontSize = fontSize,
                                fontWeight = FontWeight.Bold,
                            ),
                        ) {
                            CompositionLocalProvider(
                                LocalContentColor provides subtitleColor,
                                content = title,
                            )
                        }
                    }
                    Box(
                        Modifier
                            .layoutId("actionIcons")
                            .padding(end = GravidTopAppBarDefaults.horizontalPadding),
                    ) {
                        CompositionLocalProvider(
                            LocalContentColor provides colors.primary,
                            content = actionsRow,
                        )
                    }
                },
                measurePolicy = { measurables, constraints ->
                    val navigationIconPlaceable = measurables.first { it.layoutId == "navigationIcon" }
                        .measure(constraints.copy(minWidth = 0, minHeight = 0))
                    val actionIconsPlaceable = measurables.first { it.layoutId == "actionIcons" }
                        .measure(constraints.copy(minWidth = 0, minHeight = 0))

                    val maxTitleWidth = if (constraints.maxWidth == Constraints.Infinity) {
                        constraints.maxWidth
                    } else {
                        (
                            constraints.maxWidth -
                                (navigationIconPlaceable.width + actionIconsPlaceable.width).times(
                                    collapsedFraction,
                                ).roundToInt() -
                                (titlePadding * 2 * (1f - collapsedFraction)).roundToInt()
                            )
                            .coerceAtLeast(0)
                    }

                    val subtitlePlaceable = measurables.first { it.layoutId == "subtitle" }.measure(
                        constraints.copy(
                            minWidth = 0,
                            maxWidth = maxTitleWidth,
                            minHeight = 0,
                        ),
                    )

                    val titlePlaceable = measurables
                        .first { it.layoutId == "title" }
                        .measure(
                            constraints.copy(
                                minWidth = 0,
                                maxWidth = maxTitleWidth,
                                minHeight = 0,
                            ),
                        )

                    val titleBaseline = if (titlePlaceable[LastBaseline] != AlignmentLine.Unspecified) {
                        titlePlaceable[LastBaseline]
                    } else {
                        0
                    }

                    layout(
                        constraints.maxWidth,
                        constraints.maxHeight,
                    ) {
                        val collapsedLayoutHeight = pinnedHeightPx.roundToInt()
                        val expandedLayoutHeight = maxHeightPx.roundToInt()
                        val collapsedPositionY = (collapsedLayoutHeight - titlePlaceable.height) / 2f
                        val expandedPositionY: Int = when {
                            expandedLayoutHeight == collapsedLayoutHeight -> collapsedPositionY.roundToInt()
                            else ->
                                expandedLayoutHeight - titlePlaceable.measuredHeight -
                                    max(0, titleBottomPaddingPx - titlePlaceable.height + titleBaseline)
                        }

                        val collapsedPositionX = when (titleHorizontalArrangement) {
                            Arrangement.Center -> (constraints.maxWidth - titlePlaceable.width) / 2
                            Arrangement.End ->
                                constraints.maxWidth - titlePlaceable.width - actionIconsPlaceable.width
                            // Arrangement.Start.
                            // An TopAppBarTitleInset will make sure the title is offset in case the
                            // navigation icon is missing.
                            else -> max(
                                GravidTopAppBarDefaults.titleInset.roundToPx(),
                                navigationIconPlaceable.width,
                            )
                        }
                        val expandedPositionX = GravidTopAppBarDefaults.titleInset.roundToPx()
                        val xDistance = collapsedPositionX - expandedPositionX
                        val titlePositionX = expandedPositionX + (xDistance * collapsedFraction).roundToInt()
                        val subtitlePositionX = expandedPositionX + (xDistance * collapsedFraction).roundToInt()

                        val yDistance = (expandedPositionY - collapsedPositionY)

                        val titlePositionY = expandedPositionY - (yDistance * collapsedFraction).roundToInt()
                        val subtitlePositionY = (titlePositionY - subtitlePlaceable.measuredHeight) - collapsedHeight.roundToPx() / 2 * collapsedFraction

                        // Navigation icon
                        navigationIconPlaceable.placeRelative(
                            x = 0,
                            y = ((collapsedLayoutHeight - navigationIconPlaceable.height) / 2f).roundToInt(),
                        )

                        // Subtitle
                        subtitlePlaceable.placeRelative(
                            x = subtitlePositionX,
                            y = subtitlePositionY.roundToInt(),
                        )

                        // Title
                        titlePlaceable.placeRelative(
                            x = titlePositionX,
                            y = titlePositionY,
                        )

                        // Action icons
                        actionIconsPlaceable.placeRelative(
                            x = constraints.maxWidth - actionIconsPlaceable.width,
                            y = ((collapsedLayoutHeight - actionIconsPlaceable.height) / 2f).roundToInt(),
                        )
                    }
                },
            )
        }

        Divider(thickness = dividerHeight.dp, color = borderColor)
    }
}

@OptIn(ExperimentalMaterial3Api::class)
private suspend fun settleAppBar(
    state: TopAppBarState,
    velocity: Float,
    flingAnimationSpec: DecayAnimationSpec<Float>?,
    snapAnimationSpec: AnimationSpec<Float>?,
): Velocity {
    // Check if the app bar is completely collapsed/expanded. If so, no need to settle the app bar,
    // and just return Zero Velocity.
    // Note that we don't check for 0f due to float precision with the collapsedFraction
    // calculation.
    if (state.collapsedFraction < 0.01f || state.collapsedFraction == 1f) {
        return Velocity.Zero
    }
    var remainingVelocity = velocity
    // In case there is an initial velocity that was left after a previous user fling, animate to
    // continue the motion to expand or collapse the app bar.
    if (flingAnimationSpec != null && abs(velocity) > 1f) {
        var lastValue = 0f
        AnimationState(
            initialValue = 0f,
            initialVelocity = velocity,
        )
            .animateDecay(flingAnimationSpec) {
                val delta = value - lastValue
                val initialHeightOffset = state.heightOffset
                state.heightOffset = initialHeightOffset + delta
                val consumed = abs(initialHeightOffset - state.heightOffset)
                lastValue = value
                remainingVelocity = this.velocity
                // avoid rounding errors and stop if anything is unconsumed
                if (abs(delta - consumed) > 0.5f) this.cancelAnimation()
            }
    }
    // Snap if animation specs were provided.
    if (snapAnimationSpec != null) {
        if (state.heightOffset < 0 &&
            state.heightOffset > state.heightOffsetLimit
        ) {
            AnimationState(initialValue = state.heightOffset).animateTo(
                if (state.collapsedFraction < 0.5f) {
                    0f
                } else {
                    state.heightOffsetLimit
                },
                animationSpec = snapAnimationSpec,
            ) { state.heightOffset = value }
        }
    }

    return Velocity(0f, remainingVelocity)
}

@GravidPreview
@Composable
private fun GravidExpandableTopBarPreview() {
    GravidTheme {
        Column(verticalArrangement = Arrangement.spacedBy(16.dp)) {
            GravidExpandableTopBar(
                scrollBehavior = TopAppBarDefaults.exitUntilCollapsedScrollBehavior(),
                titleHorizontalArrangement = Arrangement.Start,
                title = { Text("Test title goes here") },
                subtitle = { Text("Subtitle goes here") },
            )

            GravidExpandableTopBar(
                titleHorizontalArrangement = Arrangement.Start,
                title = { Text("Test title goes here") },
                subtitle = { Text("Subtitle goes here") },
                actions = {
                    GravidIcon(imageVector = Icons.Default.Check)
                },
                navigationIcon = {
                    IconButton(onClick = { /*TODO*/ }) {
                        GravidIcon(imageVector = Icons.Default.ArrowBack)
                    }
                },
            )
        }
    }
}
