/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2023 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsp.mingraviditet.gravidcompose.component

import androidx.compose.foundation.background
import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.padding
import androidx.compose.material3.ColorScheme
import androidx.compose.material3.Text
import androidx.compose.runtime.Composable
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.geometry.Offset
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.graphics.Shadow
import androidx.compose.ui.text.AnnotatedString
import androidx.compose.ui.text.ParagraphStyle
import androidx.compose.ui.text.SpanStyle
import androidx.compose.ui.text.TextStyle
import androidx.compose.ui.text.font.FontStyle
import androidx.compose.ui.text.style.TextAlign
import androidx.compose.ui.text.style.TextIndent
import androidx.compose.ui.text.style.TextOverflow
import androidx.compose.ui.tooling.preview.Preview
import androidx.compose.ui.tooling.preview.PreviewParameter
import androidx.compose.ui.unit.dp
import androidx.compose.ui.unit.sp
import dk.nsp.mingraviditet.gravidcompose.theme.GravidTheme
import dk.nsp.mingraviditet.gravidcompose.utils.PreviewColorSchemeProvider

@Composable
fun GravidText(
    text: CharSequence,
    modifier: Modifier = Modifier,
    style: TextStyle = GravidTheme.typography.BodyStandard,
    overflow: TextOverflow = TextOverflow.Clip,
    maxLines: Int = Int.MAX_VALUE,
    color: Color = GravidTheme.colorScheme.onBackground,
    textAlign: TextAlign? = null,
) {
    when (text) {
        is String -> {
            Text(
                text = text,
                style = style,
                modifier = modifier,
                overflow = overflow,
                maxLines = maxLines,
                color = color,
                textAlign = textAlign,
            )
        }

        is AnnotatedString -> {
            Text(
                text = text,
                style = style,
                modifier = modifier,
                overflow = overflow,
                maxLines = maxLines,
                color = color,
                textAlign = textAlign,
            )
        }

        else -> throw IllegalArgumentException("Unknown CharSequence type. Only String and Annotated String is supported: ${text::class.java}")
    }
}

@Preview
@Composable
fun GravidTextPreview(
    @PreviewParameter(PreviewColorSchemeProvider::class) colorScheme: ColorScheme,
) {
    GravidTheme(overrideColorScheme = colorScheme) {
        Column(
            Modifier
                .background(GravidTheme.colorScheme.background)
                .padding(GravidTheme.dimens.largeSpacing),
            verticalArrangement = Arrangement.spacedBy(16.dp),
            horizontalAlignment = Alignment.CenterHorizontally,
        ) {
            GravidText(text = "Det er vigtigt, at du som gravid eller fødende oplyser, om du eller din partner har symptomer på coronavirus")

            GravidText(
                text = AnnotatedString(
                    text = "Det er vigtigt, at du som gravid eller fødende oplyser, om du eller din partner har symptomer på coronavirus",
                    spanStyles = listOf(
                        AnnotatedString.Range(
                            SpanStyle(
                                fontStyle = FontStyle.Italic,
                                shadow = Shadow(GravidTheme.colorScheme.error, offset = Offset(5f, 5f)),
                            ),
                            0,
                            5,
                        ),
                    ),
                    // create two paragraphs with different alignment and indent settings.
                    paragraphStyles = listOf(
                        AnnotatedString.Range(ParagraphStyle(textAlign = TextAlign.Center), 0, 6),
                        AnnotatedString.Range(ParagraphStyle(textIndent = TextIndent(25.sp)), 6, 15),
                    ),
                ),
            )
        }
    }
}
