/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2023 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsp.mingraviditet.gravidcompose.component

import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.shape.RoundedCornerShape
import androidx.compose.foundation.text.KeyboardActions
import androidx.compose.foundation.text.KeyboardOptions
import androidx.compose.foundation.text.selection.TextSelectionColors
import androidx.compose.material3.TextField
import androidx.compose.material3.TextFieldDefaults
import androidx.compose.runtime.Composable
import androidx.compose.runtime.LaunchedEffect
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.setValue
import androidx.compose.ui.Modifier
import androidx.compose.ui.focus.FocusRequester
import androidx.compose.ui.focus.focusRequester
import androidx.compose.ui.focus.onFocusChanged
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.text.TextRange
import androidx.compose.ui.text.input.KeyboardCapitalization
import androidx.compose.ui.text.input.TextFieldValue
import androidx.compose.ui.text.style.TextAlign
import androidx.compose.ui.unit.dp
import dk.nsp.mingraviditet.gravidcompose.theme.GravidTheme
import dk.nsp.mingraviditet.gravidcompose.utils.GravidPreview
import dk.nsp.mingraviditet.gravidcompose.utils.innerShadow

@Composable
fun GravidTextField(
    value: TextFieldValue,
    onValueChange: (TextFieldValue) -> Unit,
    modifier: Modifier = Modifier,
    keyboardOptions: KeyboardOptions = KeyboardOptions(capitalization = KeyboardCapitalization.Sentences),
    keyboardActions: KeyboardActions = KeyboardActions.Default,
    maxLength: Int = Int.MAX_VALUE,
    leadingIcon: @Composable (() -> Unit)? = null,
    trailingIcon: @Composable (() -> Unit)? = null,
) {
    val focusRequester = remember { FocusRequester() }

    var tfv by remember {
        mutableStateOf(value)
    }

    Column(Modifier.fillMaxWidth()) {
        TextField(
            modifier = modifier
                .fillMaxWidth()
                .innerShadow(color = Color.Black.copy(alpha = 0.4f), cornersRadius = 4.dp, spread = 4.dp, blur = 5.dp)
                .focusRequester(focusRequester)
                .onFocusChanged {
                    if (it.isFocused) {
                        if (tfv.text.isNotEmpty()) {
                            tfv = tfv.copy(selection = TextRange(tfv.text.length))
                        }
                    }
                },
            value = tfv,
            onValueChange = { it: TextFieldValue ->
                if (it.text.length <= maxLength) {
                    tfv = it
                    onValueChange(tfv)
                }
            },
            singleLine = true,
            keyboardOptions = keyboardOptions,
            textStyle = GravidTheme.typography.BodyStandard,
            shape = RoundedCornerShape(4.dp),
            colors = TextFieldDefaults.colors(
                cursorColor = GravidTheme.colorScheme.tertiary,
                focusedContainerColor = GravidTheme.colorScheme.tertiaryContainer,
                unfocusedContainerColor = GravidTheme.colorScheme.tertiaryContainer,
                focusedIndicatorColor = Color.Transparent,
                unfocusedIndicatorColor = Color.Transparent,
                errorContainerColor = GravidTheme.colorScheme.errorContainer,
                errorIndicatorColor = Color.Transparent,
                errorCursorColor = GravidTheme.colorScheme.tertiary,
                selectionColors = TextSelectionColors(
                    handleColor = GravidTheme.colorScheme.tertiary,
                    backgroundColor = GravidTheme.colorScheme.tertiary.copy(alpha = 0.2f),
                ),
            ),
            keyboardActions = keyboardActions,
            leadingIcon = leadingIcon,
            trailingIcon = trailingIcon,
        )

        if (maxLength < Int.MAX_VALUE) {
            GravidText(
                text = "${tfv.text.length} / $maxLength",
                modifier = Modifier.fillMaxWidth().padding(top = GravidTheme.dimens.trumpSpacing),
                textAlign = TextAlign.End,
            )
        }
    }

    LaunchedEffect(Unit) {
        focusRequester.requestFocus()
    }
}

@GravidPreview
@Composable
private fun GravidTextFieldPreview() {
    var value by remember {
        mutableStateOf("Value goes here")
    }
    GravidTheme {
        Column(verticalArrangement = Arrangement.spacedBy(16.dp)) {
            GravidTextField(
                value = TextFieldValue(value),
                onValueChange = { value = it.text },
            )

            GravidTextField(
                value = TextFieldValue(value),
                onValueChange = { value = it.text },
                maxLength = 10,
            )
        }
    }
}
