/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2023 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsp.mingraviditet.gravidcompose.component

import android.content.res.Configuration.UI_MODE_NIGHT_YES
import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Column
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.filled.ArrowBack
import androidx.compose.material.icons.filled.Close
import androidx.compose.material3.CenterAlignedTopAppBar
import androidx.compose.material3.Icon
import androidx.compose.material3.IconButton
import androidx.compose.material3.TopAppBar
import androidx.compose.material3.TopAppBarColors
import androidx.compose.material3.TopAppBarDefaults
import androidx.compose.runtime.Composable
import androidx.compose.ui.Modifier
import androidx.compose.ui.text.TextStyle
import androidx.compose.ui.tooling.preview.Preview
import androidx.compose.ui.unit.dp
import dk.nsp.mingraviditet.gravidcompose.theme.GravidTheme

@Composable
fun GravidTopBar(
    title: CharSequence,
    modifier: Modifier = Modifier,
    titleTextStyle: TextStyle = GravidTheme.typography.Header4,
    colors: TopAppBarColors = TopAppBarDefaults.topAppBarColors(
        containerColor = GravidTheme.colorScheme.surface,
    ),
    onNavigationClick: (() -> Unit)? = null,
) {
    Column(modifier = modifier) {
        TopAppBar(
            colors = colors,
            title = {
                GravidText(
                    text = title,
                    style = titleTextStyle,
                )
            },
            navigationIcon = {
                if (onNavigationClick != null) {
                    BackButton(onClick = onNavigationClick)
                }
            },
        )

        GravidDivider()
    }
}

@Composable
fun GravidDialogTopAppBar(
    title: CharSequence,
    modifier: Modifier = Modifier,
    colors: TopAppBarColors = TopAppBarDefaults.centerAlignedTopAppBarColors(containerColor = GravidTheme.colorScheme.surface),
    titleTextStyle: TextStyle = GravidTheme.typography.Header4,
    onNavigationClick: () -> Unit = {},
) {
    Column(modifier = modifier) {
        CenterAlignedTopAppBar(
            colors = colors,
            title = {
                GravidText(
                    text = title,
                    style = titleTextStyle,
                )
            },
            actions = {
                CloseButton(onNavigationClick)
            },
        )
        GravidDivider()
    }
}

@Composable
fun BackButton(
    onClick: () -> Unit,
) {
    IconButton(onClick = onClick) {
        Icon(
            imageVector = Icons.Default.ArrowBack,
            contentDescription = "Tilbage",
            tint = GravidTheme.colorScheme.onSurface,
        )
    }
}

@Composable
fun CloseButton(
    onClick: () -> Unit,
) {
    IconButton(onClick = onClick) {
        Icon(
            imageVector = Icons.Default.Close,
            contentDescription = "Luk",
            tint = GravidTheme.colorScheme.onSurface,
        )
    }
}

@Preview
@Preview(uiMode = UI_MODE_NIGHT_YES)
@Composable
fun GravidTopBarPreview() {
    GravidTheme {
        Column(verticalArrangement = Arrangement.spacedBy(16.dp)) {
            GravidTopBar(title = "Empty top bar")

            GravidTopBar(
                title = "TopBar with back",
                onNavigationClick = {
                },
            )
        }
    }
}
