/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2023 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsp.mingraviditet.gravidcompose.theme

import androidx.compose.foundation.isSystemInDarkTheme
import androidx.compose.material.ripple.LocalRippleTheme
import androidx.compose.material.ripple.RippleAlpha
import androidx.compose.material.ripple.RippleTheme
import androidx.compose.material3.ColorScheme
import androidx.compose.material3.MaterialTheme
import androidx.compose.runtime.Composable
import androidx.compose.runtime.CompositionLocalProvider
import androidx.compose.runtime.staticCompositionLocalOf
import androidx.compose.ui.graphics.Color

@Composable
internal fun GravidThemePropertiesProvider(
    dimens: GravidDimens,
    typography: GravidTypography,
    colorScheme: ColorScheme,
    gravidColorSystem: GravidColorSystem,
    content: @Composable () -> Unit,
) {
    CompositionLocalProvider(
        LocalGravidDimens provides dimens,
        LocalGravidTypography provides typography,
        LocalGravidColors provides colorScheme,
        LocalRippleTheme provides GravidRippleTheme,
        LocalGravidColorSystem provides gravidColorSystem,

        content = content,
    )
}

private object GravidRippleTheme : RippleTheme {
    @Composable
    override fun defaultColor(): Color = GravidTheme.colorScheme.onSurface

    @Composable
    override fun rippleAlpha(): RippleAlpha {
        val pressedAlpha = if (isSystemInDarkTheme()) {
            0.36f
        } else {
            0.24f
        }

        return RippleAlpha(
            pressedAlpha = pressedAlpha,
            focusedAlpha = 0.24f,
            draggedAlpha = 0.16f,
            hoveredAlpha = 0.08f,
        )
    }
}

private val LocalGravidDimens = staticCompositionLocalOf<GravidDimens> {
    error("No dimens found")
}
private val LocalGravidTypography = staticCompositionLocalOf<GravidTypography> {
    error("No typography found")
}
private val LocalGravidColors = staticCompositionLocalOf<ColorScheme> {
    error("No colorscheme found")
}
private val LocalGravidColorSystem = staticCompositionLocalOf<GravidColorSystem> {
    error("No colorscheme found")
}

@Composable
fun GravidTheme(
    overrideColorScheme: ColorScheme? = null,
    darkTheme: Boolean = isSystemInDarkTheme(),
    content: @Composable () -> Unit,
) {
    val colorScheme = when {
        overrideColorScheme != null -> overrideColorScheme
        darkTheme -> DarkColorScheme
        else -> LightColorScheme
    }

    val gravidColors = GravidColorSystem()
    val dimens = GravidDimens()
    val typography = GravidTypography(
        colors = colorScheme,
        dimens = dimens,
        fontFamily = ptSansFamily,
    )

    MaterialTheme(colorScheme = debugColors()) {
        GravidThemePropertiesProvider(
            dimens = dimens,
            typography = typography,
            colorScheme = colorScheme,
            gravidColorSystem = gravidColors,
        ) {
            CompositionLocalProvider(content = content)
        }
    }
}

object GravidTheme {
    val typography: GravidTypography
        @Composable
        get() = LocalGravidTypography.current
    val dimens: GravidDimens
        @Composable
        get() = LocalGravidDimens.current
    val colorScheme: ColorScheme
        @Composable
        get() = LocalGravidColors.current
    val gravidColorSystem: GravidColorSystem
        @Composable
        get() = LocalGravidColorSystem.current
}

/**
 * A Material [Colors] implementation which sets all colors to [debugColor] to discourage usage of
 * [MaterialTheme.colorScheme] in preference to [GravidTheme.colorScheme].
 */
fun debugColors(
    debugColor: Color = Color.Red,
) = ColorScheme(
    primary = debugColor,
    secondary = debugColor,
    background = debugColor,
    surface = debugColor,
    error = debugColor,
    onPrimary = debugColor,
    onSecondary = debugColor,
    onBackground = debugColor,
    onSurface = debugColor,
    onError = debugColor,
    errorContainer = debugColor,
    inverseOnSurface = debugColor,
    inversePrimary = debugColor,
    inverseSurface = debugColor,
    onErrorContainer = debugColor,
    onPrimaryContainer = debugColor,
    onSecondaryContainer = debugColor,
    onSurfaceVariant = debugColor,
    onTertiary = debugColor,
    onTertiaryContainer = debugColor,
    outline = debugColor,
    primaryContainer = debugColor,
    secondaryContainer = debugColor,
    surfaceVariant = debugColor,
    tertiary = debugColor,
    tertiaryContainer = debugColor,
    surfaceTint = debugColor,
    outlineVariant = debugColor,
    scrim = debugColor,
)
