/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2023 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsp.mingraviditet.gravidcompose.utils

import androidx.compose.material3.TopAppBarState
import androidx.compose.runtime.Composable
import androidx.compose.runtime.DisposableEffect
import androidx.compose.runtime.saveable.Saver
import androidx.compose.runtime.saveable.listSaver
import androidx.compose.runtime.saveable.rememberSaveable

private val SaveMap = mutableMapOf<String, TopAppBarKeyParams>()

private data class TopAppBarKeyParams(
    val params: String = "",
    val initialHeightOffsetLimit: Float = -Float.MAX_VALUE,
    val initialHeightOffset: Float = 0f,
    val initialContentOffset: Float = 0f,
)

val TopAppBarSaver: Saver<TopAppBarState, *> = listSaver(
    save = { listOf(it.heightOffsetLimit, it.heightOffset, it.contentOffset) },
    restore = {
        TopAppBarState(
            initialHeightOffsetLimit = it[0],
            initialHeightOffset = it[1],
            initialContentOffset = it[2],
        )
    },
)

@Composable
fun rememberForeverTopAppBarState(
    key: String,
    params: String = "",
    initialHeightOffsetLimit: Float = -Float.MAX_VALUE,
    initialHeightOffset: Float = 0f,
    initialContentOffset: Float = 0f,
): TopAppBarState {
    val state = rememberSaveable(saver = TopAppBarSaver) {
        var savedValue = SaveMap[key]
        if (savedValue?.params != params) savedValue = null
        val savedInitialHeightOffsetLimit = savedValue?.initialHeightOffsetLimit ?: initialHeightOffsetLimit
        val savedInitialHeightOffset = savedValue?.initialHeightOffset ?: initialHeightOffset
        val savedInitialContentOffset = savedValue?.initialContentOffset ?: initialContentOffset
        TopAppBarState(savedInitialHeightOffsetLimit, savedInitialHeightOffset, savedInitialContentOffset)
    }

    DisposableEffect(Unit) {
        onDispose {
            val lastHeightOffsetLimit = state.heightOffsetLimit
            val lastHeightOffset = state.heightOffset
            val lastContentOffset = state.contentOffset
            SaveMap[key] = TopAppBarKeyParams(params, lastHeightOffsetLimit, lastHeightOffset, lastContentOffset)
        }
    }
    return state
}
